@@ -257,7 +257,10 @@ class Campaign extends Base
 
         $embed = ($parsedParams->getString('embed') !== null) ? explode(',', $parsedParams->getString('embed')) : [];
 
-        $campaigns = $this->campaignFactory->query($this->gridRenderSort($parsedParams), $this->gridRenderFilter($filter, $parsedParams));
+        $campaigns = $this->campaignFactory->query(
+            $this->gridRenderSort($parsedParams),
+            $this->gridRenderFilter($filter, $parsedParams)
+        );
 
         foreach ($campaigns as $campaign) {
             /* @var \Xibo\Entity\Campaign $campaign */
@@ -282,13 +285,17 @@ class Campaign extends Base
             $campaign->includeProperty('buttons');
             $campaign->buttons = [];
 
-            // Schedule Now
-            if ($this->getUser()->featureEnabled('schedule.now') && $campaign->type === 'list') {
-                $campaign->buttons[] = array(
-                    'id' => 'campaign_button_schedulenow',
-                    'url' => $this->urlFor($request,'schedule.now.form', ['id' => $campaign->campaignId, 'from' => 'Campaign']),
-                    'text' => __('Schedule Now')
-                );
+            // Schedule
+            if ($this->getUser()->featureEnabled('schedule.add') && $campaign->type === 'list') {
+                $campaign->buttons[] = [
+                    'id' => 'campaign_button_schedule',
+                    'url' => $this->urlFor(
+                        $request,
+                        'schedule.add.form',
+                        ['id' => $campaign->campaignId, 'from' => 'Campaign']
+                    ),
+                    'text' => __('Schedule')
+                ];
             }
 
             // Preview
@@ -333,11 +340,22 @@ class Campaign extends Base
                     // Select Folder
                     $campaign->buttons[] = [
                         'id' => 'campaign_button_selectfolder',
-                        'url' => $this->urlFor($request,'campaign.selectfolder.form', ['id' => $campaign->campaignId]),
+                        'url' => $this->urlFor(
+                            $request,
+                            'campaign.selectfolder.form',
+                            ['id' => $campaign->campaignId]
+                        ),
                         'text' => __('Select Folder'),
                         'multi-select' => true,
                         'dataAttributes' => [
-                            ['name' => 'commit-url', 'value' => $this->urlFor($request,'campaign.selectfolder', ['id' => $campaign->campaignId])],
+                            [
+                                'name' => 'commit-url',
+                                'value' => $this->urlFor(
+                                    $request,
+                                    'campaign.selectfolder',
+                                    ['id' => $campaign->campaignId]
+                                )
+                            ],
                             ['name' => 'commit-method', 'value' => 'put'],
                             ['name' => 'id', 'value' => 'campaign_button_selectfolder'],
                             ['name' => 'text', 'value' => __('Move to Folder')],
@@ -350,7 +368,11 @@ class Campaign extends Base
                 // Copy the campaign
                 $campaign->buttons[] = [
                     'id' => 'campaign_button_copy',
-                    'url' => $this->urlFor($request,'campaign.copy.form', ['id' => $campaign->campaignId]),
+                    'url' => $this->urlFor(
+                        $request,
+                        'campaign.copy.form',
+                        ['id' => $campaign->campaignId]
+                    ),
                     'text' => __('Copy')
                 ];
             } else {
@@ -363,11 +385,22 @@ class Campaign extends Base
                 // Delete Campaign
                 $campaign->buttons[] = [
                     'id' => 'campaign_button_delete',
-                    'url' => $this->urlFor($request,'campaign.delete.form', ['id' => $campaign->campaignId]),
+                    'url' => $this->urlFor(
+                        $request,
+                        'campaign.delete.form',
+                        ['id' => $campaign->campaignId]
+                    ),
                     'text' => __('Delete'),
                     'multi-select' => true,
                     'dataAttributes' => [
-                        ['name' => 'commit-url', 'value' => $this->urlFor($request,'campaign.delete', ['id' => $campaign->campaignId])],
+                        [
+                            'name' => 'commit-url',
+                            'value' => $this->urlFor(
+                                $request,
+                                'campaign.delete',
+                                ['id' => $campaign->campaignId]
+                            )
+                        ],
                         ['name' => 'commit-method', 'value' => 'delete'],
                         ['name' => 'id', 'value' => 'campaign_button_delete'],
                         ['name' => 'text', 'value' => __('Delete')],
@@ -426,7 +459,6 @@ class Campaign extends Base
         $this->getState()->template = 'campaign-form-add';
         $this->getState()->setData([
             'layouts' => $layouts,
-            'help' => $this->getHelp()->link('Campaign', 'Add')
         ]);
 
         return $this->render($request, $response);
@@ -473,7 +505,8 @@ class Campaign extends Base
      *  @SWG\Parameter(
      *      name="cyclePlaybackEnabled",
      *      in="formData",
-     *      description="When cycle based playback is enabled only 1 Layout from this Campaign will be played each time it is in a Schedule loop. The same Layout will be shown until the 'Play count' is achieved.",
+     *      description="When cycle based playback is enabled only 1 Layout from this Campaign will be played each time
+     * it is in a Schedule loop. The same Layout will be shown until the 'Play count' is achieved.",
      *      type="integer",
      *      required=false
      *   ),
@@ -554,18 +587,29 @@ class Campaign extends Base
             $type,
             $sanitizedParams->getString('name'),
             $this->getUser()->userId,
-            $sanitizedParams->getString('tags'),
             $folder->getId()
         );
         $campaign->permissionsFolderId = $folder->getPermissionFolderIdOrThis();
 
+        if ($this->getUser()->featureEnabled('tag.tagging')) {
+            if (is_array($sanitizedParams->getParam('tags'))) {
+                $tags = $this->tagFactory->tagsFromJson($sanitizedParams->getArray('tags'));
+            } else {
+                $tags = $this->tagFactory->tagsFromString($sanitizedParams->getString('tags'));
+            }
+
+            $campaign->updateTagLinks($tags);
+        }
+
         // Cycle based playback
         if ($campaign->type === 'list') {
             $campaign->cyclePlaybackEnabled = $sanitizedParams->getCheckbox('cyclePlaybackEnabled');
             $campaign->playCount = ($campaign->cyclePlaybackEnabled) ? $sanitizedParams->getInt('playCount') : null;
 
             // For compatibility with existing API implementations we set a default here.
-            $campaign->listPlayOrder = ($campaign->cyclePlaybackEnabled) ? 'block' : $sanitizedParams->getString('listPlayOrder', ['default' => 'round']);
+            $campaign->listPlayOrder = ($campaign->cyclePlaybackEnabled)
+                    ? 'block'
+                    : $sanitizedParams->getString('listPlayOrder', ['default' => 'round']);
         } else if ($campaign->type === 'ad') {
             $campaign->targetType = $sanitizedParams->getString('targetType');
             $campaign->target = $sanitizedParams->getInt('target');
@@ -636,9 +680,9 @@ class Campaign extends Base
             if (!$this->getUser()->checkViewable($this->layoutFactory->getById($layout->layoutId))) {
                 // Hide all layout details from the user
                 $layout->layout = __('Layout');
-                $layout->locked = true;
+                $layout->setUnmatchedProperty('locked', true);
             } else {
-                $layout->locked = false;
+                $layout->setUnmatchedProperty('locked', false);
             }
             $layouts[] = $layout;
         }
@@ -647,7 +691,6 @@ class Campaign extends Base
         $this->getState()->setData([
             'campaign' => $campaign,
             'layouts' => $layouts,
-            'help' => $this->getHelp()->link('Campaign', 'Edit')
         ]);
 
         return $this->render($request, $response);
@@ -839,8 +882,8 @@ class Campaign extends Base
         if ($campaign->type === 'ad') {
             // Ad campaign
             // -----------
-            $campaign->startDt = $parsedRequestParams->getDate('startDt')->format('U');
-            $campaign->endDt = $parsedRequestParams->getDate('endDt')->format('U');
+            $campaign->startDt = $parsedRequestParams->getDate('startDt')?->format('U');
+            $campaign->endDt = $parsedRequestParams->getDate('endDt')?->format('U');
             $campaign->targetType = $parsedRequestParams->getString('targetType');
             $campaign->target = $parsedRequestParams->getInt('target');
 
@@ -898,14 +941,18 @@ class Campaign extends Base
 
         // Tags
         // ----
-        $saveTags = false;
         if ($this->getUser()->featureEnabled('tag.tagging')) {
-            $campaign->replaceTags($this->tagFactory->tagsFromString($parsedRequestParams->getString('tags')));
-            $saveTags = true;
+            if (is_array($parsedRequestParams->getParam('tags'))) {
+                $tags = $this->tagFactory->tagsFromJson($parsedRequestParams->getArray('tags'));
+            } else {
+                $tags = $this->tagFactory->tagsFromString($parsedRequestParams->getString('tags'));
+            }
+
+            $campaign->updateTagLinks($tags);
         }
 
         // Save the campaign.
-        $campaign->save(['saveTags' => $saveTags]);
+        $campaign->save();
 
         // Return
         $this->getState()->hydrate([
@@ -932,13 +979,13 @@ class Campaign extends Base
     {
         $campaign = $this->campaignFactory->getById($id);
 
-        if (!$this->getUser()->checkDeleteable($campaign))
+        if (!$this->getUser()->checkDeleteable($campaign)) {
             throw new AccessDeniedException();
+        }
 
         $this->getState()->template = 'campaign-form-delete';
         $this->getState()->setData([
             'campaign' => $campaign,
-            'help' => $this->getHelp()->link('Campaign', 'Delete')
         ]);
 
         return $this->render($request, $response);
@@ -1251,7 +1298,6 @@ class Campaign extends Base
         $this->getState()->template = 'campaign-preview';
         $this->getState()->setData([
             'campaign' => $campaign,
-            'help' => $this->getHelp()->link('Campaign', 'Preview'),
             'layouts' => $layouts,
             'duration' => $duration,
             'extendedLayouts' => $extendedLayouts
@@ -1323,6 +1369,8 @@ class Campaign extends Base
             );
         }
 
+        $newCampaign->updateTagLinks($this->tagFactory->tagsFromString($campaign->getTagString()));
+
         // is the original campaign an ad campaign?
         if ($campaign->type === 'ad') {
             // assign the same displays to the new Campaign
@@ -1433,7 +1481,9 @@ class Campaign extends Base
 
         $campaign->folderId = $folderId;
         $folder = $this->folderFactory->getById($campaign->folderId);
-        $campaign->permissionsFolderId = ($folder->getPermissionFolderId() == null) ? $folder->id : $folder->getPermissionFolderId();
+        $campaign->permissionsFolderId = ($folder->getPermissionFolderId() == null)
+            ? $folder->id
+            : $folder->getPermissionFolderId();
 
         if ($campaign->isLayoutSpecific === 1) {
             $layouts = $this->layoutFactory->getByCampaignId($campaign->campaignId, true, true);
