@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (C) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -73,9 +73,13 @@ class Connector extends Base
             // Instantiate and decorate the entity
             try {
                 $connector->decorate($this->connectorFactory->create($connector));
+            } catch (NotFoundException) {
+                $this->getLog()->info('Connector installed which is not found in this CMS. ' . $connector->className);
+                $connector->setUnmatchedProperty('isHidden', 1);
             } catch (\Exception $e) {
                 $this->getLog()->error('Incorrectly configured connector '
                     . $connector->className . '. e=' . $e->getMessage());
+                $connector->setUnmatchedProperty('isHidden', 1);
             }
         }
 
