@@ -1,8 +1,8 @@
 <?php
-/**
- * Copyright (C) 2021 Xibo Signage Ltd
+/*
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -21,7 +21,6 @@
  */
 namespace Xibo\Controller;
 
-use Carbon\Carbon;
 use Slim\Http\Response as Response;
 use Slim\Http\ServerRequest as Request;
 use Xibo\Factory\DataSetColumnFactory;
@@ -89,9 +88,6 @@ class DataSet extends Base
     public function displayPage(Request $request, Response $response)
     {
         $this->getState()->template = 'dataset-page';
-        $this->getState()->setData([
-            'users' => $this->userFactory->query(),
-        ]);
 
         return $this->render($request, $response);
     }
@@ -177,6 +173,7 @@ class DataSet extends Base
             'code' => $sanitizedParams->getString('code'),
             'userId' => $sanitizedParams->getInt('userId'),
             'folderId' => $sanitizedParams->getInt('folderId'),
+            'logicalOperatorName' => $sanitizedParams->getString('logicalOperatorName'),
         ];
 
         $dataSets = $this->dataSetFactory->query($this->gridRenderSort($sanitizedParams), $this->gridRenderFilter($filter, $sanitizedParams));
@@ -259,9 +256,23 @@ class DataSet extends Base
                         'url' => $this->urlFor($request, 'dataSet.export.csv', ['id' => $dataSet->dataSetId]),
                         'text' => __('Export (CSV)')
                     ];
+
+                    if ($dataSet->isRemote === 1) {
+                        $dataSet->buttons[] = [
+                            'id' => 'dataset_button_clear_cache',
+                            'url' => $this->urlFor($request, 'dataSet.clear.cache.form', ['id' => $dataSet->dataSetId]),
+                            'text' => __('Clear Cache'),
+                            'dataAttributes' => [
+                                ['name' => 'auto-submit', 'value' => true],
+                                ['name' => 'commit-url', 'value' => $this->urlFor($request, 'dataSet.clear.cache', ['id' => $dataSet->dataSetId])],
+                                ['name' => 'commit-method', 'value' => 'POST']
+                            ]
+                        ];
+                    }
                 }
 
                 if ($user->checkDeleteable($dataSet) && $dataSet->isLookup == 0) {
+                    $dataSet->buttons[] = ['divider' => true];
                     // Delete DataSet
                     $dataSet->buttons[] = [
                         'id' => 'dataset_button_delete',
@@ -327,7 +338,6 @@ class DataSet extends Base
         $this->getState()->template = 'dataset-form-add';
         $this->getState()->setData([
             'dataSets' => $this->dataSetFactory->query(),
-            'help' => $this->getHelp()->link('DataSet', 'Add')
         ]);
 
         return $this->render($request, $response);
@@ -547,15 +557,21 @@ class DataSet extends Base
         $dataSet->code = $sanitizedParams->getString('code');
         $dataSet->isRemote = $sanitizedParams->getCheckbox('isRemote');
         $dataSet->userId = $this->getUser()->userId;
-        $dataSet->folderId = $sanitizedParams->getInt('folderId', ['default' => 1]);
 
-        if ($this->getUser()->featureEnabled('folder.view')) {
-            $folder = $this->folderFactory->getById($dataSet->folderId);
-            $dataSet->permissionsFolderId = ($folder->getPermissionFolderId() == null) ? $folder->id : $folder->getPermissionFolderId();
-        } else {
-            $dataSet->permissionsFolderId = 1;
+        // Folders
+        $folderId = $sanitizedParams->getInt('folderId');
+        if ($folderId === 1) {
+            $this->checkRootFolderAllowSave();
         }
 
+        if (empty($folderId) || !$this->getUser()->featureEnabled('folder.view')) {
+            $folderId = $this->getUser()->homeFolderId;
+        }
+
+        $folder = $this->folderFactory->getById($folderId, 0);
+        $dataSet->folderId = $folder->getId();
+        $dataSet->permissionsFolderId = $folder->getPermissionFolderIdOrThis();
+
         // Fields for remote
         if ($dataSet->isRemote === 1) {
             $dataSet->method = $sanitizedParams->getString('method');
@@ -631,7 +647,6 @@ class DataSet extends Base
         $this->getState()->setData([
             'dataSet' => $dataSet,
             'dataSets' => $this->dataSetFactory->query(),
-            'help' => $this->getHelp()->link('DataSet', 'Edit')
         ]);
 
         return $this->render($request, $response);
@@ -860,6 +875,9 @@ class DataSet extends Base
         $dataSet->folderId = $sanitizedParams->getInt('folderId', ['default' => $dataSet->folderId]);
 
         if ($dataSet->hasPropertyChanged('folderId')) {
+            if ($dataSet->folderId === 1) {
+                $this->checkRootFolderAllowSave();
+            }
             $folder = $this->folderFactory->getById($dataSet->folderId);
             $dataSet->permissionsFolderId = ($folder->getPermissionFolderId() == null) ? $folder->id : $folder->getPermissionFolderId();
         }
@@ -926,7 +944,6 @@ class DataSet extends Base
         $this->getState()->template = 'dataset-form-delete';
         $this->getState()->setData([
             'dataSet' => $dataSet,
-            'help' => $this->getHelp()->link('DataSet', 'Delete')
         ]);
 
         return $this->render($request, $response);
@@ -1011,7 +1028,6 @@ class DataSet extends Base
         $this->getState()->template = 'dataset-form-copy';
         $this->getState()->setData([
             'dataSet' => $dataSet,
-            'help' => $this->getHelp()->link('DataSet', 'Edit')
         ]);
 
         return $this->render($request, $response);
@@ -1510,4 +1526,48 @@ class DataSet extends Base
             $dataSet->dataSet.'.csv'
         )->withHeader('Content-Type', 'text/csv;charset=utf-8'));
     }
+
+    public function clearCacheForm(Request $request, Response $response, $id)
+    {
+        $dataSet = $this->dataSetFactory->getById($id);
+
+        $this->getState()->template = 'dataset-form-clear-cache';
+        $this->getState()->autoSubmit = $this->getAutoSubmit('dataSetClearCacheForm');
+        $this->getState()->setData([
+            'dataSet' => $dataSet
+        ]);
+
+        return $this->render($request, $response);
+    }
+
+    /**
+     * Clear cache for remote dataSet, only available via web interface
+     *
+     * @param Request $request
+     * @param Response $response
+     * @param $id
+     * @return \Psr\Http\Message\ResponseInterface|Response
+     * @throws AccessDeniedException
+     * @throws GeneralException
+     * @throws InvalidArgumentException
+     * @throws NotFoundException
+     */
+    public function clearCache(Request $request, Response $response, $id)
+    {
+        $dataSet = $this->dataSetFactory->getById($id);
+
+        if (!$this->getUser()->checkEditable($dataSet)) {
+            throw new AccessDeniedException();
+        }
+
+        $dataSet->clearCache();
+
+        // Return
+        $this->getState()->hydrate([
+            'message' => __('Cache cleared for %s', $dataSet->dataSet),
+            'id' => $dataSet->dataSetId
+        ]);
+
+        return $this->render($request, $response);
+    }
 }
