@@ -2,7 +2,7 @@
 /*
  * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -20,25 +20,22 @@
  * along with Xibo.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 namespace Xibo\Controller;
 
-use Carbon\Carbon;
 use Slim\Http\Response as Response;
 use Slim\Http\ServerRequest as Request;
-use Xibo\Entity\Widget;
 use Xibo\Factory\DisplayFactory;
 use Xibo\Factory\FolderFactory;
 use Xibo\Factory\LayoutFactory;
 use Xibo\Factory\MediaFactory;
 use Xibo\Factory\ModuleFactory;
 use Xibo\Factory\PlaylistFactory;
+use Xibo\Factory\RegionFactory;
 use Xibo\Factory\ScheduleFactory;
 use Xibo\Factory\TagFactory;
 use Xibo\Factory\UserFactory;
 use Xibo\Factory\UserGroupFactory;
 use Xibo\Factory\WidgetFactory;
-use Xibo\Helper\DateFormatHelper;
 use Xibo\Support\Exception\AccessDeniedException;
 use Xibo\Support\Exception\GeneralException;
 use Xibo\Support\Exception\InvalidArgumentException;
@@ -50,29 +47,19 @@ use Xibo\Support\Exception\NotFoundException;
  */
 class Playlist extends Base
 {
-    /**
-     * @var PlaylistFactory
-     */
+    /** @var PlaylistFactory */
     private $playlistFactory;
 
-    /**
-     * @var MediaFactory
-     */
+    /** @var MediaFactory */
     private $mediaFactory;
 
-    /**
-     * @var WidgetFactory
-     */
+    /** @var WidgetFactory */
     private $widgetFactory;
 
-    /**
-     * @var ModuleFactory
-     */
+    /** @var ModuleFactory */
     private $moduleFactory;
 
-    /**
-     * @var UserGroupFactory
-     */
+    /** @var UserGroupFactory */
     private $userGroupFactory;
 
     /** @var UserFactory */
@@ -93,6 +80,9 @@ class Playlist extends Base
     /** @var FolderFactory */
     private $folderFactory;
 
+    /** @var RegionFactory */
+    private $regionFactory;
+
     /**
      * Set common dependencies.
      * @param PlaylistFactory $playlistFactory
@@ -106,6 +96,7 @@ class Playlist extends Base
      * @param DisplayFactory $displayFactory
      * @param ScheduleFactory $scheduleFactory
      * @param FolderFactory $folderFactory
+     * @param RegionFactory $regionFactory
      */
     public function __construct(
         $playlistFactory,
@@ -118,7 +109,8 @@ class Playlist extends Base
         $layoutFactory,
         $displayFactory,
         $scheduleFactory,
-        $folderFactory
+        $folderFactory,
+        $regionFactory
     ) {
         $this->playlistFactory = $playlistFactory;
         $this->mediaFactory = $mediaFactory;
@@ -131,6 +123,7 @@ class Playlist extends Base
         $this->displayFactory = $displayFactory;
         $this->scheduleFactory = $scheduleFactory;
         $this->folderFactory = $folderFactory;
+        $this->regionFactory = $regionFactory;
     }
 
     /**
@@ -148,13 +141,7 @@ class Playlist extends Base
         // Call to render the template
         $this->getState()->template = 'playlist-page';
         $this->getState()->setData([
-            'users' => $this->userFactory->query(),
-            'groups' => $this->userGroupFactory->query(),
-                        'modules' => array_map(function($element) use ($moduleFactory) { 
-                    $module = $moduleFactory->createForInstall($element->class);
-                    $module->setModule($element);
-                    return $module;
-                }, $moduleFactory->getAssignableModules())
+            'modules' => $moduleFactory->getAssignableModules()
         ]);
 
         return $this->render($request, $response);
@@ -254,7 +241,9 @@ class Playlist extends Base
         $sanitizedParams = $this->getSanitizer($request->getParams());
 
         // Embed?
-        $embed = ($sanitizedParams->getString('embed') != null) ? explode(',', $sanitizedParams->getString('embed')) : [];
+        $embed = ($sanitizedParams->getString('embed') != null)
+            ? explode(',', $sanitizedParams->getString('embed'))
+            : [];
 
         // Playlists
         $playlists = $this->playlistFactory->query($this->gridRenderSort($sanitizedParams), $this->gridRenderFilter([
@@ -275,10 +264,8 @@ class Playlist extends Base
         ], $sanitizedParams));
 
         foreach ($playlists as $playlist) {
-
             // Handle embeds
             if (in_array('widgets', $embed)) {
-
                 $loadPermissions = in_array('permissions', $embed);
                 $loadTags = in_array('tags', $embed);
                 $loadActions = in_array('actions', $embed);
@@ -291,20 +278,36 @@ class Playlist extends Base
                 ]);
 
                 foreach ($playlist->widgets as $widget) {
+                    $widget->setUnmatchedProperty('tags', []);
 
-                    /* @var Widget $widget */
-                    $widget->module = $this->moduleFactory->createWithWidget($widget);
-
-                    // Embed the name of this widget
-                    $widget->name = $widget->module->getName();
-
-                    // Augment with tags?
-                    if ($loadTags) {
-                        $widget->tags = $widget->module->getMediaTags();
+                    try {
+                        $module = $this->moduleFactory->getByType($widget->type);
+                    } catch (NotFoundException $notFoundException) {
+                        $this->getLog()->error('Module not found for widget: ' . $widget->type);
+                        continue;
                     }
 
-                    // Add widget module type name
-                    $widget->moduleName = $widget->module->getModuleName();
+                    // Embed the name of this widget
+                    $widget->setUnmatchedProperty('moduleName', $module->name);
+                    $widgetName = $widget->getOptionValue('name', null);
+
+                    if ($module->regionSpecific == 0) {
+                        // Use the media assigned to this widget
+                        $media = $this->mediaFactory->getById($widget->getPrimaryMediaId());
+                        $media->load();
+                        $widget->setUnmatchedProperty('name', $widget->getOptionValue('name', null) ?: $media->name);
+
+                        // Augment with tags
+                        $widget->setUnmatchedProperty('tags', $media->tags);
+                    } else {
+                        $widget->setUnmatchedProperty('name', $widget->getOptionValue('name', null) ?: $module->name);
+                        $widget->setUnmatchedProperty('tags', []);
+                    }
+
+                    // Sub-playlists should calculate a fresh duration
+                    if ($widget->type === 'subplaylist') {
+                        $widget->calculateDuration($module);
+                    }
 
                     // Get transitions
                     $widget->transitionIn = $widget->getOptionValue('transIn', null);
@@ -315,34 +318,53 @@ class Playlist extends Base
                     // Permissions?
                     if ($loadPermissions) {
                         // Augment with editable flag
-                        $widget->isEditable = $this->getUser()->checkEditable($widget);
+                        $widget->setUnmatchedProperty('isEditable', $this->getUser()->checkEditable($widget));
 
                         // Augment with deletable flag
-                        $widget->isDeletable = $this->getUser()->checkDeleteable($widget);
+                        $widget->setUnmatchedProperty('isDeletable', $this->getUser()->checkDeleteable($widget));
 
                         // Augment with permissions flag
-                        $widget->isPermissionsModifiable = $this->getUser()->checkPermissionsModifyable($widget);
+                        $widget->setUnmatchedProperty(
+                            'isPermissionsModifiable',
+                            $this->getUser()->checkPermissionsModifyable($widget)
+                        );
                     }
                 }
             }
 
-            if ($this->isApi($request))
+
+            if ($sanitizedParams->getCheckbox('fullScreenScheduleCheck')) {
+                $fullScreenCampaignId = $this->hasFullScreenLayout($playlist);
+                $playlist->setUnmatchedProperty('hasFullScreenLayout', (!empty($fullScreenCampaignId)));
+                $playlist->setUnmatchedProperty('fullScreenCampaignId', $fullScreenCampaignId);
+            }
+
+            if ($this->isApi($request)) {
                 continue;
+            }
 
             $playlist->includeProperty('buttons');
 
             switch ($playlist->enableStat) {
-
                 case 'On':
-                    $playlist->enableStatDescription = __('This Playlist has enable stat collection set to ON');
+                    $playlist->setUnmatchedProperty(
+                        'enableStatDescription',
+                        __('This Playlist has enable stat collection set to ON')
+                    );
                     break;
 
                 case 'Off':
-                    $playlist->enableStatDescription = __('This Playlist has enable stat collection set to OFF');
+                    $playlist->setUnmatchedProperty(
+                        'enableStatDescription',
+                        __('This Playlist has enable stat collection set to OFF')
+                    );
                     break;
 
                 default:
-                    $playlist->enableStatDescription = __('This Playlist has enable stat collection set to INHERIT');
+                    $playlist->setUnmatchedProperty(
+                        'enableStatDescription',
+                        __('This Playlist has enable stat collection set to INHERIT')
+                    );
             }
 
             // Only proceed if we have edit permissions
@@ -379,11 +401,16 @@ class Playlist extends Base
                     // Select Folder
                     $playlist->buttons[] = [
                         'id' => 'playlist_button_selectfolder',
-                        'url' => $this->urlFor($request,'playlist.selectfolder.form', ['id' => $playlist->playlistId]),
+                        'url' => $this->urlFor($request, 'playlist.selectfolder.form', ['id' => $playlist->playlistId]),
                         'text' => __('Select Folder'),
                         'multi-select' => true,
                         'dataAttributes' => [
-                            ['name' => 'commit-url', 'value' => $this->urlFor($request,'playlist.selectfolder', ['id' => $playlist->playlistId])],
+                            [
+                                'name' => 'commit-url',
+                                'value' => $this->urlFor($request, 'playlist.selectfolder', [
+                                    'id' => $playlist->playlistId
+                                ])
+                            ],
                             ['name' => 'commit-method', 'value' => 'put'],
                             ['name' => 'id', 'value' => 'playlist_button_selectfolder'],
                             ['name' => 'text', 'value' => __('Move to Folder')],
@@ -400,7 +427,12 @@ class Playlist extends Base
                     'text' => __('Enable stats collection?'),
                     'multi-select' => true,
                     'dataAttributes' => [
-                        ['name' => 'commit-url', 'value' => $this->urlFor($request, 'playlist.setenablestat', ['id' => $playlist->playlistId])],
+                        [
+                            'name' => 'commit-url',
+                            'value' => $this->urlFor($request, 'playlist.setenablestat', [
+                                'id' => $playlist->playlistId
+                            ])
+                        ],
                         ['name' => 'commit-method', 'value' => 'put'],
                         ['name' => 'id', 'value' => 'playlist_button_setenablestat'],
                         ['name' => 'text', 'value' => __('Enable stats collection?')],
@@ -419,11 +451,16 @@ class Playlist extends Base
                 // Delete Button
                 $playlist->buttons[] = [
                     'id' => 'playlist_button_delete',
-                    'url' => $this->urlFor($request,'playlist.delete.form', ['id' => $playlist->playlistId]),
+                    'url' => $this->urlFor($request, 'playlist.delete.form', ['id' => $playlist->playlistId]),
                     'text' => __('Delete'),
                     'multi-select' => true,
                     'dataAttributes' => [
-                        ['name' => 'commit-url', 'value' => $this->urlFor($request,'playlist.delete', ['id' => $playlist->playlistId])],
+                        [
+                            'name' => 'commit-url',
+                            'value' => $this->urlFor($request, 'playlist.delete', [
+                                'id' => $playlist->playlistId
+                            ])
+                        ],
                         ['name' => 'commit-method', 'value' => 'delete'],
                         ['name' => 'id', 'value' => 'playlist_button_delete'],
                         ['name' => 'text', 'value' => __('Delete')],
@@ -442,18 +479,32 @@ class Playlist extends Base
                 // Permissions button
                 $playlist->buttons[] = [
                     'id' => 'playlist_button_permissions',
-                    'url' => $this->urlFor($request,'user.permissions.form', ['entity' => 'Playlist', 'id' => $playlist->playlistId]),
+                    'url' => $this->urlFor($request, 'user.permissions.form', [
+                        'entity' => 'Playlist',
+                        'id' => $playlist->playlistId
+                    ]),
                     'text' => __('Share'),
                     'multi-select' => true,
                     'dataAttributes' => [
-                        ['name' => 'commit-url', 'value' => $this->urlFor($request,'user.permissions.multi', ['entity' => 'Playlist', 'id' => $playlist->playlistId])],
+                        [
+                            'name' => 'commit-url',
+                            'value' => $this->urlFor($request, 'user.permissions.multi', [
+                                'entity' => 'Playlist',
+                                'id' => $playlist->playlistId
+                            ])
+                        ],
                         ['name' => 'commit-method', 'value' => 'post'],
                         ['name' => 'id', 'value' => 'playlist_button_permissions'],
                         ['name' => 'text', 'value' => __('Share')],
                         ['name' => 'rowtitle', 'value' => $playlist->name],
                         ['name' => 'sort-group', 'value' => 2],
                         ['name' => 'custom-handler', 'value' => 'XiboMultiSelectPermissionsFormOpen'],
-                        ['name' => 'custom-handler-url', 'value' => $this->urlFor($request,'user.permissions.multi.form', ['entity' => 'Playlist'])],
+                        [
+                            'name' => 'custom-handler-url',
+                            'value' => $this->urlFor($request, 'user.permissions.multi.form', [
+                                'entity' => 'Playlist'
+                            ])
+                        ],
                         ['name' => 'content-id-name', 'value' => 'playlistId']
                     ]
                 ];
@@ -468,6 +519,22 @@ class Playlist extends Base
                     'text' => __('Usage Report')
                 );
             }
+
+            // Schedule
+            if ($this->getUser()->featureEnabled('schedule.add')
+                && ($this->getUser()->checkEditable($playlist)
+                    || $this->getConfig()->getSetting('SCHEDULE_WITH_VIEW_PERMISSION') == 1)
+            ) {
+                $playlist->buttons[] = [
+                    'id' => 'playlist_button_schedule',
+                    'url' => $this->urlFor(
+                        $request,
+                        'schedule.add.form',
+                        ['id' => $playlist->playlistId, 'from' => 'Playlist']
+                    ),
+                    'text' => __('Schedule')
+                ];
+            }
         }
 
         $this->getState()->recordsTotal = $this->playlistFactory->countLast();
@@ -621,7 +688,13 @@ class Playlist extends Base
 
         // Tags
         if ($this->getUser()->featureEnabled('tag.tagging')) {
-            $playlist->replaceTags($this->tagFactory->tagsFromString($sanitizedParams->getString('tags')));
+            if (is_array($sanitizedParams->getParam('tags'))) {
+                $tags = $this->tagFactory->tagsFromJson($sanitizedParams->getArray('tags'));
+            } else {
+                $tags = $this->tagFactory->tagsFromString($sanitizedParams->getString('tags'));
+            }
+
+            $playlist->updateTagLinks($tags);
         }
 
         // Do we have a tag or name filter?
@@ -666,22 +739,23 @@ class Playlist extends Base
                 $widgets = [];
 
                 foreach ($media as $item) {
-                    // Create a module
-                    $module = $this->moduleFactory->create($item->mediaType);
+                    // Assign items from the library.
+                    // Get a module to use
+                    $module = $this->moduleFactory->getByType($item->mediaType);
 
-                    // Determine the duration
-                    $itemDuration = ($item->duration == 0) ? $module->determineDuration() : $item->duration;
+                    // The item duration shouldn't ever be 0 in the library, but in case it is we set to the default
+                    $itemDuration = ($item->duration == 0) ? $module->defaultDuration : $item->duration;
 
                     // Create a widget
-                    $widget = $this->widgetFactory->create($this->getUser()->userId, $playlist->playlistId, $item->mediaType, $itemDuration);
+                    $widget = $this->widgetFactory->create(
+                        $this->getUser()->userId,
+                        $playlist->playlistId,
+                        $item->mediaType,
+                        $itemDuration,
+                        $module->schemaVersion
+                    );
                     $widget->assignMedia($item->mediaId);
 
-                    // Assign the widget to the module
-                    $module->setWidget($widget);
-
-                    // Set default options (this sets options on the widget)
-                    $module->setDefaultWidgetOptions();
-
                     // Calculate the duration
                     $widget->calculateDuration($module);
 
@@ -691,7 +765,11 @@ class Playlist extends Base
                     // Add to a list of new widgets
                     $widgets[] = $widget;
                     if ($playlist->isDynamic && count($widgets) >= $playlist->maxNumberOfItems) {
-                        $this->getLog()->debug(sprintf('Dynamic Playlist ID %d, has reached the maximum number of items %d, finishing assignments', $playlist->playlistId, $playlist->maxNumberOfItems));
+                        $this->getLog()->debug(sprintf(
+                            'Dynamic Playlist ID %d, has reached the maximum number of items %d, finishing assignments',
+                            $playlist->playlistId,
+                            $playlist->maxNumberOfItems
+                        ));
                         break;
                     }
                 }
@@ -864,7 +942,13 @@ class Playlist extends Base
         }
 
         if ($this->getUser()->featureEnabled('tag.tagging')) {
-            $playlist->replaceTags($this->tagFactory->tagsFromString($sanitizedParams->getString('tags')));
+            if (is_array($sanitizedParams->getParam('tags'))) {
+                $tags = $this->tagFactory->tagsFromJson($sanitizedParams->getArray('tags'));
+            } else {
+                $tags = $this->tagFactory->tagsFromString($sanitizedParams->getString('tags'));
+            }
+
+            $playlist->updateTagLinks($tags);
         }
 
         // Do we have a tag or name filter?
@@ -1094,14 +1178,14 @@ class Playlist extends Base
             }
         }
 
-        // Handle tags
-        $playlist->replaceTags($this->tagFactory->tagsFromString($playlist->tags));
-
         // Set from global setting
         if ($playlist->enableStat == null) {
             $playlist->enableStat = $this->getConfig()->getSetting('PLAYLIST_STATS_ENABLED_DEFAULT');
         }
 
+        // tags
+        $playlist->updateTagLinks($originalPlaylist->tags);
+
         // Save the new playlist
         $playlist->save();
 
@@ -1118,6 +1202,7 @@ class Playlist extends Base
 
         return $this->render($request, $response);
     }
+
     //</editor-fold>
 
     /**
@@ -1141,95 +1226,14 @@ class Playlist extends Base
         }
 
         // Pass to view
-        $this->getState()->template = 'region-form-timeline';
+        $this->getState()->template = 'playlist-form-timeline';
         $this->getState()->setData([
             'playlist' => $playlist,
-            'help' => $this->getHelp()->link('Layout', 'RegionOptions')
         ]);
 
         return $this->render($request, $response);
     }
 
-    /**
-     * Widget Grid
-     *
-     * @SWG\Get(
-     *  path="/playlist/widget",
-     *  operationId="playlistWidgetSearch",
-     *  tags={"playlist"},
-     *  summary="Playlist Widget Search",
-     *  description="Search widgets on a Playlist",
-     *  @SWG\Parameter(
-     *      name="playlistId",
-     *      in="query",
-     *      description="The Playlist ID to Search",
-     *      type="integer",
-     *      required=false
-     *   ),
-     *  @SWG\Parameter(
-     *      name="widgetId",
-     *      in="query",
-     *      description="The Widget ID to Search",
-     *      type="integer",
-     *      required=false
-     *   ),
-     *  @SWG\Response(
-     *      response=200,
-     *      description="successful operation",
-     *      @SWG\Schema(
-     *          type="array",
-     *          @SWG\Items(ref="#/definitions/Widget")
-     *      )
-     *  )
-     * )
-     *
-     * This is not used by the WEB app - remains here for API usage only
-     * TODO: deprecate
-     * @param Request $request
-     * @param Response $response
-     * @return \Psr\Http\Message\ResponseInterface|Response
-     * @throws GeneralException
-     * @throws InvalidArgumentException
-     * @throws NotFoundException
-     * @throws \Xibo\Support\Exception\ControllerNotImplemented
-     */
-    public function widgetGrid(Request $request, Response $response)
-    {
-        $this->getState()->template = 'grid';
-        $sanitizedParams = $this->getSanitizer($request->getParams());
-
-        $widgets = $this->widgetFactory->query($this->gridRenderSort($sanitizedParams), $this->gridRenderFilter([
-            'playlistId' => $sanitizedParams->getInt('playlistId'),
-            'widgetId' => $sanitizedParams->getInt('widgetId')
-        ], $sanitizedParams));
-
-        foreach ($widgets as $widget) {
-            /* @var Widget $widget */
-            $widget->load();
-
-            $widget->module = $this->moduleFactory->createWithWidget($widget);
-
-            // Add property for name
-            $widget->name = $widget->module->getName();
-
-            // Add property for transition
-            $widget->transition = sprintf('%s / %s', $widget->module->getTransition('in'), $widget->module->getTransition('out'));
-
-            if ($this->isApi($request)) {
-                $widget->createdDt = Carbon::createFromTimestamp($widget->createdDt)->format(DateFormatHelper::getSystemFormat());
-                $widget->modifiedDt = Carbon::createFromTimestamp($widget->modifiedDt)->format(DateFormatHelper::getSystemFormat());
-                $widget->fromDt = Carbon::createFromTimestamp($widget->fromDt)->format(DateFormatHelper::getSystemFormat());
-                $widget->toDt = Carbon::createFromTimestamp($widget->toDt)->format(DateFormatHelper::getSystemFormat());
-            }
-        }
-
-        // Store the table rows
-        $this->getState()->recordsTotal = $this->widgetFactory->countLast();
-        $this->getState()->setData($widgets);
-
-        return $this->render($request, $response);
-    }
-
     /**
      * Add Library items to a Playlist
      * @param Request $request
@@ -1322,37 +1326,35 @@ class Playlist extends Base
 
         // Loop through all the media
         foreach ($media as $mediaId) {
-            /* @var int $mediaId */
             $item = $this->mediaFactory->getById($mediaId);
 
-            if (!$this->getUser()->checkViewable($item))
+            if (!$this->getUser()->checkViewable($item)) {
                 throw new AccessDeniedException(__('You do not have permissions to use this media'));
+            }
 
-            if ($item->mediaType == 'genericfile' || $item->mediaType == 'font')
-                throw new InvalidArgumentException(sprintf(__('You cannot assign file type %s to a playlist'), $item->mediaType), 'mediaType');
+            if ($item->mediaType == 'genericfile' || $item->mediaType == 'font') {
+                throw new InvalidArgumentException(sprintf(
+                    __('You cannot assign file type %s to a playlist'),
+                    $item->mediaType
+                ), 'mediaType');
+            }
 
             // Create a module
-            $module = $this->moduleFactory->create($item->mediaType);
+            $module = $this->moduleFactory->getByType($item->mediaType);
 
             // Determine the duration
-            // if we have a duration provided, then use it, otherwise use the duration recorded on the library item already
+            // if we have a duration provided, then use it, otherwise use the duration recorded on the
+            // library item already
             $itemDuration = ($duration !== null) ? $duration : $item->duration;
 
-            // If the library item duration (or provided duration) is 0, then call the module to determine what the
-            // duration should be.
-            // in most cases calling the module will return the Module Default Duration as configured in settings.
-            $itemDuration = ($itemDuration == 0) ? $module->determineDuration() : $itemDuration;
+            // If the library item duration (or provided duration) is 0, then default to the Module Default
+            // Duration as configured in settings.
+            $itemDuration = ($itemDuration == 0) ? $module->defaultDuration : $itemDuration;
 
             // Create a widget
-            $widget = $this->widgetFactory->create($this->getUser()->userId, $id, $item->mediaType, $itemDuration);
+            $widget = $this->widgetFactory->create($this->getUser()->userId, $id, $item->mediaType, $itemDuration, $module->schemaVersion);
             $widget->assignMedia($item->mediaId);
 
-            // Assign the widget to the module
-            $module->setWidget($widget);
-
-            // Set default options (this sets options on the widget)
-            $module->setDefaultWidgetOptions();
-
             // If a duration has been provided, then we want to use it, so set useDuration to 1.
             if ($duration !== null || $sanitizedParams->getCheckbox('useDuration') == 1) {
                 $widget->useDuration = 1;
@@ -1365,6 +1367,15 @@ class Playlist extends Base
             // Assign the widget to the playlist
             $playlist->assignWidget($widget, $displayOrder);
 
+            if ($playlist->isRegionPlaylist() && count($playlist->widgets) >= 2) {
+                // Convert this region to a `playlist` (if it is a zone)
+                $widgetRegion = $this->regionFactory->getById($playlist->regionId);
+                if ($widgetRegion->type === 'zone') {
+                    $widgetRegion->type = 'playlist';
+                    $widgetRegion->save();
+                }
+            }
+
             // If we have one provided we should bump the display order by 1 so that if we have more than one
             // media to assign, we don't put the second one in the same place as the first one.
             if ($displayOrder !== null) {
@@ -1379,7 +1390,7 @@ class Playlist extends Base
         $playlist->save(['saveTags' => false]);
 
         // Add new widgets to playlist for return values
-        $playlist->newWidgets = $newWidgets;
+        $playlist->setUnmatchedProperty('newWidgets', $newWidgets);
 
         // Success
         $this->getState()->hydrate([
@@ -1448,7 +1459,7 @@ class Playlist extends Base
      *  )
      * )
      */
-    function order(Request $request, Response $response, $id)
+    public function order(Request $request, Response $response, $id)
     {
         $playlist = $this->playlistFactory->getById($id);
 
@@ -1473,10 +1484,8 @@ class Playlist extends Base
 
         // Go through each one and move it
         foreach ($widgets as $widgetId => $position) {
-
             // Find this item in the existing list and add it to our new order
             foreach ($playlist->widgets as $widget) {
-                /* @var \Xibo\Entity\Widget $widget */
                 if ($widget->getId() == $widgetId) {
                     $this->getLog()->debug('Setting Display Order ' . $position . ' on widgetId ' . $widgetId);
                     $widget->displayOrder = $position;
@@ -1801,7 +1810,6 @@ class Playlist extends Base
 
         $data = [
             'playlist' => $playlist,
-            'help' => $this->getHelp()->link('Playlist', 'EnableStat')
         ];
 
         $this->getState()->template = 'playlist-form-setenablestat';
@@ -1865,9 +1873,8 @@ class Playlist extends Base
      *      required=false
      *   ),
      *  @SWG\Response(
-     *      response=200,
-     *      description="successful operation",
-     *      @SWG\Schema(ref="#/definitions/Campaign")
+     *      response=204,
+     *      description="successful operation"
      *  )
      * )
      *
@@ -1912,4 +1919,14 @@ class Playlist extends Base
 
         return $this->render($request, $response);
     }
+
+    /**
+     * Check if we already have a full screen Layout for this Playlist
+     * @param \Xibo\Entity\Playlist $playlist
+     * @return ?int
+     */
+    private function hasFullScreenLayout(\Xibo\Entity\Playlist $playlist): ?int
+    {
+        return $this->layoutFactory->getLinkedFullScreenLayout('playlist', $playlist->playlistId)?->campaignId;
+    }
 }
