@@ -1,8 +1,8 @@
 <?php
-/**
- * Copyright (C) 2021 Xibo Signage Ltd
+/*
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -68,9 +68,6 @@ class Settings extends Base
         $this->userGroupFactory = $userGroupFactory;
         $this->transitionfactory = $transitionfactory;
         $this->userFactory = $userFactory;
-
-        // Initialise extra validation rules
-        v::with('Xibo\\Validation\\Rules\\');
     }
 
     /**
@@ -204,6 +201,7 @@ class Settings extends Base
      * @throws InvalidArgumentException
      * @throws \Xibo\Support\Exception\ControllerNotImplemented
      * @throws \Xibo\Support\Exception\GeneralException
+     * @phpcs:disable Generic.Files.LineLength.TooLong
      */
     public function update(Request $request, Response $response)
     {
@@ -468,11 +466,6 @@ class Settings extends Base
             $this->getConfig()->changeSetting('SCHEDULE_LOOKAHEAD', $sanitizedParams->getCheckbox('SCHEDULE_LOOKAHEAD'));
         }
 
-        if ($this->getConfig()->isSettingEditable('EVENT_SYNC')) {
-            $this->handleChangedSettings('EVENT_SYNC', $this->getConfig()->getSetting('EVENT_SYNC'), $sanitizedParams->getCheckbox('EVENT_SYNC'), $changedSettings);
-            $this->getConfig()->changeSetting('EVENT_SYNC', $sanitizedParams->getCheckbox('EVENT_SYNC'));
-        }
-
         if ($this->getConfig()->isSettingEditable('REQUIRED_FILES_LOOKAHEAD')) {
             $this->handleChangedSettings('REQUIRED_FILES_LOOKAHEAD', $this->getConfig()->getSetting('REQUIRED_FILES_LOOKAHEAD'), $sanitizedParams->getInt('REQUIRED_FILES_LOOKAHEAD'), $changedSettings);
             $this->getConfig()->changeSetting('REQUIRED_FILES_LOOKAHEAD', $sanitizedParams->getInt('REQUIRED_FILES_LOOKAHEAD'));
@@ -646,11 +639,6 @@ class Settings extends Base
             $this->getConfig()->changeSetting('SCHEDULE_SHOW_LAYOUT_NAME', $sanitizedParams->getCheckbox('SCHEDULE_SHOW_LAYOUT_NAME'));
         }
 
-        if ($this->getConfig()->isSettingEditable('MODULE_CONFIG_LOCKED_CHECKB')) {
-            $this->handleChangedSettings('MODULE_CONFIG_LOCKED_CHECKB', $this->getConfig()->getSetting('MODULE_CONFIG_LOCKED_CHECKB'), $sanitizedParams->getCheckbox('MODULE_CONFIG_LOCKED_CHECKB'), $changedSettings);
-            $this->getConfig()->changeSetting('MODULE_CONFIG_LOCKED_CHECKB', $sanitizedParams->getCheckbox('MODULE_CONFIG_LOCKED_CHECKB'));
-        }
-
         if ($this->getConfig()->isSettingEditable('TASK_CONFIG_LOCKED_CHECKB')) {
             $this->handleChangedSettings('TASK_CONFIG_LOCKED_CHECKB', $this->getConfig()->getSetting('TASK_CONFIG_LOCKED_CHECKB'), $sanitizedParams->getCheckbox('TASK_CONFIG_LOCKED_CHECKB'), $changedSettings);
             $this->getConfig()->changeSetting('TASK_CONFIG_LOCKED_CHECKB', $sanitizedParams->getCheckbox('TASK_CONFIG_LOCKED_CHECKB'));
@@ -661,6 +649,11 @@ class Settings extends Base
             $this->getConfig()->changeSetting('TRANSITION_CONFIG_LOCKED_CHECKB', $sanitizedParams->getCheckbox('TRANSITION_CONFIG_LOCKED_CHECKB'));
         }
 
+        if ($this->getConfig()->isSettingEditable('FOLDERS_ALLOW_SAVE_IN_ROOT')) {
+            $this->handleChangedSettings('FOLDERS_ALLOW_SAVE_IN_ROOT', $this->getConfig()->getSetting('FOLDERS_ALLOW_SAVE_IN_ROOT'), $sanitizedParams->getCheckbox('FOLDERS_ALLOW_SAVE_IN_ROOT'), $changedSettings);
+            $this->getConfig()->changeSetting('FOLDERS_ALLOW_SAVE_IN_ROOT', $sanitizedParams->getCheckbox('FOLDERS_ALLOW_SAVE_IN_ROOT'));
+        }
+
         if ($this->getConfig()->isSettingEditable('DEFAULT_LANGUAGE')) {
             $this->handleChangedSettings('DEFAULT_LANGUAGE', $this->getConfig()->getSetting('DEFAULT_LANGUAGE'), $sanitizedParams->getString('DEFAULT_LANGUAGE'), $changedSettings);
             $this->getConfig()->changeSetting('DEFAULT_LANGUAGE', $sanitizedParams->getString('DEFAULT_LANGUAGE'));
@@ -783,23 +776,4 @@ class Settings extends Base
             }
         }
     }
-
-    /**
-     * Configure Adspace settings
-     * @param Request $request
-     * @param Response $response
-     * @return \Psr\Http\Message\ResponseInterface|Response
-     * @throws AccessDeniedException
-     */
-    public function configureAdspace(Request $request, Response $response)
-    {
-        if (!$this->getUser()->isSuperAdmin()) {
-            throw new AccessDeniedException();
-        }
-
-        $params = $this->getSanitizer($request->getParams());
-        $this->getConfig()->changeSetting('isAdspaceEnabled', $params->getCheckbox('isAdspaceEnabled'));
-
-        return $response->withStatus(204);
-    }
 }
