@@ -24,8 +24,6 @@ namespace Xibo\Controller;
 use RobThree\Auth\TwoFactorAuth;
 use Slim\Http\Response as Response;
 use Slim\Http\ServerRequest as Request;
-use Slim\Routing\RouteContext;
-use Xibo\Entity\Media;
 use Xibo\Entity\Permission;
 use Xibo\Event\LayoutOwnerChangeEvent;
 use Xibo\Event\ParsePermissionEntityEvent;
@@ -284,6 +282,7 @@ class User extends Base
             'useRegexForName' => $sanitizedParams->getCheckbox('useRegexForName'),
             'retired' => $sanitizedParams->getInt('retired'),
             'logicalOperatorName' => $sanitizedParams->getString('logicalOperatorName'),
+            'userGroupIdMembers' => $sanitizedParams->getInt('userGroupIdMembers'),
         ];
 
         // Load results into an array
@@ -292,24 +291,17 @@ class User extends Base
         foreach ($users as $user) {
             /* @var \Xibo\Entity\User $user */
 
-            $user->libraryQuotaFormatted = ByteFormatter::format($user->libraryQuota * 1024);
+            $user->setUnmatchedProperty('libraryQuotaFormatted', ByteFormatter::format($user->libraryQuota * 1024));
 
             $user->loggedIn = $this->sessionFactory->getActiveSessionsForUser($user->userId);
             $this->getLog()->debug('Logged in status for user ID ' . $user->userId . ' with name ' . $user->userName . ' is ' . $user->loggedIn);
 
             // Set some text for the display status
-            switch ($user->twoFactorTypeId) {
-                case 1:
-                    $user->twoFactorDescription = __('Email');
-                    break;
-
-                case 2:
-                    $user->twoFactorDescription = __('Google Authenticator');
-                    break;
-
-                default:
-                    $user->twoFactorDescription = __('Disabled');
-            }
+            $user->setUnmatchedProperty('twoFactorDescription', match ($user->twoFactorTypeId) {
+                1 => __('Email'),
+                2 => __('Google Authenticator'),
+                default => __('Disabled'),
+            });
 
             if ($this->isApi($request)) {
                 continue;
@@ -319,14 +311,17 @@ class User extends Base
 
             // Deal with the home page
             try {
-                $user->homePage = $this->userGroupFactory->getHomepageByName($user->homePageId)->title;
+                $user->setUnmatchedProperty(
+                    'homePage',
+                    $this->userGroupFactory->getHomepageByName($user->homePageId)->title
+                );
             } catch (NotFoundException $exception) {
                 $this->getLog()->error('User has homepage which does not exist. userId: ' . $user->userId . ', homepage: ' . $user->homePageId);
-                $user->homePage = __('Unknown homepage, please edit to update.');
+                $user->setUnmatchedProperty('homePage', __('Unknown homepage, please edit to update.'));
             }
 
             // Set the home folder
-            $user->homeFolder = $user->getUnmatchedProperty('homeFolder', '/');
+            $user->setUnmatchedProperty('homeFolder', $user->getUnmatchedProperty('homeFolder', '/'));
 
             // Super admins have some buttons
             if ($this->getUser()->featureEnabled('users.modify')
@@ -1065,7 +1060,7 @@ class User extends Base
                 }
             ]);
 
-            if ($userTypeId == 1 || $userTypeId == 4) {
+            if ($userTypeId == 1) {
                 $homepages = $this->userGroupFactory->getHomepages();
             } else {
                 $groupId = $params->getInt('groupId', [
@@ -1136,9 +1131,6 @@ class User extends Base
                 'defaultGroupId' => $this->getConfig()->getSetting('DEFAULT_USERGROUP'),
                 'defaultUserType' => $defaultUserTypeId
             ],
-            'help' => [
-                'add' => $this->getHelp()->link('User', 'Add')
-            ]
         ]);
 
         return $this->render($request, $response);
@@ -1178,9 +1170,6 @@ class User extends Base
                 'homepage' => $homepage,
                 'userTypes' => ($this->getUser()->isSuperAdmin()) ? $this->userTypeFactory->getAllRoles() : $this->userTypeFactory->getNonAdminRoles()
             ],
-            'help' => [
-                'edit' => $this->getHelp()->link('User', 'Edit')
-            ]
         ]);
 
         return $this->render($request, $response);
@@ -1209,9 +1198,6 @@ class User extends Base
         $this->getState()->setData([
             'user' => $user,
             'users' => $this->userFactory->query(null, ['notUserId' => $id]),
-            'help' => [
-                'delete' => $this->getHelp()->link('User', 'Delete')
-            ]
         ]);
 
         return $this->render($request, $response);
@@ -1232,9 +1218,6 @@ class User extends Base
         $this->getState()->template = 'user-form-edit-profile';
         $this->getState()->setData([
             'user' => $user,
-            'help' => [
-                'editProfile' => $this->getHelp()->link('User', 'EditProfile')
-            ],
             'data' => [
                 'setup' => $this->urlFor($request,'user.setup.profile'),
                 'generate' => $this->urlFor($request,'user.recovery.generate.profile'),
@@ -1540,6 +1523,14 @@ class User extends Base
         if ($newPassword != $retypeNewPassword)
             throw new InvalidArgumentException(__('Passwords do not match'), 'password');
 
+        // Make sure that the new password doesn't verify against the existing hash
+        try {
+            $user->checkPassword($newPassword);
+            throw new InvalidArgumentException(__('Please choose a new password'), 'password');
+        } catch (AccessDeniedException) {
+            // This is good, they don't match.
+        }
+
         $user->setNewPassword($newPassword);
         $user->save([
             'passwordUpdate' => true
@@ -1768,12 +1759,8 @@ class User extends Base
             'objectId' => $id,
             'permissions' => $currentPermissions,
             'canSetOwner' => $object->canChangeOwner(),
-            'owners' => $this->userFactory->query(),
             'object' => $object,
             'objectNameOverride' => $this->getSanitizer($request->getParams())->getString('nameOverride'),
-            'help' => [
-                'permissions' => $this->getHelp()->link('Campaign', 'Permissions')
-            ]
         ];
 
         $this->getState()->template = 'user-form-permissions';
@@ -1810,9 +1797,6 @@ class User extends Base
         $data = [
             'entity' => $entity,
             'objectIds' => $ids,
-            'help' => [
-                'permissions' => $this->getHelp()->link('Campaign', 'Permissions')
-            ]
         ];
 
         $this->getState()->template = 'user-form-multiple-permissions';
@@ -1903,14 +1887,18 @@ class User extends Base
 
         // Should we update the owner?
         if ($sanitizedParams->getInt('ownerId') != 0) {
-
             $ownerId = $sanitizedParams->getInt('ownerId');
 
             $this->getLog()->debug('Requesting update to a new Owner - id = ' . $ownerId);
 
             if ($object->canChangeOwner()) {
                 $object->setOwner($ownerId);
-                $object->save(['notify' => false, 'manageDynamicDisplayLinks' => false, 'validate' => false]);
+                $object->save([
+                    'notify' => false,
+                    'manageDynamicDisplayLinks' => false,
+                    'validate' => false,
+                    'recalculateHash' => false
+                ]);
             } else {
                 throw new ConfigurationException(__('Cannot change owner on this Object'));
             }
@@ -1927,13 +1915,6 @@ class User extends Base
         if ($object->permissionsClass() === 'Xibo\Entity\Folder') {
             /** @var $object \Xibo\Entity\Folder */
             $object->managePermissions();
-        } else if ($object->permissionsClass() == 'Xibo\Entity\Media') {
-            // Are we a font?
-            /** @var $object Media */
-            if ($object->mediaType === 'font') {
-                // Drop permissions (we need to reassess).
-                $this->getMediaService($this->getUser())->installFonts(RouteContext::fromRequest($request)->getRouteParser(), ['invalidateCache' => true]);
-            }
         }
 
         // Return
@@ -2097,7 +2078,6 @@ class User extends Base
         $this->getState()->template = 'user-applications-form';
         $this->getState()->setData([
             'applications' => $this->applicationFactory->getAuthorisedByUserId($this->getUser()->userId),
-            'help' => $this->getHelp()->link('User', 'Applications')
         ]);
 
         return $this->render($request, $response);
@@ -2226,39 +2206,12 @@ class User extends Base
         // Groups we are assigned to
         $groupsAssigned = $this->userGroupFactory->getByUserId($user->userId);
 
-        // All Groups
-        $allGroups = $this->userGroupFactory->query();
-
-        // The available users are all users except users already in assigned users
-        $checkboxes = array();
-
-        foreach ($allGroups as $group) {
-            /* @var \Xibo\Entity\UserGroup $group */
-            // Check to see if it exists in $usersAssigned
-            $exists = false;
-            foreach ($groupsAssigned as $groupAssigned) {
-                /* @var \Xibo\Entity\UserGroup $groupAssigned */
-                if ($groupAssigned->groupId == $group->groupId) {
-                    $exists = true;
-                    break;
-                }
-            }
-
-            // Store this checkbox
-            $checkbox = array(
-                'id' => $group->groupId,
-                'name' => $group->group,
-                'value_checked' => (($exists) ? 'checked' : '')
-            );
-
-            $checkboxes[] = $checkbox;
-        }
-
         $this->getState()->template = 'user-form-membership';
         $this->getState()->setData([
             'user' => $user,
-            'checkboxes' => $checkboxes,
-            'help' =>  $this->getHelp()->link('User', 'Members')
+            'extra' => [
+                'userGroupsAssigned' => $groupsAssigned
+            ],
         ]);
 
         return $this->render($request, $response);
@@ -2436,15 +2389,6 @@ class User extends Base
         $this->getUser()->setOptionValue('isAlwaysUseManualAddUserForm', $parsedParams->getCheckbox('isAlwaysUseManualAddUserForm'));
         $this->getUser()->setOptionValue('rememberFolderTreeStateGlobally', $parsedParams->getCheckbox('rememberFolderTreeStateGlobally'));
 
-        if ($this->getUser()->isSuperAdmin()) {
-            $this->getUser()->showContentFrom = $parsedParams->getInt('showContentFrom');
-        }
-
-        // if we are not a super admin do not allow anything else than standard view.
-        if (!$this->getUser()->isSuperAdmin() && ($parsedParams->getInt('showContentFrom') == 2 || $parsedParams->getInt('showContentFrom') == 3)) {
-            throw new InvalidArgumentException(__('Option available only for Super Admins'), 'showContentFrom');
-        }
-
         // Clear auto submits?
         if ($parsedParams->getCheckbox('autoSubmitClearAll', ['checkboxReturnInteger' => false])) {
             $this->getUser()->removeOptionByPrefix('autoSubmit.');
