@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (C) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -28,7 +28,6 @@ use Respect\Validation\Validator as v;
 use Xibo\Factory\CampaignFactory;
 use Xibo\Factory\PermissionFactory;
 use Xibo\Factory\ScheduleFactory;
-use Xibo\Factory\TagFactory;
 use Xibo\Service\DisplayNotifyServiceInterface;
 use Xibo\Service\LogServiceInterface;
 use Xibo\Storage\StorageServiceInterface;
@@ -44,6 +43,9 @@ use Xibo\Support\Exception\NotFoundException;
 class Campaign implements \JsonSerializable
 {
     use EntityTrait;
+    use TagLinkTrait;
+
+    public static $availableTypes = ['ad', 'list', 'media', 'playlist'];
 
     /**
      * @SWG\Property(description="The Campaign Id")
@@ -58,7 +60,7 @@ class Campaign implements \JsonSerializable
     public $ownerId;
 
     /**
-     * @SWG\Property(description="The type of campaign, either list or ad")
+     * @SWG\Property(description="The type of campaign, either list, ad, playlist or media")
      * @var string
      */
     public $type;
@@ -87,6 +89,10 @@ class Campaign implements \JsonSerializable
      */
     public $totalDuration;
 
+    /**
+     * @SWG\Property(description="Tags associated with this Campaign, array of TagLink objects")
+     * @var TagLink[]
+     */
     public $tags = [];
 
     /**
@@ -219,7 +225,10 @@ class Campaign implements \JsonSerializable
     private $events = [];
 
     // Private
-    private $unassignTags = [];
+    /** @var TagLink[] */
+    private $unlinkTags = [];
+    /** @var TagLink[] */
+    private $linkTags = [];
 
     /** @var bool Have the Layout assignments been loaded? */
     private $layoutAssignmentsLoaded = false;
@@ -242,11 +251,6 @@ class Campaign implements \JsonSerializable
      */
     private $permissionFactory;
 
-    /**
-     * @var TagFactory
-     */
-    private $tagFactory;
-
     /**
      * @var ScheduleFactory
      */
@@ -263,7 +267,6 @@ class Campaign implements \JsonSerializable
      * @param PermissionFactory $permissionFactory
      * @param ScheduleFactory $scheduleFactory
      * @param DisplayNotifyServiceInterface $displayNotifyService
-     * @param TagFactory $tagFactory
      */
     public function __construct(
         $store,
@@ -272,20 +275,20 @@ class Campaign implements \JsonSerializable
         CampaignFactory $campaignFactory,
         $permissionFactory,
         $scheduleFactory,
-        $displayNotifyService,
-        $tagFactory
+        $displayNotifyService
     ) {
         $this->setCommonDependencies($store, $log, $dispatcher);
         $this->campaignFactory = $campaignFactory;
         $this->permissionFactory = $permissionFactory;
         $this->scheduleFactory = $scheduleFactory;
         $this->displayNotifyService = $displayNotifyService;
-        $this->tagFactory = $tagFactory;
     }
 
     public function __clone()
     {
         $this->campaignId = null;
+        $this->tags = [];
+        $this->linkTags = [];
     }
 
     /**
@@ -368,7 +371,11 @@ class Campaign implements \JsonSerializable
         }
         $startDt = $this->getStartDt();
         $endDt = $this->getEndDt();
-        $progress->daysTotal = $endDt->diffInDays($startDt);
+
+        // if start and end date are the same
+        // set the daysTotal to 1, to avoid potential division by 0 later on.
+        $progress->daysTotal = ($this->startDt === $this->endDt) ? 1 : $endDt->diffInDays($startDt);
+
         $progress->targetPerDay = $this->target / $progress->daysTotal;
 
         if ($startDt->isAfter($testDate)) {
@@ -407,7 +414,6 @@ class Campaign implements \JsonSerializable
     {
         $options = array_merge([
             'loadPermissions' => true,
-            'loadTags' => true,
             'loadEvents' => true,
             'loadDisplayGroupIds' => true,
         ], $options);
@@ -422,11 +428,6 @@ class Campaign implements \JsonSerializable
             $this->permissions = $this->permissionFactory->getByObjectId('Campaign', $this->campaignId);
         }
 
-        // Load all tags
-        if ($options['loadTags']) {
-            $this->tags = $this->tagFactory->loadByCampaignId($this->campaignId);
-        }
-
         // Events
         if ($options['loadEvents']) {
             $this->events = $this->scheduleFactory->getByCampaignId($this->campaignId);
@@ -471,7 +472,7 @@ class Campaign implements \JsonSerializable
      */
     public function validate()
     {
-        if ($this->type !== 'list' && $this->type !== 'ad') {
+        if (!in_array($this->type, self::$availableTypes)) {
             throw new InvalidArgumentException(__('Invalid type'), 'type');
         }
 
@@ -495,6 +496,13 @@ class Campaign implements \JsonSerializable
             if ($this->campaignId !== null && count($this->displayGroupIds) <= 0) {
                 throw new InvalidArgumentException(__('Please select one or more displays'), 'displayGroupId[]');
             }
+
+            if ($this->startDt !== null && $this->endDt !== null && $this->startDt > $this->endDt) {
+                throw new InvalidArgumentException(
+                    __('Cannot set end date to be earlier than the start date.'),
+                    'endDt'
+                );
+            }
         } else {
             if ($this->listPlayOrder !== 'round' && $this->listPlayOrder !== 'block') {
                 throw new InvalidArgumentException(
@@ -505,89 +513,6 @@ class Campaign implements \JsonSerializable
         }
     }
 
-
-    /**
-     * Does the campaign have the provided tag?
-     * @param $searchTag
-     * @return bool
-     * @throws NotFoundException
-     */
-    public function hasTag($searchTag)
-    {
-        $this->load();
-
-        foreach ($this->tags as $tag) {
-            /* @var Tag $tag */
-            if ($tag->tag == $searchTag)
-                return true;
-        }
-
-        return false;
-    }
-
-    /**
-     * Assign Tag
-     * @param Tag $tag
-     * @return $this
-     * @throws NotFoundException
-     */
-    public function assignTag($tag): Campaign
-    {
-        $this->load();
-        $this->handleTagAssign($tag);
-        $this->getLog()->debug(sprintf('Tags after assignment %s', json_encode($this->tags)));
-
-        return $this;
-    }
-
-    /**
-     * Unassign tag
-     * @param Tag $tag
-     * @return $this
-     * @throws NotFoundException
-     */
-    public function unassignTag($tag)
-    {
-        $this->load();
-
-        foreach ($this->tags as $key => $currentTag) {
-            if ($currentTag->tagId === $tag->tagId && $currentTag->value === $tag->value) {
-                $this->unassignTags[] = $tag;
-                array_splice($this->tags, $key, 1);
-            }
-        }
-
-        $this->getLog()->debug(sprintf('Tags after removal %s', json_encode($this->tags)));
-
-        return $this;
-    }
-
-    /**
-     * @param array[Tag] $tags
-     */
-    public function replaceTags($tags = [])
-    {
-        if (!is_array($this->tags) || count($this->tags) <= 0)
-            $this->tags = $this->tagFactory->loadByCampaignId($this->campaignId);
-
-        if ($this->tags != $tags) {
-            $this->unassignTags = array_udiff($this->tags, $tags, function ($a, $b) {
-                /* @var Tag $a */
-                /* @var Tag $b */
-                return $a->tagId - $b->tagId;
-            });
-
-            $this->getLog()->debug(sprintf('Tags to be removed: %s', json_encode($this->unassignTags)));
-
-            // Replace the arrays
-            $this->tags = $tags;
-
-            $this->getLog()->debug(sprintf('Tags remaining: %s', json_encode($this->tags)));
-        } else {
-            $this->getLog()->debug('Tags were not changed');
-        }
-    }
-
     /**
      * Save this Campaign
      * @param array $options
@@ -618,25 +543,16 @@ class Campaign implements \JsonSerializable
 
         if ($options['saveTags']) {
             // Remove unwanted ones
-            if (is_array($this->unassignTags)) {
-                foreach ($this->unassignTags as $tag) {
-                    /* @var Tag $tag */
-                    $this->getLog()->debug('Unassigning tag ' . $tag->tag);
-
-                    $tag->unassignCampaign($this->campaignId);
-                    $tag->save();
+            if (is_array($this->unlinkTags)) {
+                foreach ($this->unlinkTags as $tag) {
+                    $this->unlinkTagFromEntity('lktagcampaign', 'campaignId', $this->campaignId, $tag->tagId);
                 }
             }
 
             // Save the tags
-            if (is_array($this->tags)) {
-                foreach ($this->tags as $tag) {
-                    /* @var Tag $tag */
-
-                    $this->getLog()->debug('Assigning tag ' . $tag->tag);
-
-                    $tag->assignCampaign($this->campaignId);
-                    $tag->save();
+            if (is_array($this->linkTags)) {
+                foreach ($this->linkTags as $tag) {
+                    $this->linkTagToEntity('lktagcampaign', 'campaignId', $this->campaignId, $tag->tagId, $tag->value);
                 }
             }
         }
@@ -674,11 +590,7 @@ class Campaign implements \JsonSerializable
         }
 
         // Unassign all Tags
-        foreach ($this->tags as $tag) {
-            /* @var Tag $tag */
-            $tag->unassignCampaign($this->campaignId);
-            $tag->save();
-        }
+        $this->unlinkAllTagsFromEntity('lktagcampaign', 'campaignId', $this->campaignId);
 
         // Notify anyone interested of the changes
         // we do this before we delete from the DB (otherwise notify won't find anything)
