@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -471,7 +471,6 @@ class DataSet implements \JsonSerializable
      */
     public function getData($filterBy = [], $options = [])
     {
-
         $sanitizer = $this->getSanitizer($filterBy);
 
         $start = $sanitizer->getInt('start', ['default' => 0]);
@@ -482,7 +481,8 @@ class DataSet implements \JsonSerializable
 
         $options = array_merge([
             'includeFormulaColumns' => true,
-            'requireTotal' => true
+            'requireTotal' => true,
+            'connection' => 'default'
         ], $options);
 
         // Params
@@ -491,11 +491,11 @@ class DataSet implements \JsonSerializable
         // Sanitize the filter options provided
         // Get the Latitude and Longitude ( might be used in a formula )
         if ($displayId == 0) {
-            $displayGeoLocation = "GEOMFROMTEXT('POINT(" . $this->config->getSetting('DEFAULT_LAT') . " " . $this->config->getSetting('DEFAULT_LONG') . ")')";
-        }
-        else {
-            $displayGeoLocation = '(SELECT GeoLocation FROM `display` WHERE DisplayID = :displayId)';
-            $params['displayId'] = $displayId;
+            $displayGeoLocation =
+                "ST_GEOMFROMTEXT('POINT(" . $this->config->getSetting('DEFAULT_LAT') .
+                ' ' . $this->config->getSetting('DEFAULT_LONG') . ")')";
+        } else {
+            $displayGeoLocation = '(SELECT GeoLocation FROM `display` WHERE DisplayID =' . $displayId. ')';
         }
 
         // Build a SQL statement, based on the columns for this dataset
@@ -513,29 +513,28 @@ class DataSet implements \JsonSerializable
         // Select (columns)
         foreach ($this->getColumn() as $column) {
             /* @var DataSetColumn $column */
-            $allowedOrderCols[] = $column->heading;
-
-            if ($column->dataSetColumnTypeId == 2 && !$options['includeFormulaColumns'])
+            if ($column->dataSetColumnTypeId == 2 && !$options['includeFormulaColumns']) {
                 continue;
+            }
 
             // Formula column?
             if ($column->dataSetColumnTypeId == 2) {
-
                 // Is this a client side column?
                 if (substr($column->formula, 0, 1) === '$') {
                     $clientSideFormula[] = $column;
                     continue;
                 }
 
-                $formula = str_replace($this->blackList, '', htmlspecialchars_decode($column->formula, ENT_QUOTES));
+                $formula = str_ireplace($this->blackList, '', htmlspecialchars_decode($column->formula, ENT_QUOTES));
                 $formula = str_replace('[DisplayId]', $displayId, $formula);
 
                 $heading = str_replace('[DisplayGeoLocation]', $displayGeoLocation, $formula) . ' AS `' . $column->heading . '`';
-            }
-            else {
+            } else {
                 $heading = '`' . $column->heading . '`';
             }
 
+            $allowedOrderCols[] = $column->heading;
+
             $body .= ', ' . $heading;
         }
 
@@ -545,14 +544,13 @@ class DataSet implements \JsonSerializable
         if ($filter != '') {
             // Support display filtering.
             $filter = str_replace('[DisplayId]', $displayId, $filter);
-            $filter = str_replace($this->blackList, '', $filter);
+            $filter = str_ireplace($this->blackList, '', $filter);
 
             $body .= ' AND ' . $filter;
         }
 
         // Filter by ID
-        if (
-            $sanitizer->getInt('id') !== null) {
+        if ($sanitizer->getInt('id') !== null) {
             $body .= ' AND id = :id ';
             $params['id'] = $sanitizer->getInt('id');
         }
@@ -570,25 +568,42 @@ class DataSet implements \JsonSerializable
 
                 // Check allowable
                 if (!in_array($sanitized, $allowedOrderCols)) {
-                    $this->getLog()->info('Disallowed column: ' . $sanitized);
-                    continue;
+                    $found = false;
+                    $this->getLog()->info('Potentially disallowed column: ' . $sanitized);
+                    // the gridRenderSort will strip spaces on column names go through allowed order columns
+                    // and see if we can find a match by stripping spaces from the heading
+                    foreach ($allowedOrderCols as $allowedOrderCol) {
+                        $this->getLog()->info('Checking spaces in original name : ' . $sanitized);
+                        if (str_replace(' ', '', $allowedOrderCol) === $sanitized) {
+                            $found = true;
+                            // put the column heading with the space as sanitized to make sql happy.
+                            $sanitized = $allowedOrderCol;
+                        }
+                    }
+
+                    // we tried, but it was not found, omit this pair
+                    if (!$found) {
+                        continue;
+                    }
                 }
 
                 // Substitute
                 if (strripos($orderPair, ' DESC')) {
                     $order .= sprintf(' `%s`  DESC,', $sanitized);
-                }
-                else if (strripos($orderPair, ' ASC')) {
+                } else if (strripos($orderPair, ' ASC')) {
                     $order .= sprintf(' `%s`  ASC,', $sanitized);
-                }
-                else {
+                } else {
                     $order .= sprintf(' `%s`,', $sanitized);
                 }
             }
 
             $order = trim($order, ',');
-        }
-        else {
+
+            // if after all that we still do not have any column name to order by, default to order by id
+            if (trim($order) === 'ORDER BY') {
+                $order = ' ORDER BY id ';
+            }
+        } else {
             $order = ' ORDER BY id ';
         }
 
@@ -608,11 +623,15 @@ class DataSet implements \JsonSerializable
 
         $sql = $select . $body . $order . $limit;
 
-        $data = $this->getStore()->select($sql, $params);
+        $data = $this->getStore()->select($sql, $params, $options['connection']);
 
         // If there are limits run some SQL to work out the full payload of rows
         if ($options['requireTotal']) {
-            $results = $this->getStore()->select('SELECT COUNT(*) AS total FROM (' . $body, $params);
+            $results = $this->getStore()->select(
+                'SELECT COUNT(*) AS total FROM (' . $body,
+                $params,
+                $options['connection']
+            );
             $this->countLast = intval($results[0]['total']);
         }
 
@@ -677,7 +696,7 @@ class DataSet implements \JsonSerializable
      */
     public function hasData()
     {
-        return $this->getStore()->exists('SELECT id FROM `dataset_' . $this->dataSetId . '` LIMIT 1', []);
+        return $this->getStore()->exists('SELECT id FROM `dataset_' . $this->dataSetId . '` LIMIT 1', [], 'isolated');
     }
 
     /**
@@ -801,7 +820,12 @@ class DataSet implements \JsonSerializable
      */
     public function save($options = [])
     {
-        $options = array_merge(['validate' => true, 'saveColumns' => true], $options);
+        $options = array_merge([
+            'validate' => true,
+            'saveColumns' => true,
+            'activate' => true,
+            'notify' => true,
+        ], $options);
 
         if ($options['validate']) {
             $this->validate();
@@ -816,17 +840,20 @@ class DataSet implements \JsonSerializable
         // Columns
         if ($options['saveColumns']) {
             foreach ($this->columns as $column) {
-                /* @var \Xibo\Entity\DataSetColumn $column */
                 $column->dataSetId = $this->dataSetId;
-                $column->save();
+                $column->save($options);
             }
         }
 
         // We've been touched
-        $this->setActive();
+        if ($options['activate']) {
+            $this->setActive();
+        }
 
         // Notify Displays?
-        $this->notify();
+        if ($options['notify']) {
+            $this->notify();
+        }
     }
 
     /**
@@ -928,8 +955,7 @@ class DataSet implements \JsonSerializable
 
         // Delete Columns
         foreach ($this->columns as $column) {
-            /* @var \Xibo\Entity\DataSetColumn $column */
-            $column->delete();
+            $column->delete(true);
         }
 
         // Delete any dataSet rss
@@ -989,7 +1015,7 @@ class DataSet implements \JsonSerializable
             $params['userAgent'] = $this->userAgent;
             $params['refreshRate'] = $this->refreshRate;
             $params['clearRate'] = $this->clearRate;
-            $params['truncateOnEmpty'] = $this->truncateOnEmpty;
+            $params['truncateOnEmpty'] = $this->truncateOnEmpty ?? 0;
             $params['runsAfter'] = $this->runsAfter;
             $params['dataRoot'] = $this->dataRoot;
             $params['summarize'] = $this->summarize;
@@ -1041,7 +1067,7 @@ class DataSet implements \JsonSerializable
             $params['userAgent'] = $this->userAgent;
             $params['refreshRate'] = $this->refreshRate;
             $params['clearRate'] = $this->clearRate;
-            $params['truncateOnEmpty'] = $this->truncateOnEmpty;
+            $params['truncateOnEmpty'] = $this->truncateOnEmpty ?? 0;
             $params['runsAfter'] = $this->runsAfter;
             $params['dataRoot'] = $this->dataRoot;
             $params['summarize'] = $this->summarize;
@@ -1186,4 +1212,13 @@ class DataSet implements \JsonSerializable
     {
         $this->getStore()->insert('INSERT INTO `dataset_' . $dataSetIdTarget . '`  SELECT * FROM `dataset_' . $dataSetIdSource . '` ' ,[]);
     }
+
+    /**
+     * Clear DataSet cache
+     */
+    public function clearCache()
+    {
+        $this->getLog()->debug('Force sync detected, clear cache for remote dataSet ID ' . $this->dataSetId);
+        $this->pool->deleteItem('/dataset/cache/' . $this->dataSetId);
+    }
 }
