@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -29,7 +29,6 @@ use Respect\Validation\Validator as v;
 use Xibo\Factory\DisplayFactory;
 use Xibo\Factory\DisplayGroupFactory;
 use Xibo\Factory\PermissionFactory;
-use Xibo\Factory\TagFactory;
 use Xibo\Helper\DateFormatHelper;
 use Xibo\Service\LogServiceInterface;
 use Xibo\Storage\StorageServiceInterface;
@@ -47,6 +46,7 @@ use Xibo\Support\Exception\NotFoundException;
 class DisplayGroup implements \JsonSerializable
 {
     use EntityTrait;
+    use TagLinkTrait;
 
     /**
      * @SWG\Property(
@@ -96,6 +96,12 @@ class DisplayGroup implements \JsonSerializable
      */
     public $dynamicCriteria;
 
+    /**
+     * @SWG\Property(description="Which logical operator should be used when filtering by multiple dynamic criteria? OR|AND")
+     * @var string
+     */
+    public $dynamicCriteriaLogicalOperator;
+
     /**
      * @SWG\Property(
      *  description="Criteria for this dynamic group. A comma separated set of tags to apply",
@@ -114,7 +120,7 @@ class DisplayGroup implements \JsonSerializable
      * @SWG\Property(description="Which logical operator should be used when filtering by multiple Tags? OR|AND")
      * @var string
      */
-    public $dynamicCriteriaLogicalOperator;
+    public $dynamicCriteriaTagsLogicalOperator;
 
     /**
      * @SWG\Property(
@@ -125,8 +131,8 @@ class DisplayGroup implements \JsonSerializable
     public $userId = 0;
 
     /**
-     * @SWG\Property(description="Tags associated with this DisplayGroup")
-     * @var Tag[]
+     * @SWG\Property(description="Tags associated with this Display Group, array of TagLink objects")
+     * @var TagLink[]
      */
     public $tags = [];
 
@@ -166,6 +172,36 @@ class DisplayGroup implements \JsonSerializable
      */
     public $permissionsFolderId;
 
+    /**
+     * @SWG\Property(description="Optional Reference 1")
+     * @var string
+     */
+    public $ref1;
+
+    /**
+     * @SWG\Property(description="Optional Reference 2")
+     * @var string
+     */
+    public $ref2;
+
+    /**
+     * @SWG\Property(description="Optional Reference 3")
+     * @var string
+     */
+    public $ref3;
+
+    /**
+     * @SWG\Property(description="Optional Reference 4")
+     * @var string
+     */
+    public $ref4;
+
+    /**
+     * @SWG\Property(description="Optional Reference 5")
+     * @var string
+     */
+    public $ref5;
+
     // Child Items the Display Group is linked to
     public $displays = [];
     public $media = [];
@@ -173,7 +209,10 @@ class DisplayGroup implements \JsonSerializable
     public $events = [];
     private $displayGroups = [];
     private $permissions = [];
-    private $unassignTags = [];
+    /** @var TagLink[] */
+    private $unlinkTags = [];
+    /** @var TagLink[] */
+    private $linkTags = [];
     private $jsonInclude = ['displayGroupId', 'displayGroup'];
 
     // Track original assignments
@@ -211,11 +250,6 @@ class DisplayGroup implements \JsonSerializable
      */
     private $permissionFactory;
 
-    /**
-     * @var TagFactory
-     */
-    private $tagFactory;
-
     /**
      * Entity constructor.
      * @param StorageServiceInterface $store
@@ -223,15 +257,13 @@ class DisplayGroup implements \JsonSerializable
      * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $dispatcher
      * @param DisplayGroupFactory $displayGroupFactory
      * @param PermissionFactory $permissionFactory
-     * @param TagFactory $tagFactory
      */
-    public function __construct($store, $log, $dispatcher, $displayGroupFactory, $permissionFactory, $tagFactory)
+    public function __construct($store, $log, $dispatcher, $displayGroupFactory, $permissionFactory)
     {
         $this->setCommonDependencies($store, $log, $dispatcher);
 
         $this->displayGroupFactory = $displayGroupFactory;
         $this->permissionFactory = $permissionFactory;
-        $this->tagFactory = $tagFactory;
     }
 
     public function setDisplayFactory(DisplayFactory $displayFactory)
@@ -516,92 +548,6 @@ class DisplayGroup implements \JsonSerializable
             $this->notifyRequired = true;
     }
 
-    /**
-     * Does the campaign have the provided tag?
-     * @param $searchTag
-     * @return bool
-     * @throws NotFoundException
-     */
-    public function hasTag($searchTag)
-    {
-        $this->load();
-
-        foreach ($this->tags as $tag) {
-            /* @var Tag $tag */
-            if ($tag->tag == $searchTag)
-                return true;
-        }
-
-        return false;
-    }
-
-    /**
-     * Assign Tag
-     * @param Tag $tag
-     * @return $this
-     * @throws NotFoundException
-     */
-    public function assignTag($tag): DisplayGroup
-    {
-        $this->getLog()->debug('Assigning tag: ' . $tag->tag);
-
-        $this->load();
-        $this->handleTagAssign($tag);
-        $this->getLog()->debug(sprintf('Tags after assignment %s', json_encode($this->tags)));
-
-        return $this;
-    }
-
-    /**
-     * Unassign tag
-     * @param Tag $tag
-     * @return $this
-     * @throws NotFoundException
-     */
-    public function unassignTag($tag)
-    {
-        $this->getLog()->debug('Unassigning tag: ' . $tag->tag);
-
-        $this->load();
-
-        foreach ($this->tags as $key => $currentTag) {
-            if ($currentTag->tagId === $tag->tagId && $currentTag->value === $tag->value) {
-                $this->unassignTags[] = $tag;
-                array_splice($this->tags, $key, 1);
-            }
-        }
-
-        $this->getLog()->debug(sprintf('Tags after removal %s', json_encode($this->tags)));
-
-        return $this;
-    }
-
-    /**
-     * @param array[Tag] $tags
-     */
-    public function replaceTags($tags = [])
-    {
-        if (!is_array($this->tags) || count($this->tags) <= 0)
-            $this->tags = $this->tagFactory->loadByDisplayGroupId($this->displayGroupId);
-
-        if ($this->tags != $tags) {
-            $this->unassignTags = array_udiff($this->tags, $tags, function ($a, $b) {
-                /* @var Tag $a */
-                /* @var Tag $b */
-                return $a->tagId - $b->tagId;
-            });
-
-            $this->getLog()->debug(sprintf('Tags to be removed: %s', json_encode($this->unassignTags)));
-
-            // Replace the arrays
-            $this->tags = $tags;
-
-            $this->getLog()->debug(sprintf('Tags remaining: %s', json_encode($this->tags)));
-        } else {
-            $this->getLog()->debug('Tags were not changed');
-        }
-    }
-
     /**
      * Load the contents for this display group
      * @param array $options
@@ -621,11 +567,6 @@ class DisplayGroup implements \JsonSerializable
 
         $this->displayGroups = $this->displayGroupFactory->getByParentId($this->displayGroupId);
 
-        // Load all tags
-        if ($options['loadTags']) {
-            $this->tags = $this->tagFactory->loadByDisplayGroupId($this->displayGroupId);
-        }
-
         // Set the originals
         $this->originalDisplayGroups = $this->displayGroups;
 
@@ -640,11 +581,13 @@ class DisplayGroup implements \JsonSerializable
      */
     public function validate()
     {
-        if (!v::stringType()->notEmpty()->validate($this->displayGroup))
+        if (!v::stringType()->notEmpty()->validate($this->displayGroup)) {
             throw new InvalidArgumentException(__('Please enter a display group name'), 'displayGroup');
+        }
 
-        if (!empty($this->description) && !v::stringType()->length(null, 254)->validate($this->description))
+        if (!empty($this->description) && !v::stringType()->length(null, 254)->validate($this->description)) {
             throw new InvalidArgumentException(__('Description can not be longer than 254 characters'), 'description');
+        }
 
         if ($this->isDisplaySpecific == 0) {
             // Check the name
@@ -653,12 +596,13 @@ class DisplayGroup implements \JsonSerializable
                 'displayGroupId' => (($this->displayGroupId == null) ? 0 : $this->displayGroupId)
             ]);
 
-            if (count($result) > 0)
+            if (count($result) > 0) {
                 throw new DuplicateEntityException(sprintf(__('You already own a display group called "%s". Please choose another name.'), $this->displayGroup));
-
+            }
             // If we are dynamic, then make sure we have some criteria
-            if ($this->isDynamic == 1 && ($this->dynamicCriteria == '' && $this->dynamicCriteriaTags == ''))
+            if ($this->isDynamic == 1 && ($this->dynamicCriteria == '' && $this->dynamicCriteriaTags == '')) {
                 throw new InvalidArgumentException(__('Dynamic Display Groups must have at least one Criteria specified.'), 'dynamicCriteria');
+            }
         }
     }
 
@@ -676,44 +620,36 @@ class DisplayGroup implements \JsonSerializable
             'manageDisplayLinks' => true,
             'manageDynamicDisplayLinks' => true,
             'allowNotify' => true,
-            'saveTags' => true
+            'saveTags' => true,
+            'setModifiedDt' => true,
         ], $options);
 
         // Should we allow notification or not?
         $this->allowNotify = $options['allowNotify'];
 
-        if ($options['validate'])
+        if ($options['validate']) {
             $this->validate();
+        }
 
         if ($this->displayGroupId == null || $this->displayGroupId == 0) {
             $this->add();
             $this->loaded = true;
-        }
-        else if ($options['saveGroup']) {
-            $this->edit();
+        } else if ($options['saveGroup']) {
+            $this->edit($options);
         }
 
         if ($options['saveTags']) {
             // Remove unwanted ones
-            if (is_array($this->unassignTags)) {
-                foreach ($this->unassignTags as $tag) {
-                    /* @var Tag $tag */
-                    $this->getLog()->debug('Unassigning tag ' . $tag->tag);
-
-                    $tag->unassignDisplayGroup($this->displayGroupId);
-                    $tag->save();
+            if (is_array($this->unlinkTags)) {
+                foreach ($this->unlinkTags as $tag) {
+                    $this->unlinkTagFromEntity('lktagdisplaygroup', 'displayGroupId', $this->displayGroupId, $tag->tagId);
                 }
             }
 
-            // Save Tags
-            if (is_array($this->tags)) {
-                foreach ($this->tags as $tag) {
-                    /* @var Tag $tag */
-
-                    $this->getLog()->debug('Assigning tag ' . $tag->tag);
-
-                    $tag->assignDisplayGroup($this->displayGroupId);
-                    $tag->save();
+            // Save the tags
+            if (is_array($this->linkTags)) {
+                foreach ($this->linkTags as $tag) {
+                    $this->linkTagToEntity('lktagdisplaygroup', 'displayGroupId', $this->displayGroupId, $tag->tagId, $tag->value);
                 }
             }
         }
@@ -776,15 +712,16 @@ class DisplayGroup implements \JsonSerializable
             ]);
         }
 
-        foreach ($this->tags as $tag) {
-            /* @var Tag $tag */
-            $tag->unassignDisplayGroup($this->displayGroupId);
-            $tag->save();
-        }
+        $this->unlinkAllTagsFromEntity('lktagdisplaygroup', 'displayGroupId', $this->displayGroupId);
 
         // Delete assignments
         $this->removeAssignments();
 
+        // delete link to ad campaign.
+        $this->getStore()->update('DELETE FROM `lkcampaigndisplaygroup` WHERE displayGroupId = :displayGroupId', [
+            'displayGroupId' => $this->displayGroupId
+        ]);
+
         // Delete the Group itself
         $this->getStore()->update('DELETE FROM `displaygroup` WHERE DisplayGroupID = :displayGroupId', ['displayGroupId' => $this->displayGroupId]);
     }
@@ -816,22 +753,28 @@ class DisplayGroup implements \JsonSerializable
         $time = Carbon::now()->format(DateFormatHelper::getSystemFormat());
 
         $this->displayGroupId = $this->getStore()->insert('
-          INSERT INTO displaygroup (DisplayGroup, IsDisplaySpecific, Description, `isDynamic`, `dynamicCriteria`, `dynamicCriteriaTags`, `dynamicCriteriaExactTags`, `dynamicCriteriaLogicalOperator`, `userId`, `createdDt`, `modifiedDt`, `folderId`, `permissionsFolderId`)
-            VALUES (:displayGroup, :isDisplaySpecific, :description, :isDynamic, :dynamicCriteria, :dynamicCriteriaTags, :dynamicCriteriaExactTags, :dynamicCriteriaLogicalOperator, :userId, :createdDt, :modifiedDt, :folderId, :permissionsFolderId)
+          INSERT INTO displaygroup (DisplayGroup, IsDisplaySpecific, Description, `isDynamic`, `dynamicCriteria`, `dynamicCriteriaLogicalOperator`, `dynamicCriteriaTags`, `dynamicCriteriaExactTags`, `dynamicCriteriaTagsLogicalOperator`, `userId`, `createdDt`, `modifiedDt`, `folderId`, `permissionsFolderId`, `ref1`, `ref2`, `ref3`, `ref4`, `ref5`)
+            VALUES (:displayGroup, :isDisplaySpecific, :description, :isDynamic, :dynamicCriteria, :dynamicCriteriaLogicalOperator, :dynamicCriteriaTags, :dynamicCriteriaExactTags, :dynamicCriteriaTagsLogicalOperator, :userId, :createdDt, :modifiedDt, :folderId, :permissionsFolderId, :ref1, :ref2, :ref3, :ref4, :ref5)
         ', [
             'displayGroup' => $this->displayGroup,
             'isDisplaySpecific' => $this->isDisplaySpecific,
             'description' => $this->description,
             'isDynamic' => $this->isDynamic,
             'dynamicCriteria' => $this->dynamicCriteria,
+            'dynamicCriteriaLogicalOperator' => $this->dynamicCriteriaLogicalOperator ?? 'OR',
             'dynamicCriteriaTags' => $this->dynamicCriteriaTags,
             'dynamicCriteriaExactTags' => $this->dynamicCriteriaExactTags ?? 0,
-            'dynamicCriteriaLogicalOperator' => $this->dynamicCriteriaLogicalOperator ?? 'OR',
+            'dynamicCriteriaTagsLogicalOperator' => $this->dynamicCriteriaTagsLogicalOperator ?? 'OR',
             'userId' => $this->userId,
             'createdDt' => $time,
             'modifiedDt' => $time,
             'folderId' => ($this->folderId === null) ? 1 : $this->folderId,
-            'permissionsFolderId' => ($this->permissionsFolderId == null) ? 1 : $this-> permissionsFolderId
+            'permissionsFolderId' => ($this->permissionsFolderId == null) ? 1 : $this-> permissionsFolderId,
+            'ref1' => $this->ref1,
+            'ref2' => $this->ref2,
+            'ref3' => $this->ref3,
+            'ref4' => $this->ref4,
+            'ref5' => $this->ref5
         ]);
 
         // Insert my self link
@@ -841,7 +784,7 @@ class DisplayGroup implements \JsonSerializable
         ]);
     }
 
-    private function edit()
+    private function edit($options = [])
     {
         $this->getLog()->debug(sprintf('Updating Display Group. %s, %d', $this->displayGroup, $this->displayGroupId));
 
@@ -851,14 +794,20 @@ class DisplayGroup implements \JsonSerializable
               Description = :description,
               `isDynamic` = :isDynamic,
               `dynamicCriteria` = :dynamicCriteria,
+              `dynamicCriteriaLogicalOperator` = :dynamicCriteriaLogicalOperator,
               `dynamicCriteriaTags` = :dynamicCriteriaTags,
               `dynamicCriteriaExactTags` = :dynamicCriteriaExactTags,
-              `dynamicCriteriaLogicalOperator` = :dynamicCriteriaLogicalOperator,
+              `dynamicCriteriaTagsLogicalOperator` = :dynamicCriteriaTagsLogicalOperator,
               `bandwidthLimit` = :bandwidthLimit,
               `userId` = :userId,
               `modifiedDt` = :modifiedDt,
               `folderId` = :folderId,
-              `permissionsFolderId` = :permissionsFolderId
+              `permissionsFolderId` = :permissionsFolderId,
+              `ref1` = :ref1,
+              `ref2` = :ref2,
+              `ref3` = :ref3,
+              `ref4` = :ref4,
+              `ref5` = :ref5
            WHERE DisplayGroupID = :displayGroupId
           ', [
             'displayGroup' => $this->displayGroup,
@@ -866,14 +815,22 @@ class DisplayGroup implements \JsonSerializable
             'displayGroupId' => $this->displayGroupId,
             'isDynamic' => $this->isDynamic,
             'dynamicCriteria' => $this->dynamicCriteria,
+            'dynamicCriteriaLogicalOperator' => $this->dynamicCriteriaLogicalOperator ?? 'OR',
             'dynamicCriteriaTags' => $this->dynamicCriteriaTags,
             'dynamicCriteriaExactTags' => $this->dynamicCriteriaExactTags ?? 0,
-            'dynamicCriteriaLogicalOperator' => $this->dynamicCriteriaLogicalOperator ?? 'OR',
+            'dynamicCriteriaTagsLogicalOperator' => $this->dynamicCriteriaTagsLogicalOperator ?? 'OR',
             'bandwidthLimit' => $this->bandwidthLimit,
             'userId' => $this->userId,
-            'modifiedDt' => Carbon::now()->format(DateFormatHelper::getSystemFormat()),
+            'modifiedDt' => $options['setModifiedDt']
+                ? Carbon::now()->format(DateFormatHelper::getSystemFormat())
+                : $this->modifiedDt,
             'folderId' => $this->folderId,
-            'permissionsFolderId' => $this->permissionsFolderId
+            'permissionsFolderId' => $this->permissionsFolderId,
+            'ref1' => $this->ref1,
+            'ref2' => $this->ref2,
+            'ref3' => $this->ref3,
+            'ref4' => $this->ref4,
+            'ref5' => $this->ref5
         ]);
     }
 
@@ -897,9 +854,10 @@ class DisplayGroup implements \JsonSerializable
             // these displays must be permission checked based on the owner of the group NOT the logged in user
             $this->displays = $this->displayFactory->query(null, [
                 'display' => $this->dynamicCriteria,
+                'logicalOperatorName' => $this->dynamicCriteriaLogicalOperator,
                 'tags' => $this->dynamicCriteriaTags,
                 'exactTags' => $this->dynamicCriteriaExactTags,
-                'logicalOperator' => $this->dynamicCriteriaLogicalOperator,
+                'logicalOperator' => $this->dynamicCriteriaTagsLogicalOperator,
                 'userCheckUserId' => $this->getOwnerId(),
                 'useRegexForName' => true
             ]);
@@ -1154,4 +1112,4 @@ class DisplayGroup implements \JsonSerializable
 
         $this->getStore()->update($sql, $params);
     }
-}
\ No newline at end of file
+}
