@@ -22,10 +22,12 @@
 namespace Xibo\Entity;
 
 use Carbon\Carbon;
+use Xibo\Event\PlaylistDeleteEvent;
+use Xibo\Event\SubPlaylistDurationEvent;
+use Xibo\Event\SubPlaylistWidgetsEvent;
 use Xibo\Factory\ModuleFactory;
 use Xibo\Factory\PermissionFactory;
 use Xibo\Factory\PlaylistFactory;
-use Xibo\Factory\TagFactory;
 use Xibo\Factory\WidgetFactory;
 use Xibo\Helper\DateFormatHelper;
 use Xibo\Helper\Profiler;
@@ -36,7 +38,6 @@ use Xibo\Support\Exception\DuplicateEntityException;
 use Xibo\Support\Exception\GeneralException;
 use Xibo\Support\Exception\InvalidArgumentException;
 use Xibo\Support\Exception\NotFoundException;
-use Xibo\Widget\SubPlaylist;
 
 /**
  * Class Playlist
@@ -47,6 +48,7 @@ use Xibo\Widget\SubPlaylist;
 class Playlist implements \JsonSerializable
 {
     use EntityTrait;
+    use TagLinkTrait;
 
     /**
      * @SWG\Property(description="The ID of this Playlist")
@@ -155,8 +157,8 @@ class Playlist implements \JsonSerializable
     public $enableStat;
 
     /**
-     * @SWG\Property(description="An array of Tags")
-     * @var Tag[]
+     * @SWG\Property(description="Tags associated with this Playlist, array of TagLink objects")
+     * @var TagLink[]
      */
     public $tags = [];
 
@@ -194,7 +196,10 @@ class Playlist implements \JsonSerializable
      */
     public $permissionsFolderId;
 
-    private $unassignTags = [];
+    /** @var TagLink[] */
+    private $unlinkTags = [];
+    /** @var TagLink[] */
+    private $linkTags = [];
 
     //<editor-fold desc="Factories and Dependencies">
 
@@ -208,11 +213,6 @@ class Playlist implements \JsonSerializable
      */
     private $widgetFactory;
 
-    /**
-     * @var TagFactory
-     */
-    private $tagFactory;
-
     /**
      * @var PlaylistFactory
      */
@@ -236,10 +236,9 @@ class Playlist implements \JsonSerializable
      * @param PermissionFactory $permissionFactory
      * @param PlaylistFactory $playlistFactory
      * @param WidgetFactory $widgetFactory
-     * @param TagFactory $tagFactory
 
      */
-    public function __construct($store, $log, $dispatcher, $config, $permissionFactory, $playlistFactory, $widgetFactory, $tagFactory)
+    public function __construct($store, $log, $dispatcher, $config, $permissionFactory, $playlistFactory, $widgetFactory)
     {
         $this->setCommonDependencies($store, $log, $dispatcher);
 
@@ -247,7 +246,6 @@ class Playlist implements \JsonSerializable
         $this->permissionFactory = $permissionFactory;
         $this->playlistFactory = $playlistFactory;
         $this->widgetFactory = $widgetFactory;
-        $this->tagFactory = $tagFactory;
     }
 
     /**
@@ -269,6 +267,7 @@ class Playlist implements \JsonSerializable
         $this->playlistId = null;
         $this->regionId = null;
         $this->permissions = [];
+        $this->tags = [];
 
         $this->widgets = array_map(function ($object) { return clone $object; }, $this->widgets);
     }
@@ -465,7 +464,6 @@ class Playlist implements \JsonSerializable
             // Take the next available one
             $widget->displayOrder = count($this->widgets) + 1;
         }
-
         $this->widgets[] = $widget;
     }
 
@@ -498,69 +496,6 @@ class Playlist implements \JsonSerializable
         return $this;
     }
 
-    /**
-     * @param Tag[] $tags
-     */
-    public function replaceTags($tags = [])
-    {
-        if (!is_array($this->tags) || count($this->tags) <= 0)
-            $this->tags = $this->tagFactory->loadByPlaylistId($this->playlistId);
-
-        if ($this->tags != $tags) {
-            $this->unassignTags = array_udiff($this->tags, $tags, function ($a, $b) {
-                /* @var Tag $a */
-                /* @var Tag $b */
-                return $a->tagId - $b->tagId;
-            });
-
-            $this->getLog()->debug('Tags to be removed: %s', json_encode($this->unassignTags));
-
-            // Replace the arrays
-            $this->tags = $tags;
-
-            $this->getLog()->debug('Tags remaining: %s', json_encode($this->tags));
-        } else {
-            $this->getLog()->debug('Tags were not changed');
-        }
-    }
-
-    /**
-     * Assign Tag
-     * @param Tag $tag
-     * @return $this
-     * @throws NotFoundException
-     */
-    public function assignTag($tag)
-    {
-        $this->load();
-        $this->handleTagAssign($tag);
-        $this->getLog()->debug(sprintf('Tags after assignment %s', json_encode($this->tags)));
-
-        return $this;
-    }
-
-    /**
-     * Unassign tag
-     * @param Tag $tag
-     * @return $this
-     * @throws NotFoundException
-     */
-    public function unassignTag($tag)
-    {
-        $this->load();
-
-        foreach ($this->tags as $key => $currentTag) {
-            if ($currentTag->tagId === $tag->tagId && $currentTag->value === $tag->value) {
-                $this->unassignTags[] = $tag;
-                array_splice($this->tags, $key, 1);
-            }
-        }
-
-        $this->getLog()->debug('Tags after removal %s', json_encode($this->tags));
-
-        return $this;
-    }
-
     /**
      * Load
      * @param array $loadOptions
@@ -589,11 +524,6 @@ class Playlist implements \JsonSerializable
             $this->permissions = $this->permissionFactory->getByObjectId(get_class(), $this->playlistId);
         }
 
-        // Load all tags
-        if ($options['loadTags']) {
-            $this->tags = $this->tagFactory->loadByPlaylistId($this->playlistId);
-        }
-
         // Load the widgets
         if ($options['loadWidgets']) {
             foreach ($this->widgetFactory->getByPlaylistId($this->playlistId) as $widget) {
@@ -703,28 +633,17 @@ class Playlist implements \JsonSerializable
 
         // Save the tags?
         if ($options['saveTags']) {
-            $this->getLog()->debug('Saving tags on ' . $this);
-
             // Remove unwanted ones
-            if (is_array($this->unassignTags)) {
-                foreach ($this->unassignTags as $tag) {
-                    /* @var Tag $tag */
-                    $this->getLog()->debug('Unassigning tag ' . $tag->tag);
-
-                    $tag->unassignPlaylist($this->playlistId);
-                    $tag->save();
+            if (is_array($this->unlinkTags)) {
+                foreach ($this->unlinkTags as $tag) {
+                    $this->unlinkTagFromEntity('lktagplaylist', 'playlistId', $this->playlistId, $tag->tagId);
                 }
             }
 
             // Save the tags
-            if (is_array($this->tags)) {
-                foreach ($this->tags as $tag) {
-                    /* @var Tag $tag */
-
-                    $this->getLog()->debug('Assigning tag ' . $tag->tag);
-
-                    $tag->assignPlaylist($this->playlistId);
-                    $tag->save();
+            if (is_array($this->linkTags)) {
+                foreach ($this->linkTags as $tag) {
+                    $this->linkTagToEntity('lktagplaylist', 'playlistId', $this->playlistId, $tag->tagId, $tag->value);
                 }
             }
         }
@@ -767,6 +686,13 @@ class Playlist implements \JsonSerializable
                 'regionId'
             );
         }
+
+        // dispatch Playlist Delete Event, this will delete any full screen Layout linked to this Playlist
+        if (empty($this->regionId)) {
+            $event = new PlaylistDeleteEvent($this);
+            $this->getDispatcher()->dispatch($event, $event->getName());
+        }
+
         // Notify we're going to delete
         // we do this here, because once we've deleted we lose the references for the storage query
         $this->notifyLayouts();
@@ -780,14 +706,12 @@ class Playlist implements \JsonSerializable
             foreach ($parent->widgets as $widget) {
                 if ($widget->type === 'subplaylist') {
                     $isWidgetSaveRequired = false;
-                    /** @var $module SubPlaylist **/
-                    $module = $this->moduleFactory->createWithWidget($widget);
                     // we get an array with all subplaylists assigned to the parent
-                    $subPlaylistItems = $module->getAssignedPlaylists();
+                    $subPlaylistItems = json_decode($widget->getOptionValue('subPlaylists', '[]'), true);
                     $i = 0;
                     foreach ($subPlaylistItems as $subPlaylistItem) {
                         // find the matching playlistId to the playlistId we want to delete
-                        if ($subPlaylistItem->playlistId == $this->playlistId) {
+                        if ($subPlaylistItem['playlistId'] == $this->playlistId) {
                             // if there is only one playlistItem in subPlaylists option then remove the widget
                             if (count($subPlaylistItems) === 1) {
                                 $widget->delete(['notify' => false]);
@@ -808,20 +732,14 @@ class Playlist implements \JsonSerializable
                         // make sure spots and spotLength are saved as string if empty
                         $j = 1;
                         foreach ($subPlaylistItems as $subPlaylistItem) {
-                            $subPlaylistItem->rowNo = $j;
-                            $subPlaylistItem->spots = $subPlaylistItem->spots ?? '';
-                            $subPlaylistItem->spotLength = $subPlaylistItem->spotLength ?? '';
+                            $subPlaylistItem['rowNo'] = $j;
+                            $subPlaylistItem['spots'] = $subPlaylistItem['spots'] ?? '';
+                            $subPlaylistItem['spotLength'] = $subPlaylistItem['spotLength'] ?? '';
                             $j++;
                         }
 
                         // update subPlaylists Widget option
                         $widget->setOptionValue('subPlaylists', 'attrib', json_encode(array_values($subPlaylistItems)));
-
-                        // Tidy up any old options
-                        if ($module->getOption('subPlaylistIds') !== null) {
-                            $widget->setOptionValue('subPlaylistIds', 'attrib', null);
-                            $widget->setOptionValue('subPlaylistOptions', 'attrib', null);
-                        }
                         $widget->save();
                     }
                 }
@@ -835,11 +753,7 @@ class Playlist implements \JsonSerializable
         $this->getStore()->update('DELETE FROM `lkplaylistplaylist` WHERE childId = :playlistId', ['playlistId' => $this->playlistId]);
 
         // Unassign tags
-        foreach ($this->tags as $tag) {
-            /* @var Tag $tag */
-            $tag->unassignPlaylist($this->playlistId);
-            $tag->save();
-        }
+        $this->unlinkAllTagsFromEntity('lktagplaylist', 'playlistId', $this->playlistId);
 
         // Delete Permissions
         foreach ($this->permissions as $permission) {
@@ -1016,15 +930,14 @@ class Playlist implements \JsonSerializable
                     $this->getLog()->debug('expandWidgets: processing sub-playlist ' . $widget->widgetId
                         . ', parentWidgetId is ' . $parentWidgetId);
 
-                    /** @var SubPlaylist $module */
-                    $module = $this->moduleFactory->createWithWidget($widget);
-                    $module->isValid();
-
                     // Get the sub-playlist widgets
-                    $subPlaylistWidgets = $module->getSubPlaylistResolvedWidgets($widget->tempId);
+                    $event = new SubPlaylistWidgetsEvent($widget, $widget->tempId);
+                    $this->getDispatcher()->dispatch($event, SubPlaylistWidgetsEvent::$NAME);
+                    $subPlaylistWidgets = $event->getWidgets();
+                    $countSubPlaylistWidgets = count($subPlaylistWidgets);
 
                     // Are we the top level sub-playlist, and do we have cycle playback enabled?
-                    if ($parentWidgetId === 0 && $module->getOption('cyclePlaybackEnabled', 0) === 1) {
+                    if ($parentWidgetId === 0 && $widget->getOptionValue('cyclePlaybackEnabled', 0) === 1) {
                         $this->getLog()->debug('expandWidgets: cyclePlaybackEnabled on ' . $widget->widgetId);
 
                         // Work out the average duration
@@ -1041,15 +954,22 @@ class Playlist implements \JsonSerializable
                         // averageDuration = 50 / 5 = 10
                         // cycleDuration = 10 / 5 = 2
                         // When our 5 items are added up to make region duration, it will be 2+2+2+2+2=10
-                        $averageDuration = $totalDuration / count($subPlaylistWidgets);
-                        $cycleDuration = $averageDuration / count($subPlaylistWidgets);
+                        $averageDuration = $countSubPlaylistWidgets <= 0
+                            ? 0
+                            : $totalDuration / $countSubPlaylistWidgets;
+                        $cycleDuration = $countSubPlaylistWidgets <= 0
+                            ? 0
+                            : $averageDuration / $countSubPlaylistWidgets;
 
                         $this->getLog()->debug('expandWidgets: cycleDuration is ' . $cycleDuration
                             . ', averageDuration is ' . $averageDuration
                             . ', totalDuration is ' . $totalDuration);
 
                         foreach ($subPlaylistWidgets as $subPlaylistWidget) {
-                            $subPlaylistWidget->tempCyclePlaybackAverageDuration = $cycleDuration;
+                            $subPlaylistWidget->setUnmatchedProperty(
+                                'tempCyclePlaybackAverageDuration',
+                                $cycleDuration
+                            );
                             $widgets[] = $subPlaylistWidget;
                         }
                     } else {
@@ -1093,7 +1013,6 @@ class Playlist implements \JsonSerializable
         foreach ($this->widgets as $widget) {
             // Is this widget expired?
             if ($widget->isExpired()) {
-
                 // Remove this widget.
                 if ($widget->getOptionValue('deleteOnExpiry', 0) == 1) {
                     // Don't notify at all because we're going to do that when we finish updating our duration.
@@ -1122,9 +1041,9 @@ class Playlist implements \JsonSerializable
                 }
             } else {
                 // Add the sub playlist duration
-                /** @var SubPlaylist $module */
-                $module = $this->moduleFactory->createWithWidget($widget);
-                $duration += $module->getSubPlaylistResolvedDuration();
+                $event = new SubPlaylistDurationEvent($widget);
+                $this->getDispatcher()->dispatch($event, SubPlaylistDurationEvent::$NAME);
+                $duration += $event->getDuration();
             }
         }
 
@@ -1182,7 +1101,8 @@ class Playlist implements \JsonSerializable
      * Recursive function, that goes through all widgets on nested Playlists.
      *
      * generates nestedPlaylistDefinitions with Playlist ID as the key - later saved as nestedPlaylist.json on export
-     * generates playlistMappings which contains all relations between playlists (parent/child) - later saved as playlistMappings.json on export
+     * generates playlistMappings which contains all relations between playlists (parent/child) -
+     * later saved as playlistMappings.json on export
      * Adds dataSets data to $dataSets parameter - later saved as dataSet.json on export
      *
      * playlistMappings, nestedPLaylistDefinitions, dataSets and dataSetIds are passed by reference.
