@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -169,8 +169,16 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'downloadEndWindow', 'default' => '00:00', 'type' => 'string'],
                 ['name' => 'dayPartId', 'default' => null],
                 ['name' => 'xmrNetworkAddress', 'default' => '', 'type' => 'string'],
-                ['name' => 'statsEnabled', 'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0), 'type' => 'checkbox'],
-                ['name' => 'aggregationLevel', 'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'), 'type' => 'string'],
+                [
+                    'name' => 'statsEnabled',
+                    'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
+                    'type' => 'checkbox',
+                ],
+                [
+                    'name' => 'aggregationLevel',
+                    'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'),
+                    'type' => 'string',
+                ],
                 ['name' => 'powerpointEnabled', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'sizeX', 'default' => 0, 'type' => 'double'],
                 ['name' => 'sizeY', 'default' => 0, 'type' => 'double'],
@@ -191,7 +199,11 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'shellCommandAllowList', 'default' => '', 'type' => 'string'],
                 ['name' => 'sendCurrentLayoutAsStatusUpdate', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'screenShotRequestInterval', 'default' => 0, 'type' => 'int'],
-                ['name' => 'screenShotSize', 'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_SCREENSHOT_SIZE_DEFAULT', 200), 'type' => 'int'],
+                [
+                    'name' => 'screenShotSize',
+                    'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_SCREENSHOT_SIZE_DEFAULT', 200),
+                    'type' => 'int',
+                ],
                 ['name' => 'maxLogFileUploads', 'default' => 3, 'type' => 'int'],
                 ['name' => 'embeddedServerPort', 'default' => 9696, 'type' => 'int'],
                 ['name' => 'preventSleep', 'default' => 1, 'type' => 'checkbox'],
@@ -208,8 +220,16 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'downloadStartWindow', 'default' => '00:00'],
                 ['name' => 'downloadEndWindow', 'default' => '00:00'],
                 ['name' => 'xmrNetworkAddress', 'default' => ''],
-                ['name' => 'statsEnabled', 'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0), 'type' => 'checkbox'],
-                ['name' => 'aggregationLevel', 'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'), 'type' => 'string'],
+                [
+                    'name' => 'statsEnabled',
+                    'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
+                    'type' => 'checkbox',
+                ],
+                [
+                    'name' => 'aggregationLevel',
+                    'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'),
+                    'type' => 'string',
+                ],
                 ['name' => 'orientation', 'default' => 0],
                 ['name' => 'screenDimensions', 'default' => ''],
                 ['name' => 'blacklistVideo', 'default' => 1, 'type' => 'checkbox'],
@@ -227,14 +247,17 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'screenShotRequestInterval', 'default' => 0],
                 ['name' => 'expireModifiedLayouts', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'screenShotIntent', 'default' => ''],
-                ['name' => 'screenShotSize', 'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_SCREENSHOT_SIZE_DEFAULT', 200)],
+                [
+                    'name' => 'screenShotSize',
+                    'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_SCREENSHOT_SIZE_DEFAULT', 200),
+                ],
                 ['name' => 'updateStartWindow', 'default' => '00:00'],
                 ['name' => 'updateEndWindow', 'default' => '00:00'],
                 ['name' => 'dayPartId', 'default' => null],
                 ['name' => 'webViewPluginState', 'default' => 'DEMAND'],
                 ['name' => 'hardwareAccelerateWebViewMode', 'default' => '2'],
-                ['name' => 'timeSyncFromCms', 'default' => 0],
-                ['name' => 'webCacheEnabled', 'default' => 0],
+                ['name' => 'timeSyncFromCms', 'default' => 0, 'type' => 'checkbox'],
+                ['name' => 'webCacheEnabled', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'serverPort', 'default' => 9696],
                 ['name' => 'installWithLoadedLinkLibraries', 'default' => 1, 'type' => 'checkbox'],
                 ['name' => 'forceHttps', 'default' => 1, 'type' => 'checkbox'],
@@ -251,8 +274,16 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'downloadEndWindow', 'default' => '00:00'],
                 ['name' => 'dayPartId', 'default' => null],
                 ['name' => 'xmrNetworkAddress', 'default' => ''],
-                ['name' => 'statsEnabled', 'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0), 'type' => 'checkbox'],
-                ['name' => 'aggregationLevel', 'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'), 'type' => 'string'],
+                [
+                    'name' => 'statsEnabled',
+                    'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
+                    'type' => 'checkbox',
+                ],
+                [
+                    'name' => 'aggregationLevel',
+                    'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'),
+                    'type' => 'string',
+                ],
                 ['name' => 'sizeX', 'default' => 0],
                 ['name' => 'sizeY', 'default' => 0],
                 ['name' => 'offsetX', 'default' => 0],
@@ -264,7 +295,10 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'shellCommandAllowList', 'default' => ''],
                 ['name' => 'sendCurrentLayoutAsStatusUpdate', 'default' => 0, 'type' => 'checkbox'],
                 ['name' => 'screenShotRequestInterval', 'default' => 0],
-                ['name' => 'screenShotSize', 'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_SCREENSHOT_SIZE_DEFAULT', 200)],
+                [
+                    'name' => 'screenShotSize',
+                    'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_SCREENSHOT_SIZE_DEFAULT', 200),
+                ],
                 ['name' => 'maxLogFileUploads', 'default' => 3],
                 ['name' => 'embeddedServerPort', 'default' => 9696],
                 ['name' => 'preventSleep', 'default' => 1, 'type' => 'checkbox'],
@@ -278,8 +312,16 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'downloadEndWindow', 'default' => '00:00'],
                 ['name' => 'dayPartId', 'default' => null],
                 ['name' => 'xmrNetworkAddress', 'default' => ''],
-                ['name' => 'statsEnabled', 'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0), 'type' => 'checkbox'],
-                ['name' => 'aggregationLevel', 'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'), 'type' => 'string'],
+                [
+                    'name' => 'statsEnabled',
+                    'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
+                    'type' => 'checkbox',
+                ],
+                [
+                    'name' => 'aggregationLevel',
+                    'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'),
+                    'type' => 'string',
+                ],
                 ['name' => 'orientation', 'default' => 0],
                 ['name' => 'logLevel', 'default' => 'error'],
                 ['name' => 'versionMediaId', 'default' => null],
@@ -305,8 +347,16 @@ class DisplayProfileFactory extends BaseFactory
                 ['name' => 'downloadEndWindow', 'default' => '00:00'],
                 ['name' => 'dayPartId', 'default' => null],
                 ['name' => 'xmrNetworkAddress', 'default' => ''],
-                ['name' => 'statsEnabled', 'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0), 'type' => 'checkbox'],
-                ['name' => 'aggregationLevel', 'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'), 'type' => 'string'],
+                [
+                    'name' => 'statsEnabled',
+                    'default' => (int)$this->config->getSetting('DISPLAY_PROFILE_STATS_DEFAULT', 0),
+                    'type' => 'checkbox',
+                ],
+                [
+                    'name' => 'aggregationLevel',
+                    'default' => $this->config->getSetting('DISPLAY_PROFILE_AGGREGATION_LEVEL_DEFAULT'),
+                    'type' => 'string',
+                ],
                 ['name' => 'orientation', 'default' => 0],
                 ['name' => 'logLevel', 'default' => 'error'],
                 ['name' => 'versionMediaId', 'default' => null],
@@ -327,6 +377,20 @@ class DisplayProfileFactory extends BaseFactory
             ]
         ];
 
+        // get array keys (player types) from the default config
+        // called by getAvailableTypes function, to ensure the default types are always available for selection
+        if ($type === 'defaultTypes') {
+            // remove unknown from the array
+            array_shift($config);
+
+            // build key value array to merge with distinct types from database
+            $defaultTypes = [];
+            foreach (array_keys($config) as $type) {
+                $defaultTypes[]['type'] = $type;
+            }
+            return $defaultTypes;
+        }
+
         if (!isset($config[$type])) {
             $config[$type] = $this->getCustomProfileConfig($type);
         }
@@ -349,35 +413,44 @@ class DisplayProfileFactory extends BaseFactory
             $sortOrder = ['name'];
         }
 
-        try {
-            $params = array();
-            $select = 'SELECT displayProfileId, name, type, config, isDefault, userId, isCustom ';
 
-            $body = ' FROM `displayprofile` WHERE 1 = 1 ';
+        $params = [];
+        $select = 'SELECT displayProfileId, name, type, config, isDefault, userId, isCustom ';
 
-            if ($parsedFilter->getInt('displayProfileId') !== null) {
-                $body .= ' AND displayProfileId = :displayProfileId ';
-                $params['displayProfileId'] = $parsedFilter->getInt('displayProfileId');
-            }
+        $body = ' FROM `displayprofile` WHERE 1 = 1 ';
 
-            if ($parsedFilter->getInt('isDefault') !== null) {
-                $body .= ' AND isDefault = :isDefault ';
-                $params['isDefault'] = $parsedFilter->getInt('isDefault');
-            }
+        if ($parsedFilter->getInt('displayProfileId') !== null) {
+            $body .= ' AND displayProfileId = :displayProfileId ';
+            $params['displayProfileId'] = $parsedFilter->getInt('displayProfileId');
+        }
 
-            // Filter by DisplayProfile Name?
-            if ($parsedFilter->getString('displayProfile') != null) {
-                $terms = explode(',', $parsedFilter->getString('displayProfile'));
-                $this->nameFilter('displayprofile', 'name', $terms, $body, $params, ($parsedFilter->getCheckbox('useRegexForName') == 1));
-            }
+        if ($parsedFilter->getInt('isDefault') !== null) {
+            $body .= ' AND isDefault = :isDefault ';
+            $params['isDefault'] = $parsedFilter->getInt('isDefault');
+        }
 
-            if ($parsedFilter->getString('type') != null) {
-                $body .= ' AND type = :type ';
-                $params['type'] = $parsedFilter->getString('type');
-            }
+        // Filter by DisplayProfile Name?
+        if ($parsedFilter->getString('displayProfile') != null) {
+            $terms = explode(',', $parsedFilter->getString('displayProfile'));
+            $logicalOperator = $parsedFilter->getString('logicalOperatorName', ['default' => 'OR']);
+            $this->nameFilter(
+                'displayprofile',
+                'name',
+                $terms,
+                $body,
+                $params,
+                ($parsedFilter->getCheckbox('useRegexForName') == 1),
+                $logicalOperator
+            );
+        }
 
-            if ($parsedFilter->getInt('commandId') !== null) {
-                $body .= '
+        if ($parsedFilter->getString('type') != null) {
+            $body .= ' AND type = :type ';
+            $params['type'] = $parsedFilter->getString('type');
+        }
+
+        if ($parsedFilter->getInt('commandId') !== null) {
+            $body .= '
                     AND `displayprofile`.displayProfileId IN (
                         SELECT `lkcommanddisplayprofile`.displayProfileId
                           FROM `lkcommanddisplayprofile`
@@ -385,53 +458,56 @@ class DisplayProfileFactory extends BaseFactory
                     )
                 ';
 
-                $params['commandId'] = $parsedFilter->getInt('commandId');
-            }
-
-            if ($parsedFilter->getInt('userId') !== null) {
-                $body .= ' AND `displayprofile`.userId = :userId ';
-                $params['userId'] = $parsedFilter->getInt('userId');
-            }
-
-            // Sorting?
-            $order = '';
-            if (is_array($sortOrder)) {
-                $order .= 'ORDER BY ' . implode(',', $sortOrder);
-            }
-
-            $limit = '';
-            // Paging
-            if ($filterBy !== null && $parsedFilter->getInt('start') !== null && $parsedFilter->getInt('length') !== null) {
-                $limit = ' LIMIT ' . $parsedFilter->getInt('start', ['default' => 0]) . ', ' . $parsedFilter->getInt('length', ['default' => 10]);
-            }
-
-            $sql = $select . $body . $order . $limit;
-
-            foreach ($this->getStore()->select($sql, $params) as $row) {
-                $profile = $this->createEmpty()->hydrate($row, ['intProperties' => ['isDefault', 'isCustom']]);
-
-                $profile->excludeProperty('configDefault');
-                $profile->excludeProperty('configTabs');
-                $profiles[] = $profile;
-            }
-
-            // Paging
-            if ($limit != '' && count($profiles) > 0) {
-                $results = $this->getStore()->select('SELECT COUNT(*) AS total ' . $body, $params);
-                $this->_countLast = intval($results[0]['total']);
-            }
-
-            return $profiles;
-        } catch (\Exception $e) {
-            $this->getLog()->error($e);
-
-            throw new NotFoundException();
+            $params['commandId'] = $parsedFilter->getInt('commandId');
         }
+
+        if ($parsedFilter->getInt('userId') !== null) {
+            $body .= ' AND `displayprofile`.userId = :userId ';
+            $params['userId'] = $parsedFilter->getInt('userId');
+        }
+
+        // Sorting?
+        $order = '';
+        if (is_array($sortOrder)) {
+            $order .= 'ORDER BY ' . implode(',', $sortOrder);
+        }
+
+        $limit = '';
+        // Paging
+        if ($filterBy !== null && $parsedFilter->getInt('start') !== null && $parsedFilter->getInt('length') !== null) {
+            $limit = ' LIMIT ' . $parsedFilter->getInt('start', ['default' => 0]) .
+                ', ' . $parsedFilter->getInt('length', ['default' => 10]);
+        }
+
+        $sql = $select . $body . $order . $limit;
+
+        foreach ($this->getStore()->select($sql, $params) as $row) {
+            $profile = $this->createEmpty()->hydrate($row, ['intProperties' => ['isDefault', 'isCustom']]);
+
+            $profile->excludeProperty('configDefault');
+            $profile->excludeProperty('configTabs');
+            $profiles[] = $profile;
+        }
+
+        // Paging
+        if ($limit != '' && count($profiles) > 0) {
+            $results = $this->getStore()->select('SELECT COUNT(*) AS total ' . $body, $params);
+            $this->_countLast = intval($results[0]['total']);
+        }
+
+        return $profiles;
     }
 
     public function getAvailableTypes()
     {
-        $types = $this->getStore()->select('SELECT DISTINCT type FROM `displayprofile` ORDER BY type', []);
+        // get distinct player types from the displayprofile table, this will include any custom profile types as well
+        $dbTypes = $this->getStore()->select('SELECT DISTINCT type FROM `displayprofile` ORDER BY type', []);
+        // get an array of default player types from default config,
+        // this is to ensure we will always have the default types available for add form
+        $defaultTypes = $this->loadForType('defaultTypes');
+
+        // merge arrays removing any duplicates
+        $types = array_unique(array_merge($dbTypes, $defaultTypes), SORT_REGULAR);
 
         $entries = [];
         foreach ($types as $row) {
