@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -32,7 +32,6 @@ use Xibo\Entity\User;
 use Xibo\Helper\ByteFormatter;
 use Xibo\Helper\Environment;
 use Xibo\Service\ConfigServiceInterface;
-use Xibo\Support\Exception\InvalidArgumentException;
 use Xibo\Support\Exception\NotFoundException;
 
 /**
@@ -41,6 +40,8 @@ use Xibo\Support\Exception\NotFoundException;
  */
 class MediaFactory extends BaseFactory
 {
+    use TagTrait;
+
     /** @var Media[] */
     private $remoteDownloadQueue = [];
 
@@ -57,11 +58,6 @@ class MediaFactory extends BaseFactory
      */
     private $permissionFactory;
 
-    /**
-     * @var TagFactory
-     */
-    private $tagFactory;
-
     /**
      * @var PlaylistFactory
      */
@@ -73,16 +69,14 @@ class MediaFactory extends BaseFactory
      * @param UserFactory $userFactory
      * @param ConfigServiceInterface $config
      * @param PermissionFactory $permissionFactory
-     * @param TagFactory $tagFactory
      * @param PlaylistFactory $playlistFactory
      */
-    public function __construct($user, $userFactory, $config, $permissionFactory, $tagFactory, $playlistFactory)
+    public function __construct($user, $userFactory, $config, $permissionFactory, $playlistFactory)
     {
         $this->setAclDependencies($user, $userFactory);
 
         $this->config = $config;
         $this->permissionFactory = $permissionFactory;
-        $this->tagFactory = $tagFactory;
         $this->playlistFactory = $playlistFactory;
     }
 
@@ -98,8 +92,7 @@ class MediaFactory extends BaseFactory
             $this->getDispatcher(),
             $this->config,
             $this,
-            $this->permissionFactory,
-            $this->tagFactory
+            $this->permissionFactory
         );
     }
 
@@ -124,25 +117,14 @@ class MediaFactory extends BaseFactory
         return $media;
     }
 
-    /**
-     * Create System Module File
-     * @param $name
-     * @param string $file
-     * @return Media
-     */
-    public function createModuleSystemFile($name, $file = '')
-    {
-        return $this->createModuleFile($name, $file, 1);
-    }
-
     /**
      * Create Module File
      * @param $name
-     * @param $file
-     * @param $systemFile
+     * @param string|null $file
      * @return Media
+     * @throws \Xibo\Support\Exception\NotFoundException
      */
-    public function createModuleFile($name, $file = '', $systemFile = 0)
+    public function createModuleFile($name, ?string $file = ''): Media
     {
         if ($file == '') {
             $file = $name;
@@ -155,8 +137,7 @@ class MediaFactory extends BaseFactory
             // Reassert the new file (which we might want to download)
             $media->fileName = $file;
             $media->storedAs = $name;
-        }
-        catch (NotFoundException $e) {
+        } catch (NotFoundException $e) {
             $media = $this->createEmpty();
             $media->name = $name;
             $media->fileName = $file;
@@ -164,33 +145,7 @@ class MediaFactory extends BaseFactory
             $media->expires = 0;
             $media->storedAs = $name;
             $media->ownerId = $this->getUserFactory()->getSystemUser()->getOwnerId();
-            $media->moduleSystemFile = $systemFile;
-        }
-
-        return $media;
-    }
-
-    /**
-     * Create module files from folder
-     * @param string $folder The path to the folder to add.
-     * @return Media[]
-     * @throws InvalidArgumentException
-     */
-    public function createModuleFileFromFolder($folder)
-    {
-        $media = [];
-
-        if (!is_dir($folder)) {
-            throw new InvalidArgumentException(__('Not a folder'));
-        }
-
-        foreach (array_diff(scandir($folder), array('..', '.')) as $file) {
-            if (is_dir($folder . DIRECTORY_SEPARATOR . $file)) continue;
-            
-            $file = $this->createModuleSystemFile($file, $folder . DIRECTORY_SEPARATOR . $file);
-            $file->moduleSystemFile = true;
-
-            $media[] = $file;
+            $media->moduleSystemFile = 0;
         }
 
         return $media;
@@ -223,14 +178,15 @@ class MediaFactory extends BaseFactory
             $media->duration = $requestOptions['duration'];
             $media->moduleSystemFile = 0;
             $media->isRemote = true;
-            $media->urlDownload = true;
-            $media->extension = $requestOptions['extension'] ?? null;
+            $media->setUnmatchedProperty('urlDownload', true);
+            $media->setUnmatchedProperty('extension', $requestOptions['extension'] ?? null);
             $media->enableStat = $requestOptions['enableStat'];
             $media->folderId = $requestOptions['folderId'];
             $media->permissionsFolderId = $requestOptions['permissionsFolderId'];
         }
 
-        $this->getLog()->debug('Queue download of: ' . $uri . ', current mediaId for this download is ' . $media->mediaId . '.');
+        $this->getLog()->debug('Queue download of: ' . $uri . ', current mediaId for this download is '
+            . $media->mediaId . '.');
 
         // We update the desired expiry here - isSavedRequired is tested against the original value
         $media->expires = $expiry;
@@ -307,9 +263,14 @@ class MediaFactory extends BaseFactory
                     $requestOptions = array_merge($media->downloadRequestOptions(), [
                         'sink' => $sink,
                         'on_headers' => function (ResponseInterface $response) {
-                            $this->getLog()->debug('DEBUG: ' . $response->getStatusCode());
+                            $this->getLog()->debug('processDownloads: on_headers status code = '
+                                . $response->getStatusCode());
+
                             if ($response->getStatusCode() < 299) {
-                                $this->getLog()->debug('DEBUG: ' . var_export($response->getHeaders(), true));
+                                $this->getLog()->debug('processDownloads: successful, headers = '
+                                    . var_export($response->getHeaders(), true));
+
+                                // Get the content length
                                 $contentLength = $response->getHeaderLine('Content-Length');
                                 if (empty($contentLength)
                                     || intval($contentLength) > ByteFormatter::toBytes(Environment::getMaxUploadSize())
@@ -340,7 +301,8 @@ class MediaFactory extends BaseFactory
                             $success($item);
                         }
                     } catch (\Exception $e) {
-                        $this->getLog()->error('Unable to save:' . $item->mediaId . '. ' . $e->getMessage());
+                        $this->getLog()->error('processDownloads: Unable to save mediaId '
+                            . $item->mediaId . '. ' . $e->getMessage());
 
                         // Remove it
                         $item->delete(['rollback' => true]);
@@ -351,7 +313,7 @@ class MediaFactory extends BaseFactory
                         }
                     }
                 },
-                'rejected' => function ($reason, $index) use ($log, $rejected) {
+                'rejected' => function ($reason, $index) use ($log, $queue, $rejected) {
                     /* @var RequestException $reason */
                     $log->error(
                         sprintf(
@@ -362,7 +324,10 @@ class MediaFactory extends BaseFactory
                         )
                     );
 
-                    // If a failure callback has been provided, call it
+                    // We should remove the media record.
+                    $queue[$index]->delete(['rollback' => true]);
+
+                    // If a rejected callback has been provided, call it
                     if ($rejected !== null && is_callable($rejected)) {
                         // Do we have a wrapped exception?
                         $reasonMessage = $reason->getPrevious() !== null
@@ -401,9 +366,13 @@ class MediaFactory extends BaseFactory
      * @return Media
      * @throws NotFoundException
      */
-    public function getById($mediaId)
+    public function getById($mediaId, bool $isDisableUserCheck = true)
     {
-        $media = $this->query(null, array('disableUserCheck' => 1, 'mediaId' => $mediaId, 'allModules' => 1));
+        $media = $this->query(null, [
+            'disableUserCheck' => $isDisableUserCheck ? 1 : 0,
+            'mediaId' => $mediaId,
+            'allModules' => 1,
+        ]);
 
         if (count($media) <= 0) {
             throw new NotFoundException(__('Cannot find media'));
@@ -512,11 +481,32 @@ class MediaFactory extends BaseFactory
         return $this->query(null, ['disableUserCheck' => 1, 'layoutId' => $layoutId, 'isEdited' => $edited, 'excludeDynamicPlaylistMedia' => $excludeDynamicPlaylistMedia]);
     }
 
+    /**
+     * Get Media by campaignId
+     * @param int $campaignId
+     * @return Media[]
+     * @throws NotFoundException
+     */
+    public function getByCampaignId($campaignId)
+    {
+        return $this->query(null, ['disableUserCheck' => 1, 'campaignId' => $campaignId]);
+    }
+
     public function getForMenuBoards()
     {
         return $this->query(null, ['onlyMenuBoardAllowed' => 1]);
     }
 
+    /**
+     * @param int $folderId
+     * @return Media[]
+     * @throws NotFoundException
+     */
+    public function getByFolderId(int $folderId)
+    {
+        return $this->query(null, ['disableUserCheck' => 1, 'folderId' => $folderId]);
+    }
+
     /**
      * @param null $sortOrder
      * @param array $filterBy
@@ -572,17 +562,11 @@ class MediaFactory extends BaseFactory
                `media`.folderId,
                `media`.permissionsFolderId,
                `media`.orientation,
-               ( 
-                   SELECT GROUP_CONCAT(CONCAT_WS(\'|\', tag, value))
-                        FROM tag
-                        INNER JOIN lktagmedia
-                            ON lktagmedia.tagId = tag.tagId
-                            WHERE lktagmedia.mediaId = media.mediaId
-                        GROUP BY lktagmedia.mediaId
-               ) as tags,
+               `media`.width,
+               `media`.height,
                `user`.UserName AS owner,
             ';
-        $select .= "     (SELECT GROUP_CONCAT(DISTINCT `group`.group)
+        $select .= '     (SELECT GROUP_CONCAT(DISTINCT `group`.group)
                               FROM `permission`
                                 INNER JOIN `permissionentity`
                                 ON `permissionentity`.entityId = permission.entityId
@@ -591,17 +575,17 @@ class MediaFactory extends BaseFactory
                              WHERE entity = :entity
                                 AND objectId = media.mediaId
                                 AND view = 1
-                            ) AS groupsWithPermissions, ";
+                            ) AS groupsWithPermissions, ';
         $params['entity'] = 'Xibo\\Entity\\Media';
 
-        $select .= "   media.originalFileName AS fileName ";
+        $select .= '   media.originalFileName AS fileName ';
 
-        $body = " FROM media ";
-        $body .= "   LEFT OUTER JOIN media parentmedia ";
-        $body .= "   ON parentmedia.editedMediaId = media.mediaId ";
+        $body = ' FROM media ';
+        $body .= '   LEFT OUTER JOIN media parentmedia ';
+        $body .= '   ON parentmedia.editedMediaId = media.mediaId ';
 
         // Media might be linked to the system user (userId 0)
-        $body .= "   LEFT OUTER JOIN `user` ON `user`.userId = `media`.userId ";
+        $body .= '   LEFT OUTER JOIN `user` ON `user`.userId = `media`.userId ';
 
         if ($sanitizedFilter->getInt('displayGroupId') !== null) {
             $body .= '
@@ -613,15 +597,7 @@ class MediaFactory extends BaseFactory
             $params['displayGroupId'] = $sanitizedFilter->getInt('displayGroupId');
         }
 
-        $body .= " WHERE 1 = 1 ";
-
-        if ($sanitizedFilter->getInt('notPlayerSoftware') == 1) {
-            $body .= ' AND media.type <> \'playersoftware\' ';
-        }
-
-        if ($sanitizedFilter->getInt('notSavedReport') == 1) {
-            $body .= ' AND media.type <> \'savedreport\' ';
-        }
+        $body .= ' WHERE 1 = 1 ';
 
         if ($sanitizedFilter->getInt('allModules') == 0) {
             $body .= ' AND media.type <> \'module\' ';
@@ -646,18 +622,20 @@ class MediaFactory extends BaseFactory
         }
 
         // Unused only?
-        if ($sanitizedFilter->getInt('unusedOnly') !== null) {
-
+        if ($sanitizedFilter->getInt('unusedOnly') === 1) {
             $body .= '
-                AND media.mediaId NOT IN (SELECT mediaId FROM `lkwidgetmedia`)
-                AND media.mediaId NOT IN (SELECT mediaId FROM `lkmediadisplaygroup`)
-                AND media.mediaId NOT IN (SELECT mediaId FROM `menu_category` WHERE mediaId IS NOT NULL)
-                AND media.mediaId NOT IN (SELECT mediaId FROM `menu_product` WHERE mediaId IS NOT NULL)
-                AND media.mediaId NOT IN (SELECT backgroundImageId FROM `layout` WHERE backgroundImageId IS NOT NULL)
-                AND media.type <> \'module\'
-                AND media.type <> \'font\'
-                AND media.type <> \'playersoftware\'
-                AND media.type <> \'savedreport\'
+                AND `media`.`mediaId` NOT IN (SELECT `mediaId` FROM `display_media`)
+                AND `media`.`mediaId` NOT IN (SELECT `mediaId` FROM `lkwidgetmedia`)
+                AND `media`.`mediaId` NOT IN (SELECT `mediaId` FROM `lkmediadisplaygroup`)
+                AND `media`.`mediaId` NOT IN (SELECT `mediaId` FROM `menu_category` WHERE `mediaId` IS NOT NULL)
+                AND `media`.`mediaId` NOT IN (SELECT `mediaId` FROM `menu_product` WHERE `mediaId` IS NOT NULL)
+                AND `media`.`mediaId` NOT IN (
+                    SELECT `backgroundImageId` FROM `layout` WHERE `backgroundImageId` IS NOT NULL
+                )
+                AND `media`.`type` <> \'module\'
+                AND `media`.`type` <> \'font\'
+                AND `media`.`type` <> \'playersoftware\'
+                AND `media`.`type` <> \'savedreport\'
             ';
 
             // DataSets with library images
@@ -694,9 +672,25 @@ class MediaFactory extends BaseFactory
             }
         }
 
+        // Unlinked only?
+        if ($sanitizedFilter->getInt('unlinkedOnly') === 1) {
+            $body .= '
+                AND `media`.`mediaId` NOT IN (SELECT `mediaId` FROM `display_media`)
+            ';
+        }
+
         if ($sanitizedFilter->getString('name') != null) {
             $terms = explode(',', $sanitizedFilter->getString('name'));
-            $this->nameFilter('media', 'name', $terms, $body, $params, ($sanitizedFilter->getCheckbox('useRegexForName') == 1));
+            $logicalOperator = $sanitizedFilter->getString('logicalOperatorName', ['default' => 'OR']);
+            $this->nameFilter(
+                'media',
+                'name',
+                $terms,
+                $body,
+                $params,
+                ($sanitizedFilter->getCheckbox('useRegexForName') == 1),
+                $logicalOperator
+            );
         }
 
         if ($sanitizedFilter->getString('nameExact') != '') {
@@ -705,7 +699,7 @@ class MediaFactory extends BaseFactory
         }
 
         if ($sanitizedFilter->getInt('mediaId', ['default'=> -1]) != -1) {
-            $body .= " AND media.mediaId = :mediaId ";
+            $body .= ' AND media.mediaId = :mediaId ';
             $params['mediaId'] = $sanitizedFilter->getInt('mediaId');
         } else if ($sanitizedFilter->getInt('parentMediaId') !== null) {
             $body .= ' AND media.editedMediaId = :mediaId ';
@@ -745,7 +739,7 @@ class MediaFactory extends BaseFactory
         }
 
         if ($sanitizedFilter->getInt('ownerId') !== null) {
-            $body .= " AND media.userid = :ownerId ";
+            $body .= ' AND media.userid = :ownerId ';
             $params['ownerId'] = $sanitizedFilter->getInt('ownerId');
         }
 
@@ -756,15 +750,19 @@ class MediaFactory extends BaseFactory
         }
 
         if ($sanitizedFilter->getInt('released') !== null) {
-            $body .= " AND media.released = :released ";
+            $body .= ' AND media.released = :released ';
             $params['released'] = $sanitizedFilter->getInt('released');
         }
 
+        if ($sanitizedFilter->getCheckbox('unreleasedOnly') === 1) {
+            $body .= ' AND media.released <> 1 ';
+        }
+
         if ($sanitizedFilter->getInt('retired', ['default'=> -1]) == 1)
-            $body .= " AND media.retired = 1 ";
+            $body .= ' AND media.retired = 1 ';
 
         if ($sanitizedFilter->getInt('retired', ['default'=> -1]) == 0)
-            $body .= " AND media.retired = 0 ";
+            $body .= ' AND media.retired = 0 ';
 
         // Expired files?
         if ($sanitizedFilter->getInt('expires') != 0) {
@@ -813,6 +811,26 @@ class MediaFactory extends BaseFactory
             $params['layoutId'] = $sanitizedFilter->getInt('layoutId');
         }
 
+        if ($sanitizedFilter->getInt('campaignId') !== null) {
+            $body .= '
+                AND media.mediaId IN (
+                    SELECT `lkwidgetmedia`.mediaId
+                      FROM region
+                        INNER JOIN playlist
+                        ON playlist.regionId = region.regionId
+                        INNER JOIN lkplaylistplaylist
+                        ON lkplaylistplaylist.parentId = playlist.playlistId
+                        INNER JOIN widget
+                        ON widget.playlistId = lkplaylistplaylist.childId
+                        INNER JOIN lkwidgetmedia
+                        ON widget.widgetId = lkwidgetmedia.widgetId
+                        INNER JOIN `lkcampaignlayout` lkcl
+                        ON lkcl.layoutid = region.layoutid
+                        AND lkcl.CampaignID = :campaignId)';
+
+            $params['campaignId'] = $sanitizedFilter->getInt('campaignId');
+        }
+
         // Tags
         if ($sanitizedFilter->getString('tags') != '') {
             $tagFilter = $sanitizedFilter->getString('tags');
@@ -828,15 +846,59 @@ class MediaFactory extends BaseFactory
             } else {
                 $operator = $sanitizedFilter->getCheckbox('exactTags') == 1 ? '=' : 'LIKE';
                 $logicalOperator = $sanitizedFilter->getString('logicalOperator', ['default' => 'OR']);
-                $body .= ' AND `media`.mediaId IN (
-                SELECT `lktagmedia`.mediaId
-                  FROM tag
-                    INNER JOIN `lktagmedia`
-                    ON `lktagmedia`.tagId = tag.tagId
-                ';
+                $allTags = explode(',', $tagFilter);
+                $notTags = [];
+                $tags = [];
 
-                $tags = explode(',', $tagFilter);
-                $this->tagFilter($tags, 'lktagmedia', 'lkTagMediaId', 'mediaId', $logicalOperator, $operator, $body, $params);
+                foreach ($allTags as $tag) {
+                    if (str_starts_with($tag, '-')) {
+                        $notTags[] = ltrim(($tag), '-');
+                    } else {
+                        $tags[] = $tag;
+                    }
+                }
+
+                if (!empty($notTags)) {
+                    $body .= ' AND `media`.mediaId NOT IN (
+                    SELECT `lktagmedia`.mediaId
+                      FROM tag
+                        INNER JOIN `lktagmedia`
+                        ON `lktagmedia`.tagId = tag.tagId
+                    ';
+
+                    $this->tagFilter(
+                        $notTags,
+                        'lktagmedia',
+                        'lkTagMediaId',
+                        'mediaId',
+                        $logicalOperator,
+                        $operator,
+                        true,
+                        $body,
+                        $params
+                    );
+                }
+
+                if (!empty($tags)) {
+                    $body .= ' AND `media`.mediaId IN (
+                    SELECT `lktagmedia`.mediaId
+                      FROM tag
+                        INNER JOIN `lktagmedia`
+                        ON `lktagmedia`.tagId = tag.tagId
+                    ';
+
+                    $this->tagFilter(
+                        $tags,
+                        'lktagmedia',
+                        'lkTagMediaId',
+                        'mediaId',
+                        $logicalOperator,
+                        $operator,
+                        false,
+                        $body,
+                        $params
+                    );
+                }
             }
         }
 
@@ -857,7 +919,7 @@ class MediaFactory extends BaseFactory
         }
 
         if ($sanitizedFilter->getInt('folderId') !== null) {
-            $body .= " AND media.folderId = :folderId ";
+            $body .= ' AND media.folderId = :folderId ';
             $params['folderId'] = $sanitizedFilter->getInt('folderId');
         }
 
@@ -870,8 +932,8 @@ class MediaFactory extends BaseFactory
             $params['orientation'] = $sanitizedFilter->getString('orientation');
         }
 
-        if ($sanitizedFilter->getInt('noOrientation') === 1) {
-            $body .= ' AND media.orientation IS NULL ';
+        if ($sanitizedFilter->getInt('requiresMetaUpdate') === 1) {
+            $body .= ' AND (`media`.orientation IS NULL OR IFNULL(`media`.width, 0) = 0)';
         }
 
         // View Permissions
@@ -890,13 +952,25 @@ class MediaFactory extends BaseFactory
         }
 
         $sql = $select . $body . $order . $limit;
-        
+        $mediaIds = [];
+
         foreach ($this->getStore()->select($sql, $params) as $row) {
             $media = $this->createEmpty()->hydrate($row, [
                 'intProperties' => [
-                    'duration', 'size', 'released', 'moduleSystemFile', 'isEdited', 'expires', 'valid'
+                    'duration',
+                    'size',
+                    'released',
+                    'moduleSystemFile',
+                    'isEdited',
+                    'expires',
+                    'valid',
+                    'width',
+                    'height'
                 ]
             ]);
+
+            $mediaIds[] = $media->mediaId;
+
             $media->excludeProperty('layoutBackgroundImages');
             $media->excludeProperty('widgets');
             $media->excludeProperty('displayGroups');
@@ -904,6 +978,11 @@ class MediaFactory extends BaseFactory
             $entries[] = $media;
         }
 
+        // decorate with TagLinks
+        if (count($entries) > 0) {
+            $this->decorateWithTagLinks('lktagmedia', 'mediaId', $mediaIds, $entries);
+        }
+
         // Paging
         if ($limit != '' && count($entries) > 0) {
             unset($params['entity']);
