@@ -1,13 +1,33 @@
 <?php
+/*
+ * Copyright (C) 2023 Xibo Signage Ltd
+ *
+ * Xibo - Digital Signage - https://xibosignage.com
+ *
+ * This file is part of Xibo.
+ *
+ * Xibo is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * any later version.
+ *
+ * Xibo is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with Xibo.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 namespace Xibo\Helper;
 
 use Exception;
 use Xibo\Entity\Layout;
 use Xibo\Entity\Permission;
-use Xibo\Entity\Widget;
 use Xibo\Event\LibraryReplaceEvent;
 use Xibo\Event\LibraryReplaceWidgetEvent;
+use Xibo\Event\LibraryUploadCompleteEvent;
 use Xibo\Event\MediaDeleteEvent;
 use Xibo\Support\Exception\AccessDeniedException;
 use Xibo\Support\Exception\InvalidArgumentException;
@@ -60,20 +80,16 @@ class XiboUploadHandler extends BlueImpUploadHandler
                 : '';
 
             // Guess the type
-            $module = $controller->getModuleFactory()->getByExtension(strtolower(substr(strrchr($fileName, '.'), 1)));
-            $module = $controller->getModuleFactory()->create($module->type);
-            $module->setUser($controller->getUser());
+            $module = $controller->getModuleFactory()
+                ->getByExtension(strtolower(substr(strrchr($fileName, '.'), 1)));
 
             $controller->getLog()->debug(sprintf(
                 'Module Type = %s, Name = %s',
-                $module->getModuleType(),
-                $module->getModuleName()
+                $module->type,
+                $module->name
             ));
 
-            // Do we need to run any pre-processing on the file?
-            $module->preProcessFile($filePath);
-
-            // Old Media Id or not?
+            // If we have an oldMediaId then we are replacing that media with new one
             if ($this->options['oldMediaId'] != 0) {
                 $updateInLayouts = ($this->options['updateInLayouts'] == 1);
                 $deleteOldRevisions = ($this->options['deleteOldRevisions'] == 1);
@@ -93,7 +109,7 @@ class XiboUploadHandler extends BlueImpUploadHandler
                 }
 
                 // Check to see if we are changing the media type
-                if ($oldMedia->mediaType != $module->getModuleType() && $this->options['allowMediaTypeChange'] == 0) {
+                if ($oldMedia->mediaType != $module->type && $this->options['allowMediaTypeChange'] == 0) {
                     throw new InvalidArgumentException(
                         __('You cannot replace this media with an item of a different type')
                     );
@@ -108,36 +124,32 @@ class XiboUploadHandler extends BlueImpUploadHandler
                 $name = ($name == '') ? $oldMedia->name : $name;
                 $tags = ($tags == '') ? '' : $tags;
 
-
                 // Add the Media
                 // the userId is either the existing user
-                // (if we are changing media type) or the currently logged in user otherwise.
+                // (if we are changing media type) or the currently logged-in user otherwise.
                 $media = $controller->getMediaFactory()->create(
                     $name,
                     $fileName,
-                    $module->getModuleType(),
+                    $module->type,
                     $oldMedia->getOwnerId()
                 );
 
                 if ($tags != '') {
-                    $concatTags = (string)$oldMedia->tags . ',' . $tags;
-                    $media->replaceTags($controller->getTagFactory()->tagsFromString($concatTags));
+                    $concatTags = $oldMedia->getTagString() . ',' . $tags;
+                    $media->updateTagLinks($controller->getTagFactory()->tagsFromString($concatTags));
                 }
 
-                // Set the duration
-                if ($oldMedia->mediaType != 'video' && $media->mediaType != 'video') {
-                    $media->duration = $oldMedia->duration;
+                // Apply the duration from the old media, unless we're a video
+                if ($module->type === 'video') {
+                    $media->duration = $module->fetchDurationOrDefaultFromFile($filePath);
                 } else {
-                    $media->duration = $module->determineDuration($filePath);
+                    $media->duration = $oldMedia->duration;
                 }
 
-                // Pre-process
-                $module->preProcess($media, $filePath);
-
                 // Raise an event for this media item
                 $controller->getDispatcher()->dispatch(
-                    LibraryReplaceEvent::$NAME,
-                    new LibraryReplaceEvent($module, $media, $oldMedia)
+                    new LibraryReplaceEvent($module, $media, $oldMedia),
+                    LibraryReplaceEvent::$NAME
                 );
 
                 $media->enableStat = $oldMedia->enableStat;
@@ -148,12 +160,11 @@ class XiboUploadHandler extends BlueImpUploadHandler
                 // Save
                 $media->save(['oldMedia' => $oldMedia]);
 
-                // Post process
-                $playerVersionFactory = null;
-                if ($media->mediaType === 'playersoftware') {
-                    $playerVersionFactory = $controller->getPlayerVersionFactory();
-                }
-                $module->postProcess($media, $playerVersionFactory);
+                // Upload finished
+                $controller->getDispatcher()->dispatch(
+                    new LibraryUploadCompleteEvent($media),
+                    LibraryUploadCompleteEvent::$NAME
+                );
 
                 $controller->getLog()->debug('Copying permissions to new media');
 
@@ -173,42 +184,41 @@ class XiboUploadHandler extends BlueImpUploadHandler
                     $controller->getLog()->debug('Replace in all Layouts selected. Getting associated widgets');
 
                     foreach ($controller->getWidgetFactory()->getByMediaId($oldMedia->mediaId, 0) as $widget) {
-                        /* @var Widget $widget */
                         if (!$controller->getUser()->checkEditable($widget)) {
                             // Widget that we cannot update,
                             // this means we can't delete the original mediaId when it comes time to do so.
                             $deleteOldRevisions = false;
 
-                            $controller->getLog()->info(
-                                'Media used on Widget that we cannot edit.
-                             Delete Old Revisions has been disabled.'
-                            );
+                            $controller->getLog()->info('Media used on Widget that we cannot edit. Delete Old Revisions has been disabled.');
                         }
 
                         // If we are replacing an audio media item,
                         // we should check to see if the widget we've found has any
                         // audio items assigned.
-                        if ($module->getModuleType() == 'audio' &&
-                            in_array($oldMedia->mediaId, $widget->getAudioIds())) {
+                        if ($module->type == 'audio'
+                            && in_array($oldMedia->mediaId, $widget->getAudioIds())
+                        ) {
                             $controller->getLog()->debug('Found audio on widget that needs updating. widgetId = ' .
                                 $widget->getId() . '. Linking ' . $media->mediaId);
 
                             $widget->unassignAudioById($oldMedia->mediaId);
                             $widget->assignAudioById($media->mediaId);
                             $widget->save();
-                        } elseif (count($widget->getPrimaryMedia()) > 0 &&
-                            $widget->getPrimaryMediaId() == $oldMedia->mediaId) {
+                        } elseif (count($widget->getPrimaryMedia()) > 0
+                            && $widget->getPrimaryMediaId() == $oldMedia->mediaId
+                        ) {
                             // We're only interested in primary media at this point (no audio)
                             // Check whether this widget is of the same type as our incoming media item
                             // This needs to be applicable only to non region specific Widgets,
                             // otherwise we would not be able to replace Media references in region specific Widgets.
-                            $moduleWidget = $controller->getModuleFactory()->createWithWidget($widget);
+                            $moduleToReplace = $controller->getModuleFactory()->getByType($widget->type);
 
-                            if ($widget->type != $module->getModuleType() &&
-                                $moduleWidget->getModule()->regionSpecific == 0) {
+                            // If these types are different, and the module we're replacing isn't region specific
+                            // then we need to see if we're allowed to change it.
+                            if ($widget->type != $module->type && $moduleToReplace->regionSpecific == 0) {
                                 // Are we supposed to switch, or should we prevent?
                                 if ($this->options['allowMediaTypeChange'] == 1) {
-                                    $widget->type = $module->getModuleType();
+                                    $widget->type = $module->type;
                                 } else {
                                     throw new InvalidArgumentException(__(
                                         'You cannot replace this media with an item of a different type'
@@ -225,7 +235,6 @@ class XiboUploadHandler extends BlueImpUploadHandler
                             $widget->assignMedia($media->mediaId);
 
                             // calculate duration
-                            $module->setWidget($widget);
                             $widget->calculateDuration($module);
 
                             // replace mediaId references in applicable widgets
@@ -237,8 +246,8 @@ class XiboUploadHandler extends BlueImpUploadHandler
 
                             // Raise an event for this media item
                             $controller->getDispatcher()->dispatch(
-                                LibraryReplaceWidgetEvent::$NAME,
-                                new LibraryReplaceWidgetEvent($module, $widget, $media, $oldMedia)
+                                new LibraryReplaceWidgetEvent($module, $widget, $media, $oldMedia),
+                                LibraryReplaceWidgetEvent::$NAME
                             );
 
                             // Save
@@ -252,6 +261,7 @@ class XiboUploadHandler extends BlueImpUploadHandler
                             'Updating layouts with the old media %d as the background image.',
                             $oldMedia->mediaId
                         ));
+
                         // Get all Layouts with this as the background image
                         foreach ($controller->getLayoutFactory()->query(
                             null,
@@ -317,13 +327,18 @@ class XiboUploadHandler extends BlueImpUploadHandler
                         $controller->getLog()->debug('No prior media found');
                     }
 
-                    $controller->getDispatcher()->dispatch(MediaDeleteEvent::$NAME, new MediaDeleteEvent($oldMedia));
+                    $controller->getDispatcher()->dispatch(
+                        new MediaDeleteEvent($oldMedia),
+                        MediaDeleteEvent::$NAME
+                    );
                     $oldMedia->delete();
                 } else {
                     $oldMedia->parentId = $media->mediaId;
                     $oldMedia->save(['validate' => false]);
                 }
             } else {
+                // Not a replacement
+                // Fresh upload
                 // The media name might be empty here, because the user isn't forced to select it
                 $name = ($name == '') ? $fileName : $name;
                 $tags = ($tags == '') ? '' : $tags;
@@ -332,18 +347,16 @@ class XiboUploadHandler extends BlueImpUploadHandler
                 $media = $controller->getMediaFactory()->create(
                     $name,
                     $fileName,
-                    $module->getModuleType(),
+                    $module->type,
                     $this->options['userId']
                 );
 
                 if ($tags != '') {
-                    $media->replaceTags($controller->getTagFactory()->tagsFromString($tags));
+                    $media->updateTagLinks($controller->getTagFactory()->tagsFromString($tags));
                 }
-                // Set the duration
-                $media->duration = $module->determineDuration($filePath);
 
-                // Pre-process
-                $module->preProcess($media, $filePath);
+                // Set the duration
+                $media->duration = $module->fetchDurationOrDefaultFromFile($filePath);
 
                 if ($media->enableStat == null) {
                     $media->enableStat = $controller->getConfig()->getSetting('MEDIA_STATS_ENABLED_DEFAULT');
@@ -354,23 +367,17 @@ class XiboUploadHandler extends BlueImpUploadHandler
                 $media->folderId = $this->options['oldFolderId'];
 
                 // Permissions
-                try {
-                    $folder = $controller->getFolderFactory()->getById($this->options['oldFolderId']);
-                    $media->permissionsFolderId =
-                        ($folder->permissionsFolderId == null) ? $folder->id : $folder->permissionsFolderId;
-                } catch (NotFoundException $exception) {
-                    $media->permissionsFolderId = 1;
-                }
+                $folder = $controller->getFolderFactory()->getById($this->options['oldFolderId'], 0);
+                $media->permissionsFolderId = $folder->getPermissionFolderIdOrThis();
 
                 // Save
                 $media->save();
 
-                // Post process
-                $playerVersionFactory = null;
-                if ($media->mediaType === 'playersoftware') {
-                    $playerVersionFactory = $controller->getPlayerVersionFactory();
-                }
-                $module->postProcess($media, $playerVersionFactory);
+                // Upload finished
+                $controller->getDispatcher()->dispatch(
+                    new LibraryUploadCompleteEvent($media),
+                    LibraryUploadCompleteEvent::$NAME
+                );
             }
 
             // Configure the return values according to the media item we've added
@@ -382,7 +389,7 @@ class XiboUploadHandler extends BlueImpUploadHandler
             $file->fileSize = $media->fileSize;
             $file->md5 = $media->md5;
             $file->enableStat = $media->enableStat;
-            $file->mediaType = $module->getModuleType();
+            $file->mediaType = $module->type;
             $file->fileName = $fileName;
 
             // Test to ensure the final file size is the same as the file size we're expecting
@@ -392,31 +399,37 @@ class XiboUploadHandler extends BlueImpUploadHandler
                     'size'
                 );
             }
-            // Fonts, then install
-            if ($module->getModuleType() == 'font') {
-                $controller->getMediaService()->installFonts($this->options['routeParser']);
-            }
 
             // Are we assigning to a Playlist?
             if ($this->options['playlistId'] != 0 && $this->options['widgetId'] == 0) {
-                $controller->getLog()->debug('Assigning uploaded media to playlistId ' . $this->options['playlistId']);
+                $controller->getLog()->debug('Assigning uploaded media to playlistId '
+                    . $this->options['playlistId']);
 
                 // Get the Playlist
                 $playlist = $controller->getPlaylistFactory()->getById($this->options['playlistId']);
 
+                if (!$playlist->isEditable()) {
+                    throw new InvalidArgumentException(
+                        __('This Layout is not a Draft, please checkout.'),
+                        'layoutId'
+                    );
+                }
+
                 // Create a Widget and add it to our region
                 $widget = $controller->getWidgetFactory()->create(
                     $this->options['userId'],
                     $playlist->playlistId,
-                    $module->getModuleType(),
-                    $media->duration
+                    $module->type,
+                    $media->duration,
+                    $module->schemaVersion
                 );
 
-                // Assign the widget to the module
-                $module->setWidget($widget);
-
-                // Set default options (this sets options on the widget)
-                $module->setDefaultWidgetOptions();
+                // Default options
+                $widget->setOptionValue(
+                    'enableStat',
+                    'attrib',
+                    $controller->getConfig()->getSetting('WIDGET_STATS_ENABLED_DEFAULT')
+                );
 
                 // From/To dates?
                 $widget->fromDt = $this->options['widgetFromDt'];
