@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (C) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -36,6 +36,7 @@ use Xibo\Helper\ApplicationState;
 use Xibo\Helper\DateFormatHelper;
 use Xibo\Helper\SanitizerService;
 use Xibo\Helper\Translate;
+use Xibo\Support\Exception\GeneralException;
 use Xibo\Support\Exception\InvalidArgumentException;
 use Xibo\Support\Exception\NotFoundException;
 use Xibo\Support\Sanitizer\SanitizerInterface;
@@ -154,8 +155,8 @@ class ProofOfPlay implements ReportInterface
         $filterCriteria = [
             'filter' => $filter,
             'displayId' => $sanitizedParams->getInt('displayId'),
-            'layoutIds' => $sanitizedParams->getIntArray('layoutId'),
-            'mediaIds' => $sanitizedParams->getIntArray('mediaId'),
+            'layoutId' => $sanitizedParams->getIntArray('layoutId'),
+            'mediaId' => $sanitizedParams->getIntArray('mediaId'),
             'type' => $sanitizedParams->getString('type'),
             'sortBy' => $sanitizedParams->getString('sortBy'),
             'tagsType' => $sanitizedParams->getString('tagsType'),
@@ -315,7 +316,6 @@ class ProofOfPlay implements ReportInterface
     /** @inheritdoc */
     public function getResults(SanitizerInterface $sanitizedParams)
     {
-        $displayId = $sanitizedParams->getInt('displayId');
         $layoutIds = $sanitizedParams->getIntArray('layoutId', ['default' => []]);
         $mediaIds = $sanitizedParams->getIntArray('mediaId', ['default' => []]);
         $type = strtolower($sanitizedParams->getString('type'));
@@ -323,26 +323,15 @@ class ProofOfPlay implements ReportInterface
         $tagsType = $sanitizedParams->getString('tagsType');
         $exactTags = $sanitizedParams->getCheckbox('exactTags');
         $operator = $sanitizedParams->getString('logicalOperator', ['default' => 'OR']);
-
-        // Do not filter by display if super admin and no display is selected
-        // Super admin will be able to see stat records of deleted display, we will not filter by display later
-        $displayIds = [];
-
-        // Get user
-        $user = $this->getUser();
+        $parentCampaignId = $sanitizedParams->getInt('parentCampaignId');
 
         // Display filter.
-        if (!$user->isSuperAdmin()) {
+        try {
             // Get an array of display id this user has access to.
             $displayIds = $this->getDisplayIdFilter($sanitizedParams);
-
-            // If we don't have permission to any displays, then filter by one we know not to exist.
-            if (count($displayIds) <= 0) {
-                $displayIds = [-1];
-            }
-        } else if ($displayId !== null) {
-            // Super admin, so only filter if we've selected one.
-            $displayIds[] = $displayId;
+        } catch (GeneralException $exception) {
+            // stop the query
+            return new ReportResult();
         }
 
         // web
@@ -354,16 +343,8 @@ class ProofOfPlay implements ReportInterface
             if (is_array($sortOrder)) {
                 $columns = $sortOrder;
             }
-
-            // Paging
-            $start = intval($sanitizedParams->getInt('start'), 0);
-            $length = $sanitizedParams->getInt('length', ['default' => 10]);
         } else {
-            // xtr
-            $start = 0;
-            $length = -1;
             $sortBy = $sanitizedParams->getString('sortBy');
-
             $columns = ($sortBy == '') ? ['widgetId'] : [$sortBy];
         }
 
@@ -456,6 +437,7 @@ class ProofOfPlay implements ReportInterface
                 $fromDt,
                 $toDt,
                 $displayIds,
+                $parentCampaignId,
                 $layoutIds,
                 $mediaIds,
                 $type,
@@ -463,15 +445,14 @@ class ProofOfPlay implements ReportInterface
                 $tags,
                 $tagsType,
                 $exactTags,
-                $operator,
-                $start,
-                $length
+                $operator
             );
         } else {
             $result = $this->getProofOfPlayReportMySql(
                 $fromDt,
                 $toDt,
                 $displayIds,
+                $parentCampaignId,
                 $layoutIds,
                 $mediaIds,
                 $type,
@@ -479,9 +460,7 @@ class ProofOfPlay implements ReportInterface
                 $tags,
                 $tagsType,
                 $exactTags,
-                $operator,
-                $start,
-                $length
+                $operator
             );
         }
 
@@ -497,12 +476,15 @@ class ProofOfPlay implements ReportInterface
             $widgetName = ($widgetName == '' &&  $widgetId != 0) ? __('Deleted from Layout') : $widgetName;
             $displayName = $sanitizedRow->getString('display');
             $layoutName = $sanitizedRow->getString('layout');
+            $parentCampaignName = $sanitizedRow->getString('parentCampaign');
 
             $entry['type'] = $sanitizedRow->getString('type');
             $entry['displayId'] = $sanitizedRow->getInt('displayId');
             $entry['display'] = ($displayName != '') ? $displayName : __('Not Found');
             $entry['layoutId'] = $sanitizedRow->getInt('layoutId');
             $entry['layout'] = ($layoutName != '') ? $layoutName :  __('Not Found');
+            $entry['parentCampaignId'] = $sanitizedRow->getInt('parentCampaignId');
+            $entry['parentCampaign'] = $parentCampaignName;
             $entry['widgetId'] = $sanitizedRow->getInt('widgetId');
             $entry['media'] = $widgetName;
             $entry['tag'] = $sanitizedRow->getString('tag');
@@ -512,7 +494,6 @@ class ProofOfPlay implements ReportInterface
             $entry['maxEnd'] =  Carbon::createFromTimestamp($row['maxEnd'])->format(DateFormatHelper::getSystemFormat());
             $entry['mediaId'] = $sanitizedRow->getInt('mediaId');
 
-
             $rows[] = $entry;
         }
 
@@ -522,11 +503,7 @@ class ProofOfPlay implements ReportInterface
             'periodEnd' => $result['periodEnd'],
         ];
 
-        // Paging
-        $recordsTotal = 0;
-        if ($result['count'] > 0) {
-            $recordsTotal = intval($result['totalStats']);
-        }
+        $recordsTotal = $result['count'];
 
         // ----
         // Table Only
@@ -544,6 +521,7 @@ class ProofOfPlay implements ReportInterface
      * @param Carbon $fromDt The filter range from date
      * @param Carbon $toDt The filter range to date
      * @param $displayIds array
+     * @param $parentCampaignId int
      * @param $layoutIds array
      * @param $mediaIds array
      * @param $type string
@@ -551,14 +529,13 @@ class ProofOfPlay implements ReportInterface
      * @param $tags string
      * @param $tagsType string
      * @param $exactTags mixed
-     * @param $start int
-     * @param $length int
      * @return array[array result, date periodStart, date periodEnd, int count, int totalStats]
      */
     private function getProofOfPlayReportMySql(
         $fromDt,
         $toDt,
         $displayIds,
+        $parentCampaignId,
         $layoutIds,
         $mediaIds,
         $type,
@@ -566,9 +543,7 @@ class ProofOfPlay implements ReportInterface
         $tags,
         $tagsType,
         $exactTags,
-        $logicalOperator,
-        $start,
-        $length
+        $logicalOperator
     ) {
         $fromDt = $fromDt->format('U');
         $toDt = $toDt->format('U');
@@ -577,6 +552,8 @@ class ProofOfPlay implements ReportInterface
         $select = '
           SELECT stat.type,
               display.Display,
+              stat.parentCampaignId,
+              campaign.campaign as parentCampaign,
               IFNULL(layout.Layout, 
                   (SELECT MAX(`layout`) AS layout 
                      FROM `layout` 
@@ -612,6 +589,8 @@ class ProofOfPlay implements ReportInterface
                 AND `widgetoption`.option = \'name\'
               LEFT OUTER JOIN `media`
               ON `media`.mediaId = `stat`.mediaId
+              LEFT OUTER JOIN `campaign`
+              ON `campaign`.campaignId = `stat`.parentCampaignId
               ';
 
         if ($tags != '') {
@@ -683,65 +662,83 @@ class ProofOfPlay implements ReportInterface
                 $lkTagTable = '';
                 $lkTagTableIdColumn = '';
                 $idColumn = '';
+                $allTags = explode(',', $tags);
+                $excludeTags = [];
+                $includeTags = [];
+
+                foreach ($allTags as $tag) {
+                    if (str_starts_with($tag, '-')) {
+                        $excludeTags[] = ltrim(($tag), '-');
+                    } else {
+                        $includeTags[] = $tag;
+                    }
+                }
 
                 if ($tagsType === 'dg') {
-                    $body .= ' AND `displaygroup`.displaygroupId IN (
-                        SELECT `lktagdisplaygroup`.displaygroupId
-                          FROM tag
-                            INNER JOIN `lktagdisplaygroup`
-                            ON `lktagdisplaygroup`.tagId = tag.tagId
-                ';
                     $lkTagTable = 'lktagdisplaygroup';
                     $lkTagTableIdColumn = 'lkTagDisplayGroupId';
                     $idColumn = 'displayGroupId';
                 }
-                // old layout and latest layout have same tags
-                // old layoutId replaced with latest layoutId in the lktaglayout table and
-                // join with layout history to get campaignId then we can show old layouts that have given tag
-                if ($tagsType === 'layout') {
-                    $body .= ' AND `stat`.campaignId IN (
-                        SELECT 
-                            `layouthistory`.campaignId
-                        FROM
-                        (
-                            SELECT `lktaglayout`.layoutId
-                            FROM tag
-                            INNER JOIN `lktaglayout`
-                            ON `lktaglayout`.tagId = tag.tagId
-                        ';
 
+                if ($tagsType === 'layout') {
                     $lkTagTable = 'lktaglayout';
                     $lkTagTableIdColumn = 'lkTagLayoutId';
                     $idColumn = 'layoutId';
                 }
                 if ($tagsType === 'media') {
-                    $body .= ' AND `media`.mediaId IN (
-                        SELECT `lktagmedia`.mediaId
-                          FROM tag
-                            INNER JOIN `lktagmedia`
-                            ON `lktagmedia`.tagId = tag.tagId
-                ';
                     $lkTagTable = 'lktagmedia';
                     $lkTagTableIdColumn = 'lkTagMediaId';
                     $idColumn = 'mediaId';
                 }
-                $tagsFilter = explode(',', $tags);
-                // pass to BaseFactory tagFilter, it does not matter from which factory we do that.
-                $this->layoutFactory->tagFilter(
-                    $tagsFilter,
-                    $lkTagTable,
-                    $lkTagTableIdColumn,
-                    $idColumn,
-                    $logicalOperator,
-                    $operator,
-                    $body,
-                    $params
-                );
 
-                if ($tagsType === 'layout') {
-                    $body .= ' B
+                if (!empty($excludeTags)) {
+                    $body .= $this->getBodyForTagsType($tagsType, true);
+                    // pass to BaseFactory tagFilter, it does not matter from which factory we do that.
+                    $this->layoutFactory->tagFilter(
+                        $excludeTags,
+                        $lkTagTable,
+                        $lkTagTableIdColumn,
+                        $idColumn,
+                        $logicalOperator,
+                        $operator,
+                        true,
+                        $body,
+                        $params
+                    );
+
+                    // old layout and latest layout have same tags
+                    // old layoutId replaced with latest layoutId in the lktaglayout table and
+                    // join with layout history to get campaignId then we can show old layouts that have given tag
+                    if ($tagsType === 'layout') {
+                        $body .= ' B
                         LEFT OUTER JOIN
                         `layouthistory` ON `layouthistory`.layoutId = B.layoutId ) ';
+                    }
+                }
+
+                if (!empty($includeTags)) {
+                    $body .= $this->getBodyForTagsType($tagsType, false);
+                    // pass to BaseFactory tagFilter, it does not matter from which factory we do that.
+                    $this->layoutFactory->tagFilter(
+                        $includeTags,
+                        $lkTagTable,
+                        $lkTagTableIdColumn,
+                        $idColumn,
+                        $logicalOperator,
+                        $operator,
+                        false,
+                        $body,
+                        $params
+                    );
+
+                    // old layout and latest layout have same tags
+                    // old layoutId replaced with latest layoutId in the lktaglayout table and
+                    // join with layout history to get campaignId then we can show old layouts that have given tag
+                    if ($tagsType === 'layout') {
+                        $body .= ' C
+                        LEFT OUTER JOIN
+                        `layouthistory` ON `layouthistory`.layoutId = C.layoutId ) ';
+                    }
                 }
             }
         }
@@ -757,6 +754,12 @@ class ProofOfPlay implements ReportInterface
             $body .= ' AND `stat`.type = \'event\' ';
         }
 
+        // Campaign Filter
+        if ($parentCampaignId != null) {
+            $body .= ' AND `stat`.parentCampaignId = :parentCampaignId ';
+            $params['parentCampaignId'] = $parentCampaignId;
+        }
+
         // Layout Filter
         if (count($layoutIds) != 0) {
             $layoutSql = '';
@@ -788,6 +791,7 @@ class ProofOfPlay implements ReportInterface
             GROUP BY stat.type, 
                 stat.tag, 
                 display.Display, 
+                stat.parentCampaignId,
                 stat.displayId, 
                 stat.campaignId,
                 layout.layout, 
@@ -805,13 +809,8 @@ class ProofOfPlay implements ReportInterface
             $order = 'ORDER BY ' . implode(',', $columns);
         }
 
-        $limit= '';
-        if (($length != null) && ($length != -1)) {
-            $limit = ' LIMIT ' . $start . ', ' . $length;
-        }
-
         /*Execute sql statement*/
-        $sql = $select . $body . $order . $limit;
+        $sql = $select . $body . $order;
 
         $rows = [];
         foreach ($this->store->select($sql, $params) as $row) {
@@ -821,6 +820,8 @@ class ProofOfPlay implements ReportInterface
             $entry['displayId'] = $row['displayId'];
             $entry['display'] = $row['Display'];
             $entry['layout'] = $row['Layout'];
+            $entry['parentCampaignId'] = $row['parentCampaignId'];
+            $entry['parentCampaign'] = $row['parentCampaign'];
             $entry['media'] = $row['Media'];
             $entry['numberPlays'] = $row['NumberPlays'];
             $entry['duration'] = $row['Duration'];
@@ -834,28 +835,53 @@ class ProofOfPlay implements ReportInterface
             $rows[] = $entry;
         }
 
-        // Paging
-        $results = [];
-        if ($limit != '' && count($rows) > 0) {
-            $results = $this->store->select('
-              SELECT COUNT(*) AS total FROM (SELECT stat.type, display.Display, layout.Layout, IFNULL(`media`.name, IFNULL(`widgetoption`.value, `widget`.type)) ' . $body . ') total
-            ', $params);
-        }
-
         return [
-            'result' => $rows,
             'periodStart' => Carbon::createFromTimestamp($fromDt)->format(DateFormatHelper::getSystemFormat()),
             'periodEnd' => Carbon::createFromTimestamp($toDt)->format(DateFormatHelper::getSystemFormat()),
-            'count' => count($rows),
-            'totalStats' => $results[0]['total'] ?? 0,
+            'result' => $rows,
+            'count' => count($rows)
         ];
     }
 
+    private function getBodyForTagsType($tagsType, $exclude) :string
+    {
+        if ($tagsType === 'dg') {
+            return ' AND `displaygroup`.displaygroupId ' . ($exclude ? 'NOT' : '') .  ' IN (
+                        SELECT `lktagdisplaygroup`.displaygroupId
+                          FROM tag
+                            INNER JOIN `lktagdisplaygroup`
+                            ON `lktagdisplaygroup`.tagId = tag.tagId
+                ';
+        } else if ($tagsType === 'media') {
+            return ' AND `media`.mediaId '. ($exclude ? 'NOT' : '') . ' IN (
+                        SELECT `lktagmedia`.mediaId
+                          FROM tag
+                            INNER JOIN `lktagmedia`
+                            ON `lktagmedia`.tagId = tag.tagId
+                ';
+        } else if ($tagsType === 'layout') {
+            return ' AND `stat`.campaignId ' . ($exclude ? 'NOT' : '') . ' IN (
+                        SELECT 
+                            `layouthistory`.campaignId
+                        FROM
+                        (
+                            SELECT `lktaglayout`.layoutId
+                            FROM tag
+                            INNER JOIN `lktaglayout`
+                            ON `lktaglayout`.tagId = tag.tagId
+                        ';
+        } else {
+            $this->getLog()->error(__('Incorrect Tag type selected'));
+            return '';
+        }
+    }
+
     /**
      * MongoDB proof of play report
      * @param Carbon $filterFromDt The filter range from date
      * @param Carbon $filterToDt The filter range to date
      * @param $displayIds array
+     * @param $parentCampaignId int
      * @param $layoutIds array
      * @param $mediaIds array
      * @param $type string
@@ -863,8 +889,6 @@ class ProofOfPlay implements ReportInterface
      * @param $tags string
      * @param $tagsType string
      * @param $exactTags mixed
-     * @param $start int
-     * @param $length int
      * @return array[array result, date periodStart, date periodEnd, int count, int totalStats]
      * @throws InvalidArgumentException
      * @throws \Xibo\Support\Exception\GeneralException
@@ -873,6 +897,7 @@ class ProofOfPlay implements ReportInterface
         $filterFromDt,
         $filterToDt,
         $displayIds,
+        $parentCampaignId,
         $layoutIds,
         $mediaIds,
         $type,
@@ -880,9 +905,7 @@ class ProofOfPlay implements ReportInterface
         $tags,
         $tagsType,
         $exactTags,
-        $operator,
-        $start,
-        $length
+        $operator
     ) {
         $fromDt = new UTCDateTime($filterFromDt->format('U')*1000);
         $toDt = new UTCDateTime($filterToDt->format('U')*1000);
@@ -946,6 +969,11 @@ class ProofOfPlay implements ReportInterface
             }
         }
 
+        // Campaign Filter
+        if ($parentCampaignId != null) {
+            $match['$match']['parentCampaignId'] = $parentCampaignId;
+        }
+
         // Layout Filter
         if (count($layoutIds) != 0) {
             // Get campaignIds for selected layoutIds
@@ -976,6 +1004,8 @@ class ProofOfPlay implements ReportInterface
             'media' => 'media',
             'eventName' => 'eventName',
             'layoutId' => 'layoutId',
+            'parentCampaignId' => 'parentCampaignId',
+            'parentCampaign' => 'parentCampaign',
             'widgetId' => 'widgetId',
             '_id.displayId' => 'displayId',
             'numberPlays' => 'numberPlays',
@@ -1019,6 +1049,8 @@ class ProofOfPlay implements ReportInterface
                 'widgetName' =>  1,
                 'layoutId' =>  1,
                 'layoutName' =>  1,
+                'parentCampaignId' =>  1,
+                'parentCampaign' =>  1,
                 'displayId' =>  1,
                 'displayName' =>  1,
                 'start' => 1,
@@ -1034,6 +1066,7 @@ class ProofOfPlay implements ReportInterface
             '$group' => [
                 '_id' => [
                     'type' => '$type',
+                    'parentCampaignId'=> '$parentCampaignId',
                     'campaignId'=> [ '$ifNull' => [ '$campaignId', '$layoutId' ] ],
                     'mediaorwidget'=> [ '$ifNull' => [ '$mediaId', '$widgetId' ] ],
                     'displayId'=> [ '$ifNull' => [ '$displayId', null ] ],
@@ -1053,6 +1086,9 @@ class ProofOfPlay implements ReportInterface
                 // use the last layoutId to say that is the latest layoutId
                 'layoutId' => ['$last' => '$layoutId'],
 
+                'parentCampaign' => ['$first' => '$parentCampaign'],
+                'parentCampaignId' => ['$first' => '$parentCampaignId'],
+
                 'minStart' => ['$min' => '$start'],
                 'maxEnd' => ['$max' => '$end'],
                 'numberPlays' => ['$sum' => '$count'],
@@ -1061,62 +1097,23 @@ class ProofOfPlay implements ReportInterface
             ],
         ];
 
-        // Task run
-        if ($length == -1) {
-            $query = [
-                $match,
-                $project,
-                $group, [
-                    '$facet' => [
-                        'totalData' => [
-                            ['$sort' => $order],
-                        ]
+        $query = [
+            $match,
+            $project,
+            $group, [
+                '$facet' => [
+                    'totalData' => [
+                        ['$sort' => $order],
                     ]
-                ],
+                ]
+            ],
 
-            ];
-        } else { // Frontend
-            $query = [
-                $match,
-                $project,
-                $group,
-                [
-                    '$facet' => [
-                        'totalData' => [
-
-                            ['$skip' => $start],
-                            ['$limit' => $length],
-                            ['$sort' => $order],
-                        ],
-                        'totalCount' => [
-                            [
-                                '$group' => [
-                                    '_id' => [],
-                                    'totals' => ['$sum' => '$total'],
-
-                                ],
-                            ]
-                        ]
-                    ]
-                ],
-            ];
-        }
+        ];
 
         $result = $this->getTimeSeriesStore()->executeQuery(['collection' => $this->table, 'query' => $query]);
 
-        $totalStats = 0;
         $rows = [];
         if (count($result) > 0) {
-            if ($length == -1) { // Task run
-                $totalCount = [];
-            } else { // Get total for pagination in UI (grid)
-                $totalCount = $result[0]['totalCount'];
-            }
-
-            if (count($totalCount) > 0) {
-                $totalStats = $totalCount[0]['totals'];
-            }
-
             // Grid results
             foreach ($result[0]['totalData'] as $row) {
                 $entry = [];
@@ -1125,6 +1122,8 @@ class ProofOfPlay implements ReportInterface
                 $entry['displayId'] = $row['_id']['displayId'];
                 $entry['display'] = isset($row['_id']['display']) ? $row['_id']['display']: 'No display';
                 $entry['layout'] = isset($row['layout']) ? $row['layout']: 'No layout';
+                $entry['parentCampaignId'] = isset($row['parentCampaignId']) ? $row['parentCampaignId']: '';
+                $entry['parentCampaign'] = isset($row['parentCampaign']) ? $row['parentCampaign']: '';
                 $entry['media'] = isset($row['media']) ? $row['media'] : 'No media' ;
                 $entry['numberPlays'] = $row['numberPlays'];
                 $entry['duration'] = $row['duration'];
@@ -1140,11 +1139,10 @@ class ProofOfPlay implements ReportInterface
         }
 
         return [
-            'result' => $rows,
             'periodStart' => $filterFromDt->format(DateFormatHelper::getSystemFormat()),
             'periodEnd' => $filterToDt->format(DateFormatHelper::getSystemFormat()),
-            'count' => count($rows),
-            'totalStats' => $totalStats,
+            'result' => $rows,
+            'count' => count($rows)
         ];
     }
 }
