@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2022 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - http://www.xibo.org.uk
  *
@@ -120,11 +120,28 @@ class MySqlTimeSeriesStore implements TimeSeriesStoreInterface
     /** @inheritdoc */
     public function addStatFinalize()
     {
-
         if (count($this->stats) > 0) {
+            $sql = '
+                INSERT INTO `stat` (
+                    `type`,
+                    `statDate`,
+                    `start`,
+                    `end`,
+                    `scheduleID`,
+                    `displayID`,
+                    `campaignID`,
+                    `layoutID`,
+                    `mediaID`,
+                    `tag`,
+                    `widgetId`,
+                    `duration`,
+                    `count`,
+                    `engagements`,
+                    `parentCampaignId`
+                ) 
+                VALUES ';
 
-            $sql = 'INSERT INTO `stat` (`type`, statDate, start, `end`, scheduleID, displayID, campaignID, layoutID, mediaID, Tag, `widgetId`, duration, `count`, `engagements`) VALUES ';
-            $placeHolders = '(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)';
+            $placeHolders = '(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)';
 
             $sql = $sql . implode(', ', array_fill(1, count($this->stats), $placeHolders));
 
@@ -146,7 +163,8 @@ class MySqlTimeSeriesStore implements TimeSeriesStoreInterface
                     'widgetId' => $stat['widgetId'],
                     'duration' => $stat['duration'],
                     'count' => $stat['count'],
-                    'engagements' => $stat['engagements']
+                    'engagements' => $stat['engagements'],
+                    'parentCampaignId' => $stat['parentCampaignId'],
                 ];
 
                 // Add each value to another array in order
@@ -171,12 +189,12 @@ class MySqlTimeSeriesStore implements TimeSeriesStoreInterface
     }
 
     /** @inheritdoc */
-    public function getStats($filterBy = [])
+    public function getStats($filterBy = [], $isBufferedQuery = false)
     {
-        $fromDt = isset($filterBy['fromDt']) ? $filterBy['fromDt'] : null;
-        $toDt = isset($filterBy['toDt']) ? $filterBy['toDt'] : null;
-        $statDate = isset($filterBy['statDate']) ? $filterBy['statDate'] : null;
-        $statDateLessThan = isset($filterBy['statDateLessThan']) ? $filterBy['statDateLessThan'] : null;
+        $fromDt = $filterBy['fromDt'] ?? null;
+        $toDt = $filterBy['toDt'] ?? null;
+        $statDate = $filterBy['statDate'] ?? null;
+        $statDateLessThan = $filterBy['statDateLessThan'] ?? null;
 
         // In the case of user switches from  mongo to mysql - laststatId were saved as Mongo ObjectId string
         if (isset($filterBy['statId'])) {
@@ -189,21 +207,23 @@ class MySqlTimeSeriesStore implements TimeSeriesStoreInterface
             $statId = null;
         }
 
-        $type = isset($filterBy['type']) ? $filterBy['type'] : null;
-        $displayIds = isset($filterBy['displayIds']) ? $filterBy['displayIds'] : [];
-        $layoutIds = isset($filterBy['layoutIds']) ? $filterBy['layoutIds'] : [];
-        $mediaIds = isset($filterBy['mediaIds']) ? $filterBy['mediaIds'] : [];
-        $campaignId = isset($filterBy['campaignId']) ? $filterBy['campaignId'] : null;
-        $eventTag = isset($filterBy['eventTag']) ? $filterBy['eventTag'] : null;
+        $type = $filterBy['type'] ?? null;
+        $displayIds = $filterBy['displayIds'] ?? [];
+        $layoutIds = $filterBy['layoutIds'] ?? [];
+        $mediaIds = $filterBy['mediaIds'] ?? [];
+        $campaignId = $filterBy['campaignId'] ?? null;
+        $parentCampaignId = $filterBy['parentCampaignId'] ?? null;
+        $mustHaveParentCampaign = $filterBy['mustHaveParentCampaign'] ?? false;
+        $eventTag = $filterBy['eventTag'] ?? null;
 
         // Tag embedding
-        $embedDisplayTags = isset($filterBy['displayTags']) ? $filterBy['displayTags'] : false;
-        $embedLayoutTags = isset($filterBy['layoutTags']) ? $filterBy['layoutTags'] : false;
-        $embedMediaTags = isset($filterBy['mediaTags']) ? $filterBy['mediaTags'] : false;
+        $embedDisplayTags = $filterBy['displayTags'] ?? false;
+        $embedLayoutTags = $filterBy['layoutTags'] ?? false;
+        $embedMediaTags = $filterBy['mediaTags'] ?? false;
 
         // Limit
-        $start = isset($filterBy['start']) ? $filterBy['start'] : null;
-        $length = isset($filterBy['length']) ? $filterBy['length'] : null;
+        $start = $filterBy['start'] ?? null;
+        $length = $filterBy['length'] ?? null;
 
         $params = [];
         $select = 'SELECT stat.statId, 
@@ -214,6 +234,7 @@ class MySqlTimeSeriesStore implements TimeSeriesStoreInterface
             stat.layoutId, 
             stat.mediaId, 
             stat.campaignId, 
+            stat.parentCampaignId, 
             stat.start as start, 
             stat.end as end, 
             stat.tag, 
@@ -222,6 +243,7 @@ class MySqlTimeSeriesStore implements TimeSeriesStoreInterface
             stat.engagements, 
             display.Display as display, 
             layout.Layout as layout, 
+            campaign.campaign as parentCampaign, 
             media.Name AS media ';
 
         if ($embedDisplayTags) {
@@ -274,6 +296,8 @@ class MySqlTimeSeriesStore implements TimeSeriesStoreInterface
             ON stat.DisplayID = display.DisplayID
             LEFT OUTER JOIN layout
             ON layout.LayoutID = stat.LayoutID
+            LEFT OUTER JOIN campaign
+            ON campaign.campaignID = stat.parentCampaignID
             LEFT OUTER JOIN media
             ON media.mediaID = stat.mediaID
             LEFT OUTER JOIN widget
@@ -350,6 +374,17 @@ class MySqlTimeSeriesStore implements TimeSeriesStoreInterface
             $body .= ' AND `media`.mediaId IN (' . trim($mediaSql, ',') . ')';
         }
 
+        // Parent Campaign Filter
+        if ($parentCampaignId != null) {
+            $body .= ' AND `stat`.parentCampaignId = :parentCampaignId ';
+            $params['parentCampaignId'] = $parentCampaignId;
+        }
+
+        // Has Parent Campaign Filter
+        if ($mustHaveParentCampaign) {
+            $body .= ' AND IFNULL(`stat`.parentCampaignId, 0) != 0 ';
+        }
+
         // Campaign
         // --------
         // Filter on Layouts linked to a Campaign
@@ -391,7 +426,8 @@ class MySqlTimeSeriesStore implements TimeSeriesStoreInterface
 
         // Run our query using a connection object (to save memory)
         $connection = $this->store->getConnection();
-        $connection->setAttribute(\PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, false);
+        $connection->setAttribute(\PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, $isBufferedQuery);
+
 
         // Prepare the statement
         $statement = $connection->prepare($sql);
