@@ -2,7 +2,7 @@
 /*
  * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -37,10 +37,10 @@ use Xibo\Factory\UserGroupFactory;
 use Xibo\Helper\ByteFormatter;
 use Xibo\Helper\DateFormatHelper;
 use Xibo\Helper\Profiler;
+use Xibo\Helper\Status;
 use Xibo\Helper\WakeOnLan;
 use Xibo\Service\MediaServiceInterface;
 use Xibo\Support\Exception\GeneralException;
-use Xibo\Widget\ModuleWidget;
 
 /**
  * Class MaintenanceRegularTask
@@ -358,14 +358,13 @@ class MaintenanceRegularTask implements TaskInterface
               ON display.displayId = requiredfile.displayId
            WHERE `bytesRequested` > 0
               AND `requiredfile`.bytesRequested >= `requiredfile`.`size` * :factor
-              AND `requiredfile`.type <> :excludedType
+              AND `requiredfile`.type NOT IN (\'W\', \'D\')
               AND display.lastAccessed > :lastAccessed
               AND `requiredfile`.complete = 0
             GROUP BY display.displayId, display.display
         ', [
             'factor' => 3,
-            'excludedType' => 'W',
-            'lastAccessed' => Carbon::now()->subDay()->format('U')
+            'lastAccessed' => Carbon::now()->subDay()->format('U'),
         ]);
 
         foreach ($items as $item) {
@@ -440,7 +439,7 @@ class MaintenanceRegularTask implements TaskInterface
                         $layout = $this->layoutFactory->concurrentRequestLock($layout, true);
                         try {
                             $draft = $this->layoutFactory->getByParentId($layout->layoutId);
-                            if ($draft->status === ModuleWidget::$STATUS_INVALID
+                            if ($draft->status === Status::$STATUS_INVALID
                                 && isset($draft->statusMessage)
                                 && (count($draft->getStatusMessage()) > 1 || count($draft->getStatusMessage()) === 1 && !$draft->checkForEmptyRegion())
                             ) {
@@ -557,3 +556,4 @@ class MaintenanceRegularTask implements TaskInterface
         $this->runMessage .= ' - Done ' . $count . PHP_EOL . PHP_EOL;
     }
 }
+
