@@ -1,6 +1,6 @@
 <?php
 /*
- * Copyright (C) 2024 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
  * Xibo - Digital Signage - https://xibosignage.com
  *
@@ -23,7 +23,6 @@
 
 namespace Xibo\Xmds;
 
-
 use Carbon\Carbon;
 use Illuminate\Support\Str;
 use Stash\Invalidation;
@@ -37,6 +36,8 @@ use Xibo\Support\Exception\NotFoundException;
 /**
  * Class Soap5
  * @package Xibo\Xmds
+ *
+ * @phpcs:disable PSR1.Methods.CamelCapsMethodName.NotCamelCaps
  */
 class Soap5 extends Soap4
 {
@@ -52,13 +53,25 @@ class Soap5 extends Soap4
      * @param string $macAddress
      * @param string $xmrChannel
      * @param string $xmrPubKey
+     * @param string $licenceCheck
      * @return string
      * @throws NotFoundException
      * @throws \SoapFault
      * @throws GeneralException
      */
-    public function RegisterDisplay($serverKey, $hardwareKey, $displayName, $clientType, $clientVersion, $clientCode, $operatingSystem, $macAddress, $xmrChannel = null, $xmrPubKey = null)
-    {
+    public function RegisterDisplay(
+        $serverKey,
+        $hardwareKey,
+        $displayName,
+        $clientType,
+        $clientVersion,
+        $clientCode,
+        $operatingSystem,
+        $macAddress,
+        $xmrChannel = null,
+        $xmrPubKey = null,
+        $licenceResult = null
+    ) {
         $this->logProcessor->setRoute('RegisterDisplay');
 
         $sanitized = $this->getSanitizer([
@@ -71,7 +84,8 @@ class Soap5 extends Soap4
             'operatingSystem' => $operatingSystem,
             'macAddress' => $macAddress,
             'xmrChannel' => $xmrChannel,
-            'xmrPubKey' => $xmrPubKey
+            'xmrPubKey' => $xmrPubKey,
+            'licenceResult' => $licenceResult
         ]);
 
         // Sanitize
@@ -85,6 +99,8 @@ class Soap5 extends Soap4
         $clientAddress = $this->getIp();
         $xmrChannel = $sanitized->getString('xmrChannel');
         $xmrPubKey = trim($sanitized->getString('xmrPubKey'));
+        // this is only sent from xmds v7
+        $commercialLicenceString = $sanitized->getString('licenceResult');
 
         if ($xmrPubKey != '' && !Str::contains($xmrPubKey, 'BEGIN PUBLIC KEY')) {
             $xmrPubKey = "-----BEGIN PUBLIC KEY-----\n" . $xmrPubKey . "\n-----END PUBLIC KEY-----\n";
@@ -131,7 +147,6 @@ class Soap5 extends Soap4
                 $displayElement->setAttribute('status', 2);
                 $displayElement->setAttribute('code', 'WAITING');
                 $displayElement->setAttribute('message', 'Display is Registered and awaiting Authorisation from an Administrator in the CMS');
-
             } else {
                 // It is licensed
                 $displayElement->setAttribute('status', 0);
@@ -182,21 +197,21 @@ class Soap5 extends Soap4
                 // Player upgrades
                 $version = '';
                 try {
-                    $upgradeMediaId = $this->display->getSetting('versionMediaId', null, ['displayOverride' => true]);
+                    $versionId = $this->display->getSetting('versionMediaId', null, ['displayOverride' => true]);
 
-                    if ($clientType != 'windows' && $upgradeMediaId != null) {
-                        $version = $this->playerVersionFactory->getByMediaId($upgradeMediaId);
+                    if ($clientType != 'windows' && $versionId != null) {
+                        $version = $this->playerVersionFactory->getById($versionId);
 
                         if ($clientType == 'android') {
                             $version = json_encode([
-                                'id' => $upgradeMediaId,
-                                'file' => $version->storedAs,
+                                'id' => $versionId,
+                                'file' => $version->fileName,
                                 'code' => $version->code
                             ]);
                         } elseif ($clientType == 'lg') {
                             $version = json_encode([
-                                'id' => $upgradeMediaId,
-                                'file' => $version->storedAs,
+                                'id' => $versionId,
+                                'file' => $version->fileName,
                                 'code' => $version->code
                             ]);
                         } elseif ($clientType == 'sssp') {
@@ -208,8 +223,8 @@ class Soap5 extends Soap4
                             $this->getPool()->saveDeferred($cache);
 
                             $version = json_encode([
-                                'id' => $upgradeMediaId,
-                                'file' => $version->storedAs,
+                                'id' => $versionId,
+                                'file' => $version->fileName,
                                 'code' => $version->code,
                                 'url' => str_replace('/xmds.php', '', Wsdl::getRoot()) . '/playersoftware/' . $nonce
                             ]);
@@ -264,9 +279,11 @@ class Soap5 extends Soap4
 
                 // Adspace Enabled CMS?
                 $isAdspaceEnabled = intval($this->getConfig()->getSetting('isAdspaceEnabled', 0));
-                $node = $return->createElement('isAdspaceEnabled', $isAdspaceEnabled);
-                $node->setAttribute('type', 'checkbox');
-                $displayElement->appendChild($node);
+                if ($isAdspaceEnabled === 1) {
+                    $node = $return->createElement('isAdspaceEnabled', 1);
+                    $node->setAttribute('type', 'checkbox');
+                    $displayElement->appendChild($node);
+                }
 
                 if (!empty($display->timeZone)) {
                     // Calculate local time
@@ -310,6 +327,32 @@ class Soap5 extends Soap4
                     }
                 }
 
+                // Tags
+                if (count($display->tags) > 0) {
+                    $tagsElement = $return->createElement('tags');
+                    $displayElement->appendChild($tagsElement);
+
+                    foreach ($display->tags as $tagLink) {
+                        $tagNode = $return->createElement('tag');
+
+                        $tagNameNode = $return->createElement('tagName');
+                        $tag = $return->createTextNode($tagLink->tag);
+                        $tagNameNode->appendChild($tag);
+
+                        $tagNode->appendChild($tagNameNode);
+
+                        if ($tagLink->value !== null) {
+                            $valueNode = $return->createElement('tagValue');
+                            $value = $return->createTextNode($tagLink->value);
+                            $valueNode->appendChild($value);
+
+                            $tagNode->appendChild($valueNode);
+                        }
+
+                        $tagsElement->appendChild($tagNode);
+                    }
+                }
+
                 // Check to see if the channel/pubKey are already entered
                 if ($display->isAuditing()) {
                     $this->getLog()->debug('xmrChannel: [' . $xmrChannel . ']. xmrPublicKey: [' . $xmrPubKey . ']');
@@ -318,19 +361,20 @@ class Soap5 extends Soap4
                 // Update the Channel
                 $display->xmrChannel = $xmrChannel;
                 // Update the PUB Key only if it has been cleared
-                if ($display->xmrPubKey == '')
+                if ($display->xmrPubKey == '') {
                     $display->xmrPubKey = $xmrPubKey;
+                }
             }
-
         } catch (NotFoundException $e) {
-
             // Add a new display
             try {
                 $display = $this->displayFactory->createEmpty();
                 $this->display = $display;
                 $display->display = $displayName;
                 $display->auditingUntil = 0;
-                $display->defaultLayoutId = $this->getConfig()->getSetting('DEFAULT_LAYOUT');
+                // defaultLayoutId column cannot be null or empty string
+                // if we do not have global default layout, set it here to 0 to allow register to proceed
+                $display->defaultLayoutId = intval($this->getConfig()->getSetting('DEFAULT_LAYOUT', 0));
                 $display->license = $hardwareKey;
                 $display->licensed = $this->getConfig()->getSetting('DISPLAY_AUTO_AUTH', 0);
                 $display->incSchedule = 0;
@@ -341,17 +385,20 @@ class Soap5 extends Soap4
                 if (!$display->isDisplaySlotAvailable()) {
                     $display->licensed = 0;
                 }
-            }
-            catch (\InvalidArgumentException $e) {
+            } catch (\InvalidArgumentException $e) {
                 throw new \SoapFault('Sender', $e->getMessage());
             }
 
             $displayElement->setAttribute('status', 1);
             $displayElement->setAttribute('code', 'ADDED');
-            if ($display->licensed == 0)
-                $displayElement->setAttribute('message', 'Display is now Registered and awaiting Authorisation from an Administrator in the CMS');
-            else
+            if ($display->licensed == 0) {
+                $displayElement->setAttribute(
+                    'message',
+                    'Display is now Registered and awaiting Authorisation from an Administrator in the CMS'
+                );
+            } else {
                 $displayElement->setAttribute('message', 'Display is active and ready to start.');
+            }
         }
 
         // Send Notification if required
@@ -365,6 +412,50 @@ class Soap5 extends Soap4
         $display->clientVersion = $clientVersion;
         $display->clientCode = $clientCode;
         //$display->operatingSystem = $operatingSystem;
+
+        // Commercial Licence Check,  0 - Not licensed, 1 - licensed, 2 - trial licence, 3 - not applicable
+        // only sent by xmds v7
+        if (!empty($commercialLicenceString) && !in_array($display->clientType, ['windows', 'linux'])) {
+            $commercialLicenceString = strtolower($commercialLicenceString);
+            if ($commercialLicenceString === 'licensed' || $commercialLicenceString === 'full') {
+                $commercialLicence = 1;
+            } elseif ($commercialLicenceString === 'trial') {
+                $commercialLicence = 2;
+            } else {
+                $commercialLicence = 0;
+            }
+
+            $display->commercialLicence = $commercialLicence;
+            $node = $return->createElement('commercialLicence', $commercialLicenceString);
+            $displayElement->appendChild($node);
+        }
+
+        // commercial licence not applicable for Windows and Linux players.
+        if (in_array($display->clientType, ['windows', 'linux'])) {
+            $display->commercialLicence = 3;
+        }
+
+        if (!empty($display->syncGroupId)) {
+            $syncGroup = $this->syncGroupFactory->getById($display->syncGroupId);
+
+            if ($syncGroup->leadDisplayId === $display->displayId) {
+                $syncNodeValue = 'lead';
+            } else {
+                $leadDisplay = $this->syncGroupFactory->getLeadDisplay($syncGroup->leadDisplayId);
+                $syncNodeValue = $leadDisplay->lanIpAddress;
+            }
+
+            $syncNode = $return->createElement('syncGroup');
+            $value = $return->createTextNode($syncNodeValue ?? '');
+            $syncNode->appendChild($value);
+            $displayElement->appendChild($syncNode);
+
+            $syncPublisherPortNode = $return->createElement('syncPublisherPort');
+            $value = $return->createTextNode($syncGroup->syncPublisherPort ?? 9590);
+            $syncPublisherPortNode->appendChild($value);
+            $displayElement->appendChild($syncPublisherPortNode);
+        }
+
         $display->save(Display::$saveOptionsMinimum);
 
         // cache checks
@@ -397,8 +488,8 @@ class Soap5 extends Soap4
      * @throws NotFoundException
      * @throws \SoapFault
      */
-    function Schedule($serverKey, $hardwareKey)
+    public function Schedule($serverKey, $hardwareKey)
     {
         return $this->doSchedule($serverKey, $hardwareKey, ['dependentsAsNodes' => true, 'includeOverlays' => true]);
     }
-}
\ No newline at end of file
+}
