@@ -1,8 +1,8 @@
 <?php
 /*
- * Copyright (c) 2022 Xibo Signage Ltd
+ * Copyright (C) 2023 Xibo Signage Ltd
  *
- * Xibo - Digital Signage - http://www.xibo.org.uk
+ * Xibo - Digital Signage - https://xibosignage.com
  *
  * This file is part of Xibo.
  *
@@ -27,6 +27,10 @@ use Xibo\Middleware\SuperAdminAuth;
 
 defined('XIBO') or die('Sorry, you are not allowed to directly access this page.');
 
+if (file_exists(PROJECT_ROOT . '/lib/routes-cypress.php')) {
+    include(PROJECT_ROOT . '/lib/routes-cypress.php');
+}
+
 /**
  * @SWG\Swagger(
  *  basePath="/api",
@@ -37,7 +41,7 @@ defined('XIBO') or die('Sorry, you are not allowed to directly access this page.
  *  },
  *  @SWG\ExternalDocumentation(
  *      description="Manual",
- *      url="http://xibo.org.uk/manual"
+ *      url="https://xibosignage.com/manual"
  *  )
  * )
  *
@@ -46,8 +50,8 @@ defined('XIBO') or die('Sorry, you are not allowed to directly access this page.
  *  description="Xibo CMS API.
        Using HTTP formData requests.
        All PUT requests require Content-Type:application/x-www-form-urlencoded header.",
- *  version="3.1",
- *  termsOfService="http://xibo.org.uk/legal",
+ *  version="4.0",
+ *  termsOfService="https://xibosignage.com/legal",
  *  @SWG\License(
  *      name="AGPLv3 or later",
  *      url="http://www.gnu.org/licenses/"
@@ -88,6 +92,7 @@ $app->post('/tfa', ['\Xibo\Controller\Login' , 'twoFactorAuthValidate'])->setNam
  *  description="Schedule"
  * )
  */
+$app->get('/schedule', ['\Xibo\Controller\Schedule','grid'])->setName('schedule.search');
 $app->get('/schedule/data/events', ['\Xibo\Controller\Schedule','eventData'])->setName('schedule.calendar.data');
 $app->get('/schedule/{id}/events', ['\Xibo\Controller\Schedule','eventList'])->setName('schedule.events');
 
@@ -141,6 +146,7 @@ $app->put('/layout/lock/release/{id}', ['\Xibo\Controller\Layout', 'releaseLock'
 
 $app->group('', function (RouteCollectorProxy $group) {
     $group->post('/layout', ['\Xibo\Controller\Layout', 'add'])->setName('layout.add');
+    $group->post('/layout/fullscreen', ['\Xibo\Controller\Layout', 'createFullScreenLayout'])->setName('layout.add.full.screen.schedule');
     $group->post('/layout/copy/{id}', ['\Xibo\Controller\Layout','copy'])->setName('layout.copy');
 
     // TODO: why commented out? Layout Import
@@ -152,6 +158,8 @@ $app->group('', function (RouteCollectorProxy $group) {
 $app->group('', function (RouteCollectorProxy $group) {
     $group->put('/layout/{id}', ['\Xibo\Controller\Layout','edit'])->setName('layout.edit');
     $group->delete('/layout/{id}', ['\Xibo\Controller\Layout','delete'])->setName('layout.delete');
+    $group->put('/layout/applyTemplate/{id}', ['\Xibo\Controller\Layout', 'applyTemplate'])
+        ->setName('layout.apply.template');
     $group->put('/layout/background/{id}', ['\Xibo\Controller\Layout','editBackground'])->setName('layout.edit.background');
     $group->put('/layout/publish/{id}', ['\Xibo\Controller\Layout','publish'])->setName('layout.publish');
     $group->put('/layout/discard/{id}', ['\Xibo\Controller\Layout','discard'])->setName('layout.discard');
@@ -194,8 +202,6 @@ $app->group('/region', function (RouteCollectorProxy $group) {
  * )
  */
 $app->get('/playlist', ['\Xibo\Controller\Playlist','grid'])->setName('playlist.search');
-// Widgets Order
-$app->get('/playlist/widget', ['\Xibo\Controller\Playlist','widgetGrid'])->setName('playlist.widget.search');
 
 $app->post('/playlist', ['\Xibo\Controller\Playlist','add'])
     ->addMiddleware(new FeatureAuth($app->getContainer(), ['playlist.add']))
@@ -228,17 +234,20 @@ $app->group('', function (RouteCollectorProxy $group) {
  *  description="Widgets"
  * )
  */
+$app->get('/widget/{id}/edit/options', ['\Xibo\Controller\Widget', 'additionalWidgetEditOptions'])->setName('widget.edit.options');
 $app->group('/playlist/widget', function (RouteCollectorProxy $group) {
-    $group->post('/{type}/{id}', ['\Xibo\Controller\Module','addWidget'])->setName('module.widget.add');
-    $group->put('/{id}', ['\Xibo\Controller\Module','editWidget'])->setName('module.widget.edit');
-    $group->delete('/{id}', ['\Xibo\Controller\Module','deleteWidget'])->setName('module.widget.delete');
-    $group->put('/transition/{type}/{id}', ['\Xibo\Controller\Module','editWidgetTransition'])->setName('module.widget.transition.edit');
-    $group->put('/{id}/audio', ['\Xibo\Controller\Module','widgetAudio'])->setName('module.widget.audio');
-    $group->delete('/{id}/audio', ['\Xibo\Controller\Module','widgetAudioDelete']);
-    $group->put('/{id}/expiry', ['\Xibo\Controller\Module','widgetExpiry'])->setName('module.widget.expiry');
+    $group->post('/{type}/{id}', ['\Xibo\Controller\Widget','addWidget'])->setName('module.widget.add');
+    $group->put('/{id}', ['\Xibo\Controller\Widget','editWidget'])->setName('module.widget.edit');
+    $group->delete('/{id}', ['\Xibo\Controller\Widget','deleteWidget'])->setName('module.widget.delete');
+    $group->put('/transition/{type}/{id}', ['\Xibo\Controller\Widget','editWidgetTransition'])->setName('module.widget.transition.edit');
+    $group->put('/{id}/audio', ['\Xibo\Controller\Widget','widgetAudio'])->setName('module.widget.audio');
+    $group->delete('/{id}/audio', ['\Xibo\Controller\Widget','widgetAudioDelete']);
+    $group->put('/{id}/expiry', ['\Xibo\Controller\Widget','widgetExpiry'])->setName('module.widget.expiry');
+    $group->put('/{id}/elements', ['\Xibo\Controller\Widget','saveElements'])->setName('module.widget.elements');
+    $group->get('/{id}/dataType', ['\Xibo\Controller\Widget','getDataType'])->setName('module.widget.dataType');
 
     // Drawer widgets Region
-    $group->put('/{id}/target', ['\Xibo\Controller\Module','widgetSetRegion'])->setName('module.widget.set.region');
+    $group->put('/{id}/target', ['\Xibo\Controller\Widget','widgetSetRegion'])->setName('module.widget.set.region');
 })
     ->addMiddleware(new FeatureAuth($app->getContainer(), ['layout.modify', 'playlist.modify']))
     ->addMiddleware(new LayoutLock($app));
@@ -260,7 +269,10 @@ $app->group('', function (RouteCollectorProxy $group) {
     $group->delete('/campaign/{id}', ['\Xibo\Controller\Campaign','delete'])->setName('campaign.delete');
     $group->post('/campaign/{id}/copy', ['\Xibo\Controller\Campaign','copy'])->setName('campaign.copy');
     $group->put('/campaign/{id}/selectfolder', ['\Xibo\Controller\Campaign','selectFolder'])->setName('campaign.selectfolder');
-    $group->post('/campaign/layout/assign/{id}', ['\Xibo\Controller\Campaign','assignLayout'])->setName('campaign.assign.layout');
+    $group->post('/campaign/layout/assign/{id}', ['\Xibo\Controller\Campaign','assignLayout'])
+        ->setName('campaign.assign.layout');
+    $group->delete('/campaign/layout/remove/{id}', ['\Xibo\Controller\Campaign','removeLayout'])
+        ->setName('campaign.remove.layout');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['campaign.modify']));
 
 /**
@@ -308,11 +320,18 @@ $app->group('', function (RouteCollectorProxy $group) {
     $group->get('/library/usage/layouts/{id}', ['\Xibo\Controller\Library','usageLayouts'])->setName('library.usage.layouts');
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['library.view']));
 
-$app->get('/library/download/{id}[/{type}]', ['\Xibo\Controller\Library','download'])->setName('library.download');
+$app->get('/library/download/{id}', [
+    '\Xibo\Controller\Library', 'download'
+])->setName('library.download');
+$app->get('/library/thumbnail/{id}', [
+    '\Xibo\Controller\Library', 'thumbnail'
+])->setName('library.thumbnail');
+
+$app->post('/library', ['\Xibo\Controller\Library','add'])->setName('library.add')
+    ->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['library.add', 'dashboard.playlist']));
 
 $app->group('', function (RouteCollectorProxy $group) {
     //$group->map(['HEAD'],'/library', ['\Xibo\Controller\Library','  addgroup
-    $group->post('/library', ['\Xibo\Controller\Library','add'])->setName('library.add');
     $group->post('/library/uploadUrl', ['\Xibo\Controller\Library','uploadFromUrl'])->setName('library.uploadFromUrl');
     $group->post('/library/thumbnail', ['\Xibo\Controller\Library','addThumbnail'])->setName('library.thumbnail.add');
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['library.add']));
@@ -373,6 +392,7 @@ $app->group('', function (RouteCollectorProxy $group) {
  *  description="Display Groups"
  * )
  */
+$app->get('/displayvenue', ['\Xibo\Controller\Display','displayVenue'])->setName('display.venue.search');
 $app->get('/displaygroup', ['\Xibo\Controller\DisplayGroup','grid'])->setName('displayGroup.search');
 
 $app->post('/displaygroup', ['\Xibo\Controller\DisplayGroup','add'])
@@ -473,10 +493,14 @@ $app->group('', function (RouteCollectorProxy $group) {
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['dataset.data']));
 
 /**
- *  Folders
+ * Folders
+ * @SWG\Tag(
+ *  name="folder",
+ *  description="Folders"
+ * )
  */
+$app->get('/folders[/{folderId}]', ['\Xibo\Controller\Folder', 'grid'])->setName('folders.search');
 $app->group('', function (RouteCollectorProxy $group) {
-    $group->get('/folders', ['\Xibo\Controller\Folder', 'grid'])->setName('folders.search');
     $group->get('/folders/contextButtons/{folderId}', ['\Xibo\Controller\Folder', 'getContextMenuButtons'])->setName('folders.context.buttons');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['folder.view']));
 
@@ -487,6 +511,7 @@ $app->post('/folders', ['\Xibo\Controller\Folder', 'add'])
 $app->group('', function (RouteCollectorProxy $group) {
     $group->put('/folders/{folderId}', ['\Xibo\Controller\Folder', 'edit'])->setName('folders.edit');
     $group->delete('/folders/{folderId}', ['\Xibo\Controller\Folder', 'delete'])->setName('folders.delete');
+    $group->put('/folders/{folderId}/move', ['\Xibo\Controller\Folder', 'move'])->setName('folders.move');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['folder.modify']));
 
 /**
@@ -506,13 +531,8 @@ $app->get('/stats/export', ['\Xibo\Controller\Stats','export'])
     ->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['proof-of-play']))
     ->setName('stats.export');
 
-/**
- * Log
- * @SWG\Tag(
- *  name="log",
- *  description="Logs"
- * )
- */
+// Log (no APIs)
+// -------------
 $app->group('', function (RouteCollectorProxy $group) {
     $group->get('/log', ['\Xibo\Controller\Logging', 'grid'])->setName('log.search');
     $group->delete('/log', ['\Xibo\Controller\Logging', 'truncate'])->setName('log.truncate');
@@ -551,6 +571,9 @@ $app->group('', function (RouteCollectorProxy $group) {
     $group->put('/user/{id}', ['\Xibo\Controller\User','edit'])->setName('user.edit');
     $group->delete('/user/{id}', ['\Xibo\Controller\User','delete'])->setName('user.delete');
     $group->post('/user/{id}/usergroup/assign', ['\Xibo\Controller\User','assignUserGroup'])->setName('user.assign.userGroup');
+    $group->post('/user/{id}/setHomeFolder', ['\Xibo\Controller\User', 'setHomeFolder'])
+        ->addMiddleware(new FeatureAuth($group->getContainer(), ['folder.userHome']))
+        ->setName('user.homeFolder');
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['users.modify']));
 
 /**
@@ -582,7 +605,6 @@ $app->get('/application', ['\Xibo\Controller\Applications','grid'])->setName('ap
 
 $app->group('', function (RouteCollectorProxy $group) {
     $group->post('/application', ['\Xibo\Controller\Applications','add'])->setName('application.add');
-    $group->post('/application/dooh', ['\Xibo\Controller\Applications','addDooh'])->setName('application.addDooh');
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['application.add']));
 
 /**
@@ -593,9 +615,23 @@ $app->group('', function (RouteCollectorProxy $group) {
  * )
  */
 $app->get('/module', ['\Xibo\Controller\Module','grid'])->setName('module.search');
+$app->get('/module/templates/{dataType}', [
+    '\Xibo\Controller\Module', 'templateGrid'
+])->setName('module.template.search');
+
+$app->get('/module/asset/{assetId}', [
+    '\Xibo\Controller\Module',
+    'assetDownload',
+])->setName('module.asset.download');
+
+// Properties
+$app->get('/module/properties/{id}', ['\Xibo\Controller\Module','getProperties'])
+    ->setName('module.get.properties');
+$app->get('/module/template/{dataType}/properties/{id}', ['\Xibo\Controller\Module','getTemplateProperties'])
+    ->setName('module.template.get.properties');
+
 $app->group('', function (RouteCollectorProxy $group) {
     $group->put('/module/settings/{id}', ['\Xibo\Controller\Module','settings'])->setName('module.settings');
-    $group->put('/module/verify', ['\Xibo\Controller\Module','verify'])->setName('module.verify');
     $group->put('/module/clear-cache/{id}', ['\Xibo\Controller\Module','clearCache'])->setName('module.clear.cache');
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['module.view']));
 
@@ -615,16 +651,6 @@ $app->group('', function (RouteCollectorProxy $group) {
     $group->delete('/sessions/logout/{id}', ['\Xibo\Controller\Sessions','logout'])->setName('sessions.confirm.logout');
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['session.view']));
 
-//
-// Help
-//
-$app->get('/help', ['\Xibo\Controller\Help','grid'])->setName('help.search');
-$app->group('', function (RouteCollectorProxy $group) {
-    $group->post('/help/add', ['\Xibo\Controller\Help','add'])->setName('help.add');
-    $group->put('/help/edit/{id}', ['\Xibo\Controller\Help','edit'])->setName('help.edit');
-    $group->delete('/help/delete/{id}', ['\Xibo\Controller\Help','delete'])->setName('help.delete');
-})->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['help.view']));
-
 //
 // Settings
 //
@@ -688,13 +714,8 @@ $app->group('', function (RouteCollectorProxy $group) {
     $group->delete('/daypart/{id}', ['\Xibo\Controller\DayPart','delete'])->setName('daypart.delete');
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['daypart.modify']));
 
-/**
- * Tasks
- * @SWG\Tag(
- *  name="task",
- *  description="Tasks"
- * )
- */
+// Tasks (no APIs)
+// ----
 $app->group('', function (RouteCollectorProxy $group) {
     $group->get('/task', ['\Xibo\Controller\Task', 'grid'])->setName('task.search');
     $group->post('/task', ['\Xibo\Controller\Task', 'add'])->setName('task.add');
@@ -703,14 +724,8 @@ $app->group('', function (RouteCollectorProxy $group) {
     $group->post('/task/{id}/run', ['\Xibo\Controller\Task', 'runNow'])->setName('task.runNow');
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['task.view']));
 
-/**
- * Report schedule
- * @SWG\Tag(
- *  name="report",
- *  description="Report schedule"
- * )
- */
-
+// Report schedule (no APIs)
+// -------------------------
 $app->get('/report/reportschedule', ['\Xibo\Controller\ScheduleReport','reportScheduleGrid'])->setName('reportschedule.search');
 $app->group('', function (RouteCollectorProxy $group) {
     $group->post('/report/reportschedule', ['\Xibo\Controller\ScheduleReport','reportScheduleAdd'])->setName('reportschedule.add');
@@ -739,6 +754,8 @@ $app->delete('/report/savedreport/{id}', ['\Xibo\Controller\SavedReport','savedR
 $app->get('/playersoftware', ['\Xibo\Controller\PlayerSoftware','grid'])->setName('playersoftware.search');
 
 $app->group('', function (RouteCollectorProxy $group) {
+    $group->get('/playersoftware/download/{id}', ['\Xibo\Controller\PlayerSoftware', 'download'])->setName('playersoftware.download');
+    $group->post('/playersoftware', ['\Xibo\Controller\PlayerSoftware','add'])->setName('playersoftware.add');
     $group->put('/playersoftware/{id}', ['\Xibo\Controller\PlayerSoftware','edit'])->setName('playersoftware.edit');
     $group->delete('/playersoftware/{id}', ['\Xibo\Controller\PlayerSoftware','delete'])->setName('playersoftware.delete');
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['playersoftware.view']));
@@ -763,18 +780,14 @@ $app->group('', function (RouteCollectorProxy $group) {
     $group->delete('/tag/{id}', ['\Xibo\Controller\Tag','delete'])->setName('tag.delete');
     $group->get('/tag/name', ['\Xibo\Controller\Tag','loadTagOptions'])->setName('tag.getByName');
     $group->put('/tag/{type}/multi', ['\Xibo\Controller\Tag','editMultiple'])->setName('tag.editMultiple');
+    $group->get('/tag/usage/{id}', ['\Xibo\Controller\Tag', 'usage'])->setName('tag.usage');
 })->addMiddleware(new \Xibo\Middleware\FeatureAuth($app->getContainer(), ['tag.view']));
 
-/**
- * Actions
- * @SWG\Tag(
- *  name="actions",
- *  description="Actions"
- * )
- */
+// Actions (no APIs)
+// -----------------
 $app->get('/action', ['\Xibo\Controller\Action', 'grid'])->setName('action.search');
 $app->group('', function (RouteCollectorProxy $group) {
-    $group->post('/action/{source}/{id}', ['\Xibo\Controller\Action', 'add'])->setName('action.add');
+    $group->post('/action', ['\Xibo\Controller\Action', 'add'])->setName('action.add');
     $group->put('/action/{id}', ['\Xibo\Controller\Action', 'edit'])->setName('action.edit');
     $group->delete('/action/{id}', ['\Xibo\Controller\Action', 'delete'])->setName('action.delete');
 })->addMiddleware(new FeatureAuth($app->getContainer(), ['layout.modify', 'playlist.modify']));
@@ -806,3 +819,25 @@ $app->group('', function (RouteCollectorProxy $group) {
     $group->delete('/menuboard/{id}/product', ['\Xibo\Controller\MenuBoardProduct', 'delete'])->setName('menuBoard.product.delete');
 })
     ->addMiddleware(new FeatureAuth($app->getContainer(), ['menuBoard.modify']));
+
+$app->get('/fonts', ['\Xibo\Controller\Font', 'grid'])->setName('font.search');
+$app->group('', function (RouteCollectorProxy $group) {
+    $group->get('/fonts/details/{id}', ['\Xibo\Controller\Font', 'getFontLibDetails'])->setName('font.details');
+    $group->get('/fonts/download/{id}', ['\Xibo\Controller\Font', 'download'])->setName('font.download');
+    $group->post('/fonts', ['\Xibo\Controller\Font','add'])->setName('font.add');
+    $group->delete('/fonts/{id}/delete', ['\Xibo\Controller\Font','delete'])->setName('font.delete');
+})->addMiddleware(new FeatureAuth($app->getContainer(), ['font.view']));
+
+$app->get('/syncgroups', ['\Xibo\Controller\SyncGroup', 'grid'])->setName('syncgroup.search');
+$app->group('', function (RouteCollectorProxy $group) {
+    $group->get('/syncgroup/{id}/displays', ['\Xibo\Controller\SyncGroup', 'fetchDisplays'])
+        ->setName('syncgroup.fetch.displays');
+    $group->post('/syncgroup/add', ['\Xibo\Controller\SyncGroup', 'add'])->setName('syncgroup.add');
+    $group->post('/syncgroup/{id}/members', ['\Xibo\Controller\SyncGroup', 'members'])->setName('syncgroup.members');
+    $group->put('/syncgroup/{id}/edit', ['\Xibo\Controller\SyncGroup', 'edit'])->setName('syncgroup.edit');
+    $group->delete('/syncgroup/{id}/delete', ['\Xibo\Controller\SyncGroup', 'delete'])->setName('syncgroup.delete');
+})->addMiddleware(new FeatureAuth($app->getContainer(), ['display.syncView']));
+
+$app->group('', function (RouteCollectorProxy $group) {
+    $group->post('/schedule/sync/add', ['\Xibo\Controller\Schedule', 'syncAdd'])->setName('schedule.add.sync');
+})->addMiddleware(new FeatureAuth($app->getContainer(), ['schedule.sync']));
